-- SRB2 PSO
-- dumb mockup by Lat'
-- 22/01/2023

rawset(_G, "PSO", {})

PSO = {
	
	--debug = true,
	info = {},
	lib = {},			-- library functions
	frameFuncs = {},	-- functions called with no argument in a thinkframe
	playerFuncs = {},	-- functions called with a player argument in a thinkframe
	hudFuncs = {},		-- functions called with drawer, player & camera args in hud rendering
	mChangeFuncs = {},	-- functions called with no argument when the map is changed. Used to reset variables.
}

hud.disable("rings")
hud.disable("score")
hud.disable("time")
hud.disable("lives")


PSO.info.playerPrefFile = "pso_playerprefs.cfg"

PSO.info.serverDataFile = "client/srb2i/pso_serverdata.sav2"

PSO.info.serverKeyFile = "client/srb2i/pso_serverkey.sav2"
PSO.info.clientKeyFile = "client/srb2i/pso_clientkey.sav2"
PSO.info.serverSaves = "client/srb2i/players/"

PSO.info.soloData = "client/srb2i/pso_localdata.sav2"
PSO.info.soloSave = "client/srb2i/pso_localsave.sav2"


PSO.info.INVENTORYSIZE = 60
PSO.info.STORAGESIZE = 200
PSO.info.MAXPALSIZE = 6
PSO.info.LEVELCAP = 50
PSO.info.MAXGRIND = 10

-- this is a little hack,
-- we consider anything below that y pos in maps is in the lobby.
PSO.info.LOBBY_YPOS = -23000
PSO.info.LOBBY_MAP = 1

PSO.info.ENCORETAG = 32000

PSO.info.SHIFTA = 15
PSO.info.DEBAND = 25
PSO.info.JELLEN = -15
PSO.info.ZALLURE = -10

-- for symbol arts
PSO.info.SAFile = "client/srb2i/symbolarts/pso_symbolarts.txt"
PSO.info.MAXSYMBOLARTS = 8
PSO.info.SA_WIDTH = 90	
PSO.info.SA_HEIGHT = 44

PSO.lib.dprint = function(s)
	if PSO.debug
		print(s)
	end
end


PSO.info.maxpalette = 6	-- max: 6 slots on each palette

local difficulty = {
	"DIFF_NORMAL",
	"DIFF_HARD",
	"DIFF_VERYHARD",
	"DIFF_SUPERHARD",
	"DIFF_EXTREMELYHARD",
	"DIFF_ULTRAHARD",
}

PSO.info.diff_shortnames = {
	"N",
	"H",
	"VH",
	"SH",
	"XH",
	"UH"
}

local types = {
	
	"TYPE_HUNTER",
	"TYPE_RANGER",
	"TYPE_FORCE",
}

local typenames = {
	"Hunter",
	"Ranger",
	"Force",
}	


local typedescs = {
	"A warrior that wields Melee weapons and excels at close range combat and crowd control with their high resilience.",
	"An adept of ranged weapons that thrives in the back line of fights and excels at one-on-one encounters.",
	"A master of Technique casting, allowing them to adapt to many situations so long as they use their PP properly."
	--[TYPE_VANGUARD] = "A master of none proficient with dual-wielding. Their high adaptability allows them to use otherwise unconventional combinations."
}

local weapons = {
	"WPT_SWORD",
	"WPT_DOUBLESABER",
	"WPT_RIFLE",
	"WPT_SHOTGUN",
	"WPT_ROD",
	"WPT_SABER",
	"WPT_HANDGUN",
	"WPT_WAND",
}

local weaponnames = {
	"Sword",
	"Double Saber",
	"Rifle",
	"Shotgun",
	"Rod",
	"Saber",
	"Handgun",
	"Wand",
}

-- defined in an order convenient for sorting
local itemtypes = {
	"IT_WEAPON",		-- can be equipped
	"IT_ARMOR",			-- can be equipped
	"IT_UNIT",			-- can be equipped on armor	
	"IT_CONSUMABLE",	-- can be used
	"IT_MATERIAL",		-- can only be sold / exchanged?
}

-- used to know which attack stat to use
local damagetypes = {
	"DMT_STRIKE",
	"DMT_RANGED",
	"DMT_TECHNIQUE",
}

-- used to know what hitboxes can inflict:
local reactiontypes = {
	"RT_NONE",
	"RT_STAGGER",
	"RT_KNOCKBACK",
	"RT_BLOWDOWN",
}

local elements = {
	"ELEM_FIRE",
	"ELEM_ICE",
	"ELEM_ELEC",
	"ELEM_EARTH",
	"ELEM_LIGHT",
	"ELEM_DARK",
}

local statuses = {
	"COND_BURN",
	"COND_FREEZE",
	"COND_SHOCK",
	"COND_POISON",
	"COND_CONFUSE",
	"COND_CURSE",
}

local rank = {
	"RANK_S",
	"RANK_A",
	"RANK_B",
	"RANK_C",
}

for i = 1, #rank
	rawset(_G, rank[i], i)
end

for i = 1, #statuses
	rawset(_G, statuses[i], i)
end

-- shifta/deband/jellen/zallure go into pso.attackboost and pso.defenceboost as positive or negative


for i = 1, #difficulty
	rawset(_G, difficulty[i], i)
end


PSO.info.typenames = {}
PSO.info.typedescs = {}
for i = 1, #types
	rawset(_G, types[i], i)
	PSO.info.typenames[i] = typenames[i]
	PSO.info.typedescs[i] = typedescs[i]
end

PSO.info.weaponnums = {}
PSO.info.weaponnames = {}
for i = 1, #weapons
	PSO.info.weaponnums[2^(i-1)] = i
	PSO.info.weaponnames[2^(i-1)] = weaponnames[i]
	rawset(_G, weapons[i], 2^(i-1))
end

for i = 1, #itemtypes
	rawset(_G, itemtypes[i], i)
end

for i = 1, #damagetypes 
	rawset(_G, damagetypes[i], i)
end

for i = 1, #reactiontypes
	rawset(_G, reactiontypes[i], i)
end

for i = 1, #elements
	rawset(_G, elements[i], i)
end	

PSO.info.elementColours = {
	[ELEM_FIRE] = SKINCOLOR_FLAME,
	[ELEM_ICE] = SKINCOLOR_SKY,
	[ELEM_ELEC] = SKINCOLOR_YELLOW,
	[ELEM_EARTH] = SKINCOLOR_ORANGE,
	[ELEM_LIGHT] = SKINCOLOR_TAN,
	[ELEM_DARK] = SKINCOLOR_NEON,
}

PSO.lib.registerFrameFunc = function(fn)
	assert(type(fn) == "function", "Attempting to register non function value")
	table.insert(PSO.frameFuncs, fn)
end

PSO.lib.registerPlayerFunc = function(fn)
	assert(type(fn) == "function", "Attempting to register non function value")
	table.insert(PSO.playerFuncs, fn)
end

PSO.lib.registerHudFunc = function(fn)
	assert(type(fn) == "function", "Attempting to register non function value")
	table.insert(PSO.hudFuncs, fn)
end

PSO.lib.registerMChangeFunc = function(fn)
	assert(type(fn) == "function", "Attempting to register non function value")
	table.insert(PSO.mChangeFuncs, fn)
end

addHook("ThinkFrame", do
	
	-- run global functions:
	if gamestate ~= GS_LEVEL return end	-- what the fuck.
	if not server or not server.valid return end	-- also what the fuck.
	
	for i = 1, #PSO.frameFuncs
		local fn = PSO.frameFuncs[i]
		local run, res = pcall(fn)
		if not run
			print("\x82".."WARNING".."\x80"..": PSO.frameFuncs #"..i..": "..res)
		end
	end
	
	-- run player functions:
	for p in players.iterate do
		for i = 1, #PSO.playerFuncs
			local fn = PSO.playerFuncs[i]
			local run, res = pcall(fn, p)
			if not run
				print("\x82".."WARNING".."\x80"..": PSO.playerFuncs #"..i..": "..res)
			end
		end
	end
end)

hud.add(function(v, p, c)
	
	for i = 1, #PSO.hudFuncs
		local fn = PSO.hudFuncs[i]
		local run, res = pcall(fn, v, p, c)	--fn(v, p, c)
		if not run
			print("\x82".."WARNING".."\x80"..": PSO.hudFuncs: #"..i..": "..res)
		end
	end
end)




-- on mobjspawn, initialize all the values to 0
-- mapthingspawn is called AFTERWARDS.
addHook("MobjSpawn", function(mo)
	mo.mtangle = 0
	mo.extrainfo = 0
	mo.options = 0
end)

-- mapthing spawn handling
-- moves options and extrainfo onto the mobj
-- also adds 'mtangle' as the spawnpoint's angle
-- (this lets us have values above 359)

addHook("MapThingSpawn", function(mo, mt)
	mo.mtangle = mt.angle
	mo.extrainfo = mt.extrainfo
	mo.options = mt.options
end)

-- debug cheats
COM_AddCommand("psod_debugcheats", function(p)
	
	if not server.psocheats
		print("Cheats have been enabled, up to you what you do, saving will remain enabled.")
	end
	server.psocheats = true
end, COM_ADMIN)

