-- basic linear interpolation. Percent must be between 0 and FRACUNIT
PSO.lib.lerp = function(start, finish, percent)
	return start + FixedMul(finish-start, FRACUNIT-percent)
end

-- adds \n characters to a string to word wrap it properly.
PSO.lib.wordWrap = function(v, str, width, flags, display)
	
	local w = 0
	local nstr = {}
	local fstr = ""
	
	local lastspace
	local lastw
	
	for i = 1, #str
		local s = str:sub(i, i)
		w = $ + v.stringWidth(s, flags, display)
		
		if s == " "
			lastspace = i
			lastw = w
		end
		
		nstr[#nstr+1] = s
		
		if w > width
			
			if lastspace
				nstr[lastspace] = "\n"
				w = $ - lastw
			else
				nstr[#nstr+1] = "\n"	-- yikes.
				w = 0
			end
			
			lastspace = nil
			lastw = nil
		end
	end
	
	for i = 1, #nstr
		fstr = $..nstr[i]
	end
	return fstr
end

-- for large numbers, so that 3000000 for instance is converted into a much more readable 3,000,000
PSO.lib.numCommas = function(n)
  return tostring(n):reverse():gsub("(%d%d%d)","%1,"):gsub(",(%-?)$","%1"):reverse()
end

-- yes good idea lets push uint32 to int32, itll be ok they said
-- NO IT'S NOT FUCKING OK YOU ABSOLUTE DUMBASS
PSO.lib.angToInt = function(a)
    if not a return 0 end
    a = $/ANG1
    if a < 0
        a = $+360
    end
    return a
end

PSO.lib.countPlayers = function()
	local count = 0
	for p in players.iterate
		if p.mo and p.mo.valid
			count = $+1
		end
	end
	return count
end

-- spline garbage, returns a position along a path of points.
-- This takes a fixed_t between 0 and FRACUNIT
-- ...And gives us the point on a spline.
-- Basically, fixed math hell
PSO.lib.pointOnSpline = function(path, frac)
	
	local p0, p1, p2, p3
	p1 = frac/FRACUNIT +1	-- +1 because lua is not 0-indexed
	
	-- out of bounds check
	if p1 == #path
		p2 = p1
	else	
		p2 = p1 + 1
	end	
	
	-- another out of bounds check
	if p2 == #path
		p3 = p2
	else	
		p3 = p2 + 1
	end

	-- this one is safe
	p0 = max(1, p1 - 1)
	
	frac = $ - FixedFloor($)
	
	-- cache frac^2 and frac^3
	local fsq = FixedMul(frac, frac)
	local fcb = FixedMul(fsq, frac)
	
	-- equations:
	local q1 = -fcb + FixedMul(2<<FRACBITS, fsq) - frac
	local q2 = FixedMul(3<<FRACBITS, fcb) - FixedMul(5<<FRACBITS, fsq) + 2<<FRACBITS
	local q3 = -FixedMul(3<<FRACBITS, fcb) + FixedMul(4<<FRACBITS, fsq) + frac
	local q4 = fcb - fsq
	
	-- now get the points from the dark magic above:
	--print(p0, p1, p2, p3)
	
	local tx = FixedMul(FRACUNIT>>1, FixedMul(path[p0].x, q1) + FixedMul(path[p1].x, q2) + FixedMul(path[p2].x, q3) + FixedMul(path[p3].x, q4)) 
	local ty = FixedMul(FRACUNIT>>1, FixedMul(path[p0].y, q1) + FixedMul(path[p1].y, q2) + FixedMul(path[p2].y, q3) + FixedMul(path[p3].y, q4))
	local tz = FixedMul(FRACUNIT>>1, FixedMul(path[p0].z, q1) + FixedMul(path[p1].z, q2) + FixedMul(path[p2].z, q3) + FixedMul(path[p3].z, q4)) 
	
	return {x = tx, y = ty, z = tz}
end

-- makes a distinct copy of a table and any sub table it may have
PSO.lib.copyTable = function(t)
	
	if type(t) ~= "table"
		return t
	end
	
	local new = {}
	for k,v in pairs(t)
		if type(v) == "table"
			v = PSO.lib.copyTable(v)
		end
		
		new[k] = v
	end
	return new
end

-- These are used for player save serialization
-- compacts a sparse table
PSO.lib.compactPow2Table = function(t)
	local new = {}
	
	for i = 0, 31
		local pow = 2^i
		if t[pow] ~= nil
			new[i+1] = t[pow]
		end
	end	
	
	return new
end

PSO.lib.sparsePow2Table = function(t)
	local new = {}
	
	for i = 1, #t
		new[2^(i-1)] = t[i]
	end
	return new
end

-- converts int keys of a sparse table into strings so that it may be serialized into json
PSO.lib.sparseStrTable = function(t)
	local new = {}
	
	for k,v in pairs(t)
		if type(v) == "table"
			v = PSO.lib.sparseStrTable(v)
		end
		
		new[tostring(k)] = v
	end
	return new
end

PSO.lib.sparseIntTable = function(t)
	local new = {}
	
	for k,v in pairs(t)
		if type(v) == "table"
			v = PSO.lib.sparseIntTable(v)
		end
		
		new[tonumber(k)] = v
	end
	return new
end