-- Blue Crawla
/*
	Drives towards players.
	Once they get close enough, charge for 1/4th of a second and lunge forward at higher speed, ending in lag
	
	HARD MODE: Throws projectiles if you stay out of range
*/

PSO.info.enemies[MT_BLUECRAWLA] = {
		
		name = "Blue Crawla",
		sprite = SPR_POSS,
		frame = A,
		painframe = A,
		
		bhp = 150,
		satk = 30,
		ratk = 20,
		tatk = 20,
		def = 25,
		exp = 5,
		weight = 1,
		
		
		drops = {
			
			-- Lv 1-10
			{100, 1, 9, 1, 15},				-- meseta (100% mult) 2/5 (Lv1-10)
			{"sword", 1, 9, 1, 100},		-- sword: 	1/100 (Lv1-10)
			{"rifle", 1, 9, 1, 100},		-- rifle: 	1/100 (Lv1-10)
			{"rod", 1, 9, 1, 100},			-- rod: 	1/100 (Lv1-10)
			{"lockgun", 1, 9, 1, 100},
			
			{"monomate", 1, 9, 1, 25},	-- monomate: 1/5 (Lv1-10)
			
			-- Lv10-19
			
			{200, 10, 19, 1, 15},
			{"gigush", 10, 19, 1, 100},
			{"beam", 10, 19, 1, 100},
			{"cane", 10, 19, 1, 100},
			{"monomate", 10, 19, 1, 25},
			{"autogun", 10, 19, 1, 100},

			-- Lv20-29
			{300, 20, 29, 1, 15},
			{"breaker", 20, 29, 1, 150},
			{"doublegladius", 20, 29, 1, 150},
			{"laser", 20, 29, 1, 150},
			{"canon", 20, 29, 1, 150},
			{"ulrod", 20, 29, 1, 150},
			{"peacekeeper", 20, 29, 1, 100},
			{"dimate", 20, 29, 1, 25},

			-- Lv30-50
			{500, 30, 50, 1, 15},
			{"peacekeeper", 30, 50, 1, 50},
			{"alva peacekeeper", 30, 50, 1, 100},
			{"vita peacekeeper", 30, 50, 1, 150},
			{"poss shooter", 30, 50, 1, 250},
			{"rostig frame", 30, 50, 1, 60},
			{"fehler frame", 30, 50, 1, 200},
			{"dimate", 30, 50, 1, 35},
			
		},
		
		stagger_duration = TICRATE/2,
		
		weakness = ELEM_FIRE,
		
		reset =	function(mo)
			mo.actiontimer = 0	-- reset action if we get staggered etc.
			mo.outranged = 0
		end,
		
		thinker = function(mo)
			
			local diff = PSO.lib.questDifficulty()
			
			local attack_potency = 250
			local move_speed = mo.scale*3/2 * (diff >= DIFF_HARD and 2 or 1)
			local attack_range = mo.scale*80
			local attack_charge = TICRATE/2
			local attack_delay = TICRATE
			local attack_speed = mo.scale*14
			local anim_speed = 3 
			
			local shoot_wait = TICRATE*5
			local shoot_charge = TICRATE*3/4
			local shoot_knockback = TICRATE*3/4
			
			PSO.lib.enemyManageAggro(mo)
			
			mo.sprite = SPR_POSS
			mo.frame = A
			
			if mo.target and mo.target.valid
				
				if not mo.actiontimer
				and not mo.shooting
					A_FaceTarget(mo)
					P_InstaThrust(mo, mo.angle, move_speed)
					
					mo.frame = (leveltime%(anim_speed*6)) /anim_speed
					
					--print(mo)
					if R_PointToDist2(mo.x, mo.y, mo.target.x, mo.target.y) < attack_range
						mo.actiontimer = 1
						mo.outranged = 0
						S_StartSound(mo, sfx_spndsh)
					elseif diff >= DIFF_HARD
						mo.outranged = $ and $+1 or P_RandomRange(1, TICRATE)
						if mo.outranged > shoot_wait
							mo.shooting = 1
							mo.outranged = 0
						end
					end
				elseif mo.shooting
					
					mo.shooting = $+1
					
					if mo.shooting < shoot_charge
						A_FaceTarget(mo)
						P_InstaThrust(mo, 0, 0)
					elseif mo.shooting == shoot_charge
						
						P_InstaThrust(mo, mo.angle, -FRACUNIT*16)
						S_StartSound(mo, sfx_trfire)
						local msl = PSO.lib.spawnMissile(mo, MT_JETTBULLET, mo.angle, 30*FRACUNIT, attack_potency, DMT_RANGED, 0, 0, mo.height/2, RT_NONE)
						PSO.lib.aimMissile(msl, mo.target)	
						
						for i = 1, 8
							local d = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SPINDUST)
							d.angle = mo.angle + P_RandomRange(-30, 30)*ANG1
							P_InstaThrust(d, d.angle, P_RandomRange(2, 16)*FRACUNIT)
						end
						
					elseif mo.shooting == shoot_charge+shoot_knockback
						mo.shooting = nil
					end
					
				elseif mo.actiontimer
					mo.actiontimer = $+1
					
					if mo.actiontimer == attack_charge
						P_InstaThrust(mo, mo.angle, attack_speed)
						S_StartSound(mo, sfx_zoom)
						
						local hx = mo.x + 64*cos(mo.angle)
						local hy = mo.y + 64*sin(mo.angle)
						
						local hb = PSO.lib.spawnHurtBox(hx, hy, mo.z, 80<<FRACBITS, 40<<FRACBITS, mo, attack_potency, DMT_STRIKE, 3, RT_STAGGER)
						hb.angle = mo.angle
						P_InstaThrust(hb, hb.angle, attack_speed)
						
					elseif mo.actiontimer > attack_charge
					and mo.actiontimer < attack_delay
						mo.momx = $*98/100
						mo.momy = $*98/100
					
					elseif mo.actiontimer >= attack_delay
						mo.actiontimer = 0
					end
				end
			end
		end,	
}