-- Dark Falz
/*
	Depths Area boss, Phase 2
*/

local DF_NEUTRAL = 0
local DF_BURST = 1
local DF_BEAM = 2
local DF_JUMP = 3
local DF_DOUBLECROSS = 4	-- the ctf map

local DF_SPAWNWHIP = 9

local DF_WHIPSMASH = 10
local DF_WHIPSLASH = 11


freeslot("MT_PSO_DARKFALZ", "MT_PSO_DARKBALZ", "S_PSO_DARKFALZ_DEATH", "S_PSO_DARKBALZ", "S_PSO_DARKBALZ_DIE", "MT_PSO_DARKFALZ_EXPLOSION")
mobjinfo[MT_PSO_DARKFALZ] = {
	doomednum = 2554,
	spawnhealth = 1000,
	deathstate = S_PSO_DARKFALZ_DEATH,
	spawnstate = S_THOK,
	height = 240<<FRACBITS,
	radius = 80<<FRACBITS,
	flags = MF_ENEMY|MF_SOLID|MF_SHOOTABLE,
}

function A_DarkFalzDeathLoop(mo)
	mo.deathtimer = $ and $+1 or 1
	mo.flags = $|MF_NOBLOCKMAP|MF_NOCLIPTHING
	
	if mo.deathtimer == 1
		S_FadeOutStopMusic(1000)
	end
	
	if mo.deathtimer < TICRATE*5
		
		local boom = P_SpawnMobj(mo.x + P_RandomRange(-128, 128)*FRACUNIT, mo.y + P_RandomRange(-128, 128)*FRACUNIT, mo.z + P_RandomRange(0, 256)*FRACUNIT, MT_SONIC3KBOSSEXPLODE)
		boom.colorized = true
		boom.color = SKINCOLOR_EMERALD
		boom.renderflags = $|RF_FULLBRIGHT
		boom.tics = $ + P_RandomRange(0, 4)
		boom.scale = FRACUNIT*3 + P_RandomRange(-FRACUNIT/3, FRACUNIT/3)
		
		S_StartSound(mo, sfx_s3kb4)
	end

	
	if mo.deathtimer >= TICRATE*2
	and mo.deathtimer < TICRATE*3
		mo.sprite = leveltime%4 < 2 and SPR_BRAK or SPR_NULL
	end
	
	if mo.deathtimer == TICRATE*3
		S_StartSound(nil, sfx_bedie2)
		
		mo.core = P_SpawnMobj(mo.x, mo.y, mo.z + mo.height/2, MT_THOK)
		mo.core.basez = mo.core.z
	end
	
	if mo.core and mo.core.valid
		
		mo.sprite = SPR_NULL
		
		mo.core.tics = -1
		mo.core.fuse = -1
		mo.core.frame = A|FF_ADD|FF_FULLBRIGHT
		mo.core.color = SKINCOLOR_EMERALD
		mo.core.scale = FRACUNIT*2
		
		if mo.deathtimer < TICRATE*6
			mo.core.z = mo.core.basez + cos(leveltime*ANG1*3)*16
		end
		
		for i = 1, 4
			local p = P_SpawnMobj(mo.core.x, mo.core.y, mo.core.z, MT_THOK)
			p.frame = A|FF_ADD|FF_FULLBRIGHT
			p.color = SKINCOLOR_EMERALD
			p.destscale = 1
			p.momx = P_RandomRange(-32, 32)*FRACUNIT
			p.momy = P_RandomRange(-32, 32)*FRACUNIT
			p.momz = P_RandomRange(-32, 32)*FRACUNIT
		end
		
		if mo.deathtimer == TICRATE*6
			
			for i = 1, 16
				local a = 360/16*ANG1*i
				local t = P_SpawnMobj(mo.core.x, mo.core.y, mo.core.z, MT_THOK)
				t.frame = A|FF_ADD|FF_FULLBRIGHT
				t.destscale = 1
				t.scale = $/2
				t.color = SKINCOLOR_EMERALD
				
				P_InstaThrust(t, a, FRACUNIT*32)
			end
			
			mo.core.momz = FRACUNIT*24
		elseif mo.deathtimer > TICRATE*6
			local t = P_SpawnMobj(mo.core.x, mo.core.y, mo.core.z, MT_THOK)
			t.frame = A|FF_ADD|FF_FULLBRIGHT	
			t.scale = FRACUNIT*2
			t.tics = TICRATE
			t.fuse = TICRATE
			t.destscale = 1
			t.color = SKINCOLOR_EMERALD
		end
	end
	
	if mo.deathtimer == TICRATE*8
		P_RemoveMobj(mo)
		return
	end
	
end
states[S_PSO_DARKFALZ_DEATH] = {SPR_BRAK, A, 1, A_DarkFalzDeathLoop, 1, 1, S_PSO_DARKFALZ_DEATH}

mobjinfo[MT_PSO_DARKFALZ_EXPLOSION] = {
	doomednum = -1,
	spawnhealth = 1000,
	deathstate = S_NULL,
	spawnstate = S_THOK,
	height = 50<<FRACBITS,
	radius = 50<<FRACBITS,
	flags = MF_RUNSPAWNFUNC,
}

freeslot("SPR_NTHK", "sfx_srip")

addHook("MobjThinker", function(mo)
	mo.sprite = SPR_NTHK
	mo.frame = D|FF_FULLBRIGHT
	mo.color = SKINCOLOR_EMERALD
	
	if mo.finalscale == nil
		mo.finalscale = FRACUNIT
		mo.fuse = TICRATE
	end	
	
	mo.finalscale = $ + FRACUNIT/4
	
	mo.scale = mo.finalscale + (leveltime & 1)*FRACUNIT/6
	mo.tics = -1
	
	if mo.fuse < 10 and mo.fuse
		mo.frame = $ | (10 - mo.fuse)<<FF_TRANSSHIFT
	end
	
	mo.existtimer = TICRATE - mo.fuse
	if mo.existtimer == TICRATE/2
		local range = PSO.lib.questDifficulty() >= DIFF_HARD and 256 or 192
		for i = 1, 32
			local x = mo.x + P_RandomRange(-range, range)*FRACUNIT
			local y = mo.y + P_RandomRange(-range, range)*FRACUNIT

			local fire = P_SpawnMobj(x, y, mo.z, MT_PSO_VIPERAFTERLASER)
			fire.pso = {}
			fire.colorized = true
			fire.color = SKINCOLOR_EMERALD
			for k,v in pairs(mo.target.pso)
				fire.pso[k] = v
			end
			
			fire.tics = TICRATE*15
			fire.fuse = TICRATE*15
		end
	end
	
end, MT_PSO_DARKFALZ_EXPLOSION)

function A_DarkBalzTrail(mo)
	mo.frame = leveltime%10 /2
	--print("A")
	if leveltime & 1
		local g = P_SpawnGhostMobj(mo)
		g.destscale = mo.scale*4
		g.frame = $|FF_ADD|FF_FULLBRIGHT
		g.colorized = true
		g.color = SKINCOLOR_EMERALD
	end
end
states[S_PSO_DARKBALZ] = {SPR_ROIJ, A, 1, A_DarkBalzTrail, 0, 0, S_PSO_DARKBALZ}

function A_DarkBalzDie(mo)
	local msl = PSO.lib.spawnMissile(mo.target, MT_PSO_BFLOW_EXPLOSION, 0, 0, mo.pso.potency, DMT_RANGED, 0, 0, 0)
	P_MoveOrigin(msl, mo.x, mo.y, mo.z)
end
states[S_PSO_DARKBALZ_DIE] = {SPR_NULL, A, 1, A_DarkBalzDie, 0, 0, S_NULL}

mobjinfo[MT_PSO_DARKBALZ] = {
	doomednum = -1,
	spawnhealth = 1000,
	spawnstate = S_PSO_DARKBALZ,
	deathstate = S_PSO_DARKBALZ_DIE,
	xdeathstate = S_PSO_DARKBALZ_DIE,
	height = 48<<FRACBITS,
	radius = 24<<FRACBITS,
	flags = MF_MISSILE,
}


local function DF_isNeutral(mo)
	return not mo.dfstate
end

local function DF_initBurst(mo)
	P_InstaThrust(mo, 0, 0)
	mo.dfstate = DF_BURST
	
	mo.burst = 1
	S_StartSound(nil, sfx_beragh)
	P_StartQuake(FRACUNIT*10, TICRATE*4)
end

local function DF_initBeam(mo)
	P_InstaThrust(mo, 0, 0)
	mo.dfstate = DF_BEAM
	
	mo.beam = 1
	S_StartSound(nil, sfx_bewar3)
	S_StartSound(nil, sfx_bechrg)
end

local function DF_initJump(mo)
	P_InstaThrust(mo, 0, 0)
	mo.dfstate = DF_JUMP
	
	mo.jumping = 1
end

local function DF_initDoubleCross(mo)
	P_InstaThrust(mo, 0, 0)
	mo.dfstate = DF_DOUBLECROSS
	
	mo.doublecross = 1
	S_StartSound(nil, sfx_bewar2)
end

local function DF_doDoubleCross(mo)
	
	local wait = TICRATE*3/2
	local endlag = TICRATE*3

	local potency = 400
	
	if mo.doublecross
		
		mo.doublecross = $+1
		
		if mo.doublecross < wait
			mo.frame = S
			mo.angle = R_PointToAngle2(mo.x, mo.y, mo.target.x, mo.target.y)
		else
			
			local tmr = mo.doublecross-wait
			local frame = min(U, S +tmr/3)
			
			mo.frame = frame
			
			if mo.doublecross == wait
				
				local range = 64
				local qcross = 1
				if mo.whip
					qcross = 2
				end
				
				for p = 1, qcross
					
					local rot_ang = (p-1)*ANG1*45
					
					for i = 1, 2
						local base_ang = mo.angle + ANG1*90*(i-1) + rot_ang
						local base_delay = TICRATE/4 + (i-1)*TICRATE*3/4
						
						for j = 1, 2
							
							local ang = base_ang + ANG1*180*(j-1)
							local dist = 48
							
							for k = 1, range
								
								local x = mo.x + dist*cos(ang)
								local y = mo.y + dist*sin(ang)
								
								local msl = PSO.lib.spawnMissile(mo, MT_PSO_BFLOW_EXPLOSION, 0, 0, potency, DMT_RANGED, 0, 0, 0)
								P_MoveOrigin(msl, x, y, mo.z)
								msl.angle = ang
								
								msl.delay = base_delay + (k-1)*3
								dist = $+96
							end
							
						end
						
					end
				end
				
			elseif mo.doublecross >= wait + endlag
				mo.doublecross = nil
				mo.dfstate = DF_NEUTRAL
			end
		end
	end	
end

local function DF_doJump(mo)
	
	local wait = TICRATE/3
	local airtime = TICRATE*2
	local potency = 400
	
	if mo.jumping
		
		mo.jumping = $+1
		
		if mo.jumping < wait
			mo.frame = W
			
			mo.angle = R_PointToAngle2(mo.x, mo.y, mo.target.x, mo.target.y)
		
		elseif mo.jumping == wait
			
			local gravity = P_GetMobjGravity(mo)/2
			local dist = R_PointToDist2(mo.x, mo.y, mo.target.x, mo.target.y)
			
			local horz = dist / airtime
			local vert = -FixedMul((gravity*airtime)/2, mo.scale)
			
			vert = max($, FRACUNIT*16)
			
			P_InstaThrust(mo, mo.angle, horz)
			mo.momz = vert
			
			mo.flags = $ & ~MF_SOLID
			
		elseif mo.jumping > wait
			
			mo.frame = X
			if leveltime & 1
				local g = P_SpawnGhostMobj(mo)
				g.color = SKINCOLOR_EMERALD
				g.colorized = true
			end
			
			if P_IsObjectOnGround(mo)
				for i = 1, 16
					local a = 360/16 *ANG1 *i
					local d = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SPINDUST)
					P_InstaThrust(d, a, FRACUNIT*32)
				end
				
				P_StartQuake(FRACUNIT*20, 8)
				S_StartSound(nil, sfx_befall)
				
				mo.momz = $ - P_GetMobjGravity(mo)/2
				
				local h = PSO.lib.spawnHurtBox(mo.x, mo.y, mo.z-FRACUNIT*256, 512<<FRACBITS, 896<<FRACBITS, mo, potency, DMT_MELEE, 4, RT_BLOWDOWN)
				h.angle = mo.angle
				h.pso.reactionradialangle = true				

				local range = 256
				for i = 1, 32
					local x = mo.x + P_RandomRange(-range, range)*FRACUNIT
					local y = mo.y + P_RandomRange(-range, range)*FRACUNIT

					local fire = P_SpawnMobj(x, y, mo.z, MT_PSO_VIPERAFTERLASER)
					fire.pso = {}
					fire.colorized = true
					fire.color = SKINCOLOR_EMERALD
					
					for k,v in pairs(mo.pso)
						fire.pso[k] = v
					end
					
					fire.tics = TICRATE*15
					fire.fuse = TICRATE*15
				end
				
				mo.jumping = nil
				mo.dfstate = DF_NEUTRAL
				mo.flags = $|MF_SOLID
				
			end
		end
	end
end

local function DF_doBurst(mo)
	
	local wait = TICRATE*3/2
	local endlag = TICRATE*2
	local diff = PSO.lib.questDifficulty()

	local rock_potency = 400
	
	if mo.burst
		
		mo.burst = $+1
		
		if mo.burst < wait
			mo.frame = S
		else
			
			local tmr = mo.burst-wait
			local frame = min(U, S +tmr/3)
			
			mo.frame = frame
			
			if mo.burst == wait
				
				for i = 1, diff >= DIFF_HARD and 64 or 32
					local msl = PSO.lib.spawnMissile(mo, MT_PSO_DARKBALZ, 0, 0, rock_potency, DMT_RANGED, 0, 0, 0)
					msl.target = mo
					msl.scale = $*2
					P_MoveOrigin(msl, mo.x, mo.y, mo.z + mo.height)
					
					local spd = diff >= DIFF_HARD and 13 or 8
					msl.momz = P_RandomRange(16, 24)*FRACUNIT
					msl.momx = P_RandomRange(-spd, spd)*FRACUNIT
					msl.momy = P_RandomRange(-spd, spd)*FRACUNIT
				end
				
			elseif mo.burst >= wait + endlag
				mo.burst = nil
				mo.dfstate = DF_NEUTRAL
			end
		end
	end
end

local function DF_LaserThinker(mo)
	local x, y, z = mo.x, mo.y, mo.z

	if mo.momx ~= 0 or mo.momy ~= 0
		if not P_TryMove(mo, mo.x + mo.momx/2, mo.y + mo.momy/2, true) return true end
		if not mo.valid return true end
	end

	if mo.momz ~= 0
		mo.z = $ + mo.momz/2
		if mo.z < mo.floorz or mo.z + mo.height > mo.ceilingz return true end
	end

	return not mo.valid or x == mo.x and y == mo.y and z == mo.z
end

local function DF_SpawnLaser(mo)
	local nblockmap = not (mo.flags & MF_NOBLOCKMAP)

	for i = 0, 512
		if nblockmap
			mo.flags = $ | MF_NOBLOCKMAP
		end

		local p = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
		p.scale = mo.scale
		p.destscale = 1
		p.scalespeed = mo.scale/12
		p.frame = A|FF_ADD|FF_FULLBRIGHT
		p.color = SKINCOLOR_EMERALD
		p.tics = TICRATE

		p = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
		p.scale = mo.scale*3/2
		p.destscale = 1
		p.frame = A|TR_TRANS80|FF_FULLBRIGHT
		p.scalespeed = mo.scale/12
		p.color = SKINCOLOR_WHITE
		p.tics = TICRATE

		if DF_LaserThinker(mo)
			break
		end
	end
	
	local msl = PSO.lib.spawnMissile(mo.target, MT_PSO_DARKFALZ_EXPLOSION, 0, 0, 0, DMT_RANGED, 0, 0, 0)
	msl.target = mo.target
	P_MoveOrigin(msl, mo.x, mo.y, mo.z)
end

local function DF_doBeam(mo)
	
	local wait = TICRATE*3
	local endlag = TICRATE*2
	local range = 3
	local angrange = ANG1*70
	
	local burst_potency = 500
	local rock_potency = 400
	
	if mo.beam
		
		mo.beam = $+1
		
		if mo.beam < wait
			mo.frame = V
			
			if mo.beam < wait-16
				for i = 1, 2
					local p = P_SpawnMobj(mo.x+P_RandomRange(-256, 256)*FRACUNIT, mo.y+P_RandomRange(-256, 256)*FRACUNIT, mo.z+P_RandomRange(0, 256)*FRACUNIT, MT_THOK)
					p.momx = (mo.x - p.x)/16
					p.momy = (mo.y - p.y)/16
					p.momz = ((mo.z+mo.height/2 + FRACUNIT*48) - p.z)/16
					
					p.destscale = 1
					p.color = SKINCOLOR_EMERALD
					p.frame = A|FF_ADD|FF_FULLBRIGHT
					p.tics = 16
				end
			end
			
			mo.angle = R_PointToAngle2(mo.x, mo.y, mo.target.x, mo.target.y)
		else
			
			if mo.beam == wait
				--local t = P_SpawnMobj(mo.x, mo.y, mo.z + mo.height/2 + FRACUNIT*48, MT_THOK)
				local t = PSO.lib.spawnMissile(mo.target, MT_THOK, 0, 0, 100, DMT_RANGED, 0, 0, 0)
				P_MoveOrigin(t, mo.x, mo.y, mo.z + mo.height/2 + FRACUNIT*48)
				P_InstaThrust(t, mo.angle, FRACUNIT*32)
				t.target = mo
				
				PSO.lib.aimMissile(t, mo.target)
				DF_SpawnLaser(t)	-- makes it laser forward, source: trust me
				P_RemoveMobj(t)
				
			elseif mo.beam >= wait + endlag
				mo.beam = nil
				mo.dfstate = DF_NEUTRAL
			end
		end
	end
end

local function DF_walk(mo)
	local numframes = 6
	local speed = 8
	local movespeed = 4*FRACUNIT
	
	if not mo.pushbox
		local push = P_SpawnMobj(mo.x, mo.y, mo.z, MT_PSO_PUSHBOX)
		push.target = mo
		mo.pushbox = push
	end
	
	mo.frame = B + (leveltime % (numframes*speed)) / speed
	
	mo.angle = R_PointToAngle2(mo.x, mo.y, mo.target.x, mo.target.y)
	P_InstaThrust(mo, mo.angle, movespeed)
	
	if leveltime%27 == 0
		if leveltime & 1
			S_StartSound(mo, sfx_bestep)
		else
			S_StartSound(mo, sfx_bestp2)
		end
		
		if PSO.lib.questDifficulty() >= DIFF_HARD
		
			local x = mo.x - 96*cos(mo.angle)
			local y = mo.y - 96*sin(mo.angle)

			local fire = P_SpawnMobj(x, y, mo.z, MT_PSO_VIPERAFTERLASER)
			fire.pso = {}
			fire.colorized = true
			fire.color = SKINCOLOR_EMERALD
			for k,v in pairs(mo.pso)
				fire.pso[k] = v
			end
			
			fire.tics = TICRATE*25
			fire.fuse = TICRATE*25		
		end
	end
end

local function DF_halo(mo)
	
	local scale = {FRACUNIT, FRACUNIT*3/2}
	local angs = {ANG1*2, -ANG1*2}
	
	for i = 1, 2
		local halo = P_SpawnMobj(mo.x, mo.y, mo.z+mo.height + FRACUNIT*140, MT_THOK)
		halo.sprite = SPR_BFLW
		halo.frame = D|FF_ADD
		halo.flags2 = $ | MF2_SPLAT
		halo.renderflags = $|RF_FLOORSPRITE|RF_NOSPLATBILLBOARD|RF_FULLBRIGHT
		halo.tics = 1
		halo.rollangle = leveltime*angs[i]
		halo.scale = scale[i]
	end
end

-- readies the whip...
local function DF_initWhip(mo)
	
	if not mo.whip
		
		mo.whip = {}
		mo.whip_targets = {}
		
		for i = 1, 3
			local w = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
			w.tics = -1
			--w.frame = A|FF_FULLBRIGHT
			--w.color = SKINCOLOR_PURPLE
			
			mo.whip[#mo.whip+1] = w
			mo.whip_targets[#mo.whip_targets+1] = {mo.x, mo.y, mo.z}
		end
	end
end

local function DF_whipUpdate(mo)
	
	if not mo.whip or not #mo.whip return end
	
	local lags = {1<<FRACBITS, 8<<FRACBITS, 32<<FRACBITS}
	for i = 1, #mo.whip
		
		local targ = mo.whip_targets[i]
		local w = mo.whip[i]
		local lag = lags[i]
		
		if mo.whipfastlag
			lag = $/8
		end
		
		w.tics = 3
		w.momx = FixedDiv(targ[1] -w.x, lag)
		w.momy = FixedDiv(targ[2] -w.y, lag)
		w.momz = FixedDiv(targ[3] -w.z, lag)
	end
end

-- (the actual visual)
local spline_step = FRACUNIT/32

local function DF_whipSpline(mo)
	
	if not mo.whip return end
	local path = mo.whip
	local trail
	local spawning = mo.dfspawnwhip
	
	if mo.dfstate == DF_WHIPSMASH
	or mo.dfstate == DF_WHIPSLASH
		trail = true
	end
	
	mo.splinedta = $ or {}
	mo.whiphitboxes = $ or {}
	
	if not mo.whiphitbox
		mo.whiphitboxes = {}
	end
	
	local frac = 0
	local prev
	local num = 0
	local range = 32
	
	while frac < ((#path-1) << FRACBITS)
		--print(frac..", "..(frac/FRACUNIT))
		local pos = PSO.lib.pointOnSpline(path, frac)
		num = $+1
		
		if pos

			if mo.whiphitbox
				if mo.whiphitboxes[num]
				and mo.whiphitboxes[num].valid
					
					local hb = mo.whiphitboxes[num]
					hb.fuse = 2
					hb.tics = 2
					
					P_MoveOrigin(hb, pos.x, pos.y, pos.z)
					hb.angle = R_PointToAngle2(hb.prevpos[1], hb.prevpos[2], hb.x, hb.y)
					hb.prevpos = {hb.x, hb.y, hb.z}
					
					hb.pso.links = mo.whiphitboxes
					
				else
					--print("A")
					local hb = PSO.lib.spawnHurtBox(pos.x, pos.y, pos.z, 192<<FRACBITS, 192<<FRACBITS, mo, mo.whiphitbox, DMT_MELEE, 4, RT_BLOWDOWN)
					hb.prevpos = {hb.x, hb.y, hb.z}
					mo.whiphitboxes[num] = hb
				end
					
			end	
			
			mo.splinedta[num] = $ or {}
			for i = 1, 3
				
				if mo.splinedta[num][i]
					local seg = mo.splinedta[num][i]
					
					local tx = pos.x + seg.diffs[1]
					local ty = pos.y + seg.diffs[2]
					local tz = pos.z + seg.diffs[3]
					
					P_MoveOrigin(seg, tx, ty, tz)
					seg.tics = 3
					
					if spawning-4 == num/2
						seg.flags2 = $ & ~MF2_DONTDRAW
						seg.scale = 1
						seg.destscale = FRACUNIT
						seg.scalespeed = FRACUNIT/6
						
						if i == 1 and num%5 == 0
							S_StartSound(seg, sfx_s3k81)
						end
						
						for i = 1, 8
							local p = P_SpawnMobj(seg.x, seg.y, seg.z, MT_THOK)
							p.destscale = 1
							p.color = SKINCOLOR_EMERALD
							p.frame = A|FF_ADD|FF_FULLBRIGHT
							p.momx = P_RandomRange(-32, 32)*FRACUNIT
							p.momy = P_RandomRange(-32, 32)*FRACUNIT
							p.momz = P_RandomRange(-32, 32)*FRACUNIT
						end
						
					end
					
					if trail and leveltime & 1
						local g = P_SpawnGhostMobj(seg)
						g.colorized = true
						g.frame = $|FF_FULLBRIGHT
						g.color = SKINCOLOR_EMERALD
					end					
				else
					local seg = P_SpawnMobj(pos.x + P_RandomRange(-range, range)*FRACUNIT, pos.y + P_RandomRange(-range, range)*FRACUNIT, pos.z + P_RandomRange(-range, range)*FRACUNIT, MT_THOK)
					seg.tics = -1
					seg.sprite = SPR_BRKN
					seg.frame = P_RandomRange(A, H)
					seg.flags2 = $|MF2_DONTDRAW
					--seg.scale = FRACUNIT*2
					seg.rollangle = P_RandomRange(0, 359)*ANG1
					
					seg.diffs = {seg.x - pos.x, seg.y - pos.y, seg.z - pos.z}
					
					table.insert(mo.splinedta[num], seg)
				end
			end
		end

		frac = $ + spline_step
	end
end

-- whip neutral position
local function DF_whipSetNeutral(mo)
	
	if not mo.whip return end
	
	local start_ang = mo.angle + ANG1*90
	local start_dist = 72
	local start_height = mo.z + mo.height + FRACUNIT*32
	
	-- point 1:
	local p1x = mo.x + start_dist*cos(start_ang)
	local p1y = mo.y + start_dist*sin(start_ang)
	local p1z = start_height
	
	mo.whip_targets[1] = {p1x, p1y, p1z}
	
	-- point 2 is below and behind point 1 at an angle
	local cont_angle = mo.angle + ANG1*165
	local cont_dist = 72
	local cont_height = -mo.height*2/3
	
	local p2x = p1x + cont_dist*cos(cont_angle)
	local p2y = p1y + cont_dist*sin(cont_angle)
	local p2z = p1z + cont_height
	
	mo.whip_targets[2] = {p2x, p2y, p2z}
	
	-- point 3 is far behind point 2 trailing on the ground
	cont_angle = mo.angle + ANG1*180
	cont_dist = 320
	cont_height = mo.z
	
	local p3x = p2x + cont_dist*cos(cont_angle)
	local p3y = p2y + cont_dist*sin(cont_angle)
	local p3z = cont_height
	
	mo.whip_targets[3] = {p3x, p3y, p3z}
end

local function DF_initWhipSmash(mo)
	P_InstaThrust(mo, 0, 0)
	mo.dfstate = DF_WHIPSMASH
	S_StartSound(nil, sfx_s3k60)
	S_StartSound(nil, sfx_bewar2)
	
	mo.whipsmash = 1
end

local function DF_doWhipSmash(mo)
	
	local whip_wait = PSO.lib.questDifficulty() >= DIFF_HARD and TICRATE or TICRATE*2
	local whip_speed = 7	-- it takes 5 frames for the whip to reach the player (fast!)
	local whip_endlag = TICRATE
	
	local whip_potency = 400
	
	if mo.whipsmash
		
		mo.whipsmash = $+1
		
		if mo.whipsmash == 3
			
			mo.whip_targets[2][3] = mo.z + mo.height*2
			mo.whip_targets[3][3] = mo.z + mo.height*4
			
		elseif mo.whipsmash == whip_wait
			
			mo.whipfastlag = true
			
			local ang = R_PointToAngle2(mo.x, mo.y, mo.target.x, mo.target.y)
			
			local t3x = mo.target.x + 128*cos(ang)
			local t3y = mo.target.y + 128*sin(ang)
			
			mo.whip_targets[3] = {t3x, t3y, mo.z}
			
			local t2x = (mo.x/FRACUNIT + mo.target.x/FRACUNIT)/2 *FRACUNIT
			local t2y = (mo.y/FRACUNIT + mo.target.y/FRACUNIT)/2 *FRACUNIT
			mo.whip_targets[2] = {t2x, t2y, mo.z+mo.height/2}
			
			S_StartSound(nil, sfx_s3k61)
			
		elseif mo.whipsmash == whip_wait + whip_speed
			
			local x = mo.whip_targets[3][1]
			local y = mo.whip_targets[3][2]
			local z = mo.whip_targets[3][3]
			
			local h = PSO.lib.spawnHurtBox(x, y, z-FRACUNIT*256, 512<<FRACBITS, 896<<FRACBITS, mo, whip_potency, DMT_MELEE, 4, RT_BLOWDOWN)
			h.angle = mo.angle
			h.pso.reactionradialangle = true
			
			P_StartQuake(FRACUNIT*30, 4)
			
			DF_whipSetNeutral(mo)
			mo.whip_targets[2][3] = $ + mo.height*2
			mo.whip_targets[3][1] = mo.whip_targets[2][1]
			mo.whip_targets[3][2] = mo.whip_targets[2][2]
			mo.whip_targets[3][3] = mo.whip_targets[2][3] - mo.height
			
			mo.whipfastlag = false
			
			for i = 1, 32
				
				local d = P_SpawnMobj(x + P_RandomRange(-256, 256)*FRACUNIT, y + P_RandomRange(-256, 256)*FRACUNIT, z, MT_SPINDUST)
				d.momx = P_RandomRange(-32, 32)*FRACUNIT
				d.momy = P_RandomRange(-32, 32)*FRACUNIT
				d.momz = P_RandomRange(0, 16)*FRACUNIT
				
			end
			
			S_StartSound(nil, sfx_s3k9b)
		
		elseif mo.whipsmash == whip_wait + whip_speed + whip_endlag
			mo.whipsmash = nil
			mo.dfstate = DF_NEUTRAL
		end
	end
end

local function DF_initWhipSlash(mo)
	P_InstaThrust(mo, 0, 0)
	mo.dfstate = DF_WHIPSLASH
	
	mo.whipslash = 1
	
	S_StartSound(nil, sfx_s3ka3)
	S_StartSound(nil, sfx_bewar4)
end


local function DF_doWhipSlash(mo)
	
	local whip_wait = PSO.lib.questDifficulty() >= DIFF_HARD and TICRATE or TICRATE*2
	local whip_endlag = TICRATE
	
	local whip_potency = 500
	
	local swing_speed = ANG1*10
	local swing_duration = TICRATE*3/4
	
	if mo.whipslash
		
		mo.whipslash = $+1
		
		if mo.whipslash == 3
						
			local ref_ang = mo.angle + ANG1*100
			local dist1 = 256
			local z1 = -64
			
			local dist2 = 768
			local z2 = -96
			
			local x = mo.whip_targets[1][1] + dist1*cos(ref_ang)
			local y = mo.whip_targets[1][2] + dist1*sin(ref_ang)
			
			mo.whip_targets[2][1] = x
			mo.whip_targets[2][2] = y
			mo.whip_targets[2][3] = mo.whip_targets[1][3] + z1*FRACUNIT
			
			local x = mo.whip_targets[1][1] + dist2*cos(ref_ang)
			local y = mo.whip_targets[1][2] + dist2*sin(ref_ang)

			mo.whip_targets[3][1] = x
			mo.whip_targets[3][2] = y
			mo.whip_targets[3][3] = mo.whip_targets[1][3] + z2*FRACUNIT
			
		elseif mo.whipslash >= whip_wait
		and mo.whipslash < whip_wait + swing_duration
			
			if mo.whipslash == whip_wait+4
				S_StartSound(nil, sfx_s3ka2)
			end
			
			local t = mo.whipslash - whip_wait
			local ang = mo.angle + ANG1*100 - swing_speed*t
			local dist1 = 320
			local dist2 = 1024
			
			mo.whipfastlag = true
			mo.whiphitbox = 500
			
			mo.whip_targets[2][1] = mo.x + dist1*cos(ang)
			mo.whip_targets[2][2] = mo.y + dist1*sin(ang)
			mo.whip_targets[2][3] = mo.z + mo.height/4
			
			
			mo.whip_targets[3][1] = mo.x + dist2*cos(ang)
			mo.whip_targets[3][2] = mo.y + dist2*sin(ang)
			mo.whip_targets[3][3] = mo.z			
			
			
		elseif mo.whipslash == whip_wait + swing_duration
			
			local x = mo.whip_targets[3][1]
			local y = mo.whip_targets[3][2]
			local z = mo.whip_targets[3][3]
			
			DF_whipSetNeutral(mo)
			mo.whip_targets[2][3] = $ + mo.height*2
			mo.whip_targets[3][1] = mo.whip_targets[2][1]
			mo.whip_targets[3][2] = mo.whip_targets[2][2]
			mo.whip_targets[3][3] = mo.whip_targets[2][3] - mo.height
			
			mo.whipfastlag = false
			mo.whiphitbox = nil
		
		elseif mo.whipslash == whip_wait + swing_duration + whip_endlag
			mo.whipsmash = nil
			mo.dfstate = DF_NEUTRAL
		end
	end
end

local function DF_initSpawnWhip(mo)
	P_InstaThrust(mo, 0, 0)
	mo.dfstate = DF_SPAWNWHIP
	mo.dfspawnwhip = 1
	
	S_StartSound(nil, sfx_bewar3)
end

local function DF_doSpawnWhip(mo)
	
	if mo.dfspawnwhip
		
		mo.dfspawnwhip = $+1
		if mo.dfspawnwhip > TICRATE*69
			mo.dfspawnwhip = TICRATE*69
		end
		
		if mo.dfspawnwhip == 2
			
			S_StartSound(nil, sfx_srip)
			
			DF_initWhip(mo)
			DF_whipSetNeutral(mo)
			
			local ref_ang = mo.angle + ANG1*100
			local dist1 = 256
			local z1 = -64
			
			local dist2 = 768
			local z2 = -96
			
			local x = mo.whip_targets[1][1] + dist1*cos(ref_ang)
			local y = mo.whip_targets[1][2] + dist1*sin(ref_ang)
			
			mo.whip_targets[2][1] = x
			mo.whip_targets[2][2] = y
			mo.whip_targets[2][3] = mo.whip_targets[1][3] + z1*FRACUNIT
			
			local x = mo.whip_targets[1][1] + dist2*cos(ref_ang)
			local y = mo.whip_targets[1][2] + dist2*sin(ref_ang)

			mo.whip_targets[3][1] = x
			mo.whip_targets[3][2] = y
			mo.whip_targets[3][3] = mo.whip_targets[1][3] + z2*FRACUNIT			
			
			P_MoveOrigin(mo.whip[2], mo.whip_targets[2][1], mo.whip_targets[2][2], mo.whip_targets[2][3])
			P_MoveOrigin(mo.whip[3], mo.whip_targets[3][1], mo.whip_targets[3][2], mo.whip_targets[3][3])
			
			for i = 1, 32
				
				local p = P_SpawnMobj(mo.whip_targets[1][1], mo.whip_targets[1][2], mo.whip_targets[1][3], MT_THOK)
				p.scale = $ + P_RandomRange(-FRACUNIT/3, FRACUNIT/3)
				p.color = SKINCOLOR_EMERALD
				p.frame = $|FF_ADD|FF_FULLBRIGHT
				p.fuse = P_RandomRange(8, 16)
				p.tics = p.fuse
				
				p.angle = mo.angle + ANG1*90 + P_RandomRange(-30, 30)*ANG1
				P_InstaThrust(p, p.angle, P_RandomRange(4, 32)*FRACUNIT)
				p.momz = P_RandomRange(-12, 12)*FRACUNIT
			end
			
		elseif mo.dfspawnwhip == TICRATE*3/2
			mo.dfstate = DF_NEUTRAL
		end
	end
end

freeslot("sfx_emerch", "sfx_cdisa")
-- the god damn 30s long intro...
local function DF_intro(mo)
	
	-- N O P Q R
	local fast = mo.options & 8
	
	local wait = TICRATE*(fast and 1 or 3)
	local lift = TICRATE*(fast and 3 or 8)
	local hover = TICRATE*(fast and 2 or 4)
	
	local rotspeed = TICRATE/4
	
	local chargewait = TICRATE*(fast and 2 or 6)
	local quakewait = TICRATE*(fast and 1 or 4)
	local flash_begin = TICRATE*(fast and 1 or 4)
	
	if not mo.introd
		mo.intro = $ and $+1 or 1
		
		if mo.intro < wait
			mo.frame = R
			mo.flags = $|MF_NOBLOCKMAP
		elseif mo.intro == wait
		and not (mo.options & 8)
			S_ChangeMusic("FBOSS2", true)
		end
		
		if mo.intro > wait
		and mo.intro < wait+lift
			mo.momz = $ - P_GetMobjGravity(mo)*10002/10000
		end
		
		if mo.intro == wait+lift
			mo.introsetz = mo.z
		elseif mo.intro >= wait+lift
			mo.z = mo.introsetz + 10*cos(leveltime*ANG1*3)
		end
		
		if mo.intro >= wait + lift + hover
			local tmr = mo.intro - wait - lift - hover
			mo.frame = max(N, R - tmr/rotspeed)
		end
		
		if mo.intro >= wait + lift + hover + chargewait
			for i = 1, 3
				
				local x = mo.x + P_RandomRange(-256, 256)*FRACUNIT
				local y = mo.y + P_RandomRange(-256, 256)*FRACUNIT
				local z = mo.z + P_RandomRange(0, 256)*FRACUNIT
				
				local p = P_SpawnMobj(x, y, z, MT_THOK)
				p.frame = A|FF_ADD|FF_FULLBRIGHT
				p.destscale = 1
				p.color = SKINCOLOR_EMERALD
				p.tics = 8
				
				p.momx = (mo.x - p.x)/8
				p.momy = (mo.y - p.y)/8
				p.momz = ((mo.z+mo.height/2) - p.z)/8
			end
		end
		
		if mo.intro == wait + lift + hover +chargewait + quakewait
			P_StartQuake(FRACUNIT*20, TICRATE*8)
		end

		if mo.intro == wait + lift + hover +chargewait + quakewait + flash_begin
			for p in players.iterate do
				P_FlashPal(p, 1, 17)
				
				PSO.lib.startEmergency(p, "CODE UPDATE", "(DISASTER)", sfx_emerch, sfx_cdisa, TICRATE/2)
				
			end
			mo.introd = true
			mo.z = mo.floorz
			
			mo.flags = $ & ~MF_NOBLOCKMAP
		end
	end
end

PSO.info.enemies[MT_PSO_DARKFALZ] = {
		
		name = "Dark Falz",
		sprite = SPR_BRAK,
		frame = A,
		painframe = A,
		
		bhp = 6500,
		satk = 35,
		ratk = 35,
		tatk = 35,
		def = 20,
		exp = 600,
		weight = 4,

		boss = true,		-- enemy is a major boss!
		waitclear = true,	-- only clear the room when the entity is gone
		--bossmusic = "FBOSS1",
		
		
		drops = {
			{"alva pine", 1, 35, 1, 100},
			{"alva claymore", 1, 35, 1, 100},
			{"vita pine", 1, 35, 1, 100},
			{"vita claymore", 1, 35, 1, 100},
			
			{"alva doubliss", 1, 35, 1, 100},
			{"alva doublecutter", 1, 35, 1, 100},	
			{"vita doubliss", 1, 35, 1, 100},
			{"vita doublecutter", 1, 35, 1, 100},
			
			{"alva shooter", 1, 35, 1, 100},
			{"alva burster", 1, 35, 1, 100},				
			{"vita shooter", 1, 35, 1, 100},
			{"vita burster", 1, 35, 1, 100},
				
			{"alva shot", 1, 35, 1, 100},
			{"alva concentrator", 1, 35, 1, 100},	
			{"vita shot", 1, 35, 1, 100},
			{"vita concentrator", 1, 35, 1, 100},
			
			{"alva crest", 1, 35, 1, 100},
			{"alva crescent", 1, 35, 1, 100},
			{"vita crest", 1, 35, 1, 100},
			{"vita crescent", 1, 35, 1, 100},
				
			{"alva claw", 1, 35, 1, 100},
			{"alva cutter", 1, 35, 1, 100},			
			{"vita claw", 1, 35, 1, 100},
			{"vita cutter", 1, 35, 1, 100},
			
			{"alva beamgun", 1, 35, 1, 100},
			{"alva peacekeeper", 1, 35, 1, 100},			
			{"vita beamgun", 1, 35, 1, 100},
			{"vita peacekeeper", 1, 35, 1, 100},
					
			{"alva hammer", 1, 35, 1, 100},
			{"alva scepter", 1, 35, 1, 100},
			{"vita hammer", 1, 35, 1, 100},
			{"vita scepter", 1, 35, 1, 100},
			
			{"unit_hp_2", 1, 35, 2, 50},
			{"unit_pp_2", 1, 35, 2, 50},
			{"unit_satk_2", 1, 35, 2, 50},
			{"unit_tatk_2", 1, 35, 2, 50},
			{"unit_ratk_2", 1, 35, 2, 50},

			{"digrinder", 1, 35, 1, 5},


			-- Lv36-55
			{"vita pine", 36, 55, 1, 100},
			{"vita claymore", 36, 55, 1, 100},
			{"vita doubliss", 36, 55, 1, 100},
			{"vita doublecutter", 36, 55, 1, 100},				
			{"vita shooter", 36, 55, 1, 100},
			{"vita burster", 36, 55, 1, 100},						
			{"vita shot", 36, 55, 1, 100},
			{"vita concentrator", 36, 55, 1, 100},			
			{"vita crest", 36, 55, 1, 100},
			{"vita crescent", 36, 55, 1, 100},			
			{"vita claw", 36, 55, 1, 100},
			{"vita cutter", 36, 55, 1, 100},					
			{"vita beamgun", 36, 55, 1, 100},
			{"vita peacekeeper", 36, 55, 1, 100},						
			{"vita hammer", 36, 55, 1, 100},
			{"vita scepter", 36, 55, 1, 100},
			
			{"unit_ppsatk_2", 36, 55, 3, 50},
			{"unit_pptatk_2", 36, 55, 3, 50},
			{"unit_ppratk_2", 36, 55, 3, 50},
			{"unit_rainbow_2", 36, 55, 3, 50},
			
			{"flow frame", 36, 55, 1, 100},
			{"scape doll", 36, 55, 1, 40},
			
			{"orakl sword", 36, 55, 1, 300},
			{"orakl doublesaber", 36, 55, 1, 300},
			{"orakl soaringblade", 36, 55, 1, 300},
			{"orakl rifle", 36, 55, 1, 300},
			{"orakl shot", 36, 55, 1, 300},
			{"orakl orphan", 36, 55, 1, 300},
			{"orakl rod", 36, 55, 1, 300},
			{"orakl wand", 36, 55, 1, 300},
			{"digrinder", 36, 55, 6, 50},
			
			{10000, nil, nil, 1, 10},
		},
		
		stagger_duration = TICRATE/2,
		
		weakness = ELEM_LIGHT,
		
		reset =	function(mo)
			mo.actiontimer = 0	-- reset action if we get staggered etc.
		end,
		
		thinker = function(mo)
			
			PSO.lib.enemyManageAggro(mo)
				
			mo.sprite = SPR_BRAK
			--mo.scale = FRACUNIT*2
			mo.spritexscale = FRACUNIT*2
			mo.spriteyscale = FRACUNIT*2
			mo.shadowscale = FRACUNIT
			
			if not mo.introd
				DF_intro(mo)
				return
			end
			
			mo.frame = A	--R
			
			-- todo: guard this on p.02
			--DF_initWhip(mo)			
			DF_halo(mo)
			
			if mo.target and mo.target.valid
				DF_doBurst(mo)
				DF_doBeam(mo)
				DF_doJump(mo)
				DF_doDoubleCross(mo)
				
				-- phase 2
				DF_doSpawnWhip(mo)
				DF_doWhipSmash(mo)
				DF_doWhipSlash(mo)

				if DF_isNeutral(mo)
					mo.waittimer = $ and $+1 or 1
					--mo.waittimer = 1
					
					if mo.waittimer > TICRATE*5
						
						local dist = R_PointToDist2(mo.x, mo.y, mo.target.x, mo.target.y)
						mo.numattacks = $ and $+1 or 1
						
						if not mo.whip
							local rng = P_RandomRange(0, 2)
							
							if mo.pso.hp <= mo.pso.maxhp*6/10
								DF_initSpawnWhip(mo)
								mo.waittimer = TICRATE*4
								return
							end
							
							-- phase 1 thinker
							if dist < 768*FRACUNIT
								
								if rng == 0
									DF_initBurst(mo)
								elseif rng == 1
									DF_initDoubleCross(mo)
								elseif rng == 2
									DF_initJump(mo)
								end
							else
								if rng == 0
									DF_initJump(mo)
								elseif rng == 1
									DF_initBeam(mo)
								elseif rng == 2
									DF_initDoubleCross(mo)
								end
							end
						else	-- whip ready
							
							local rng = P_RandomRange(0, 3)
							if dist < 768*FRACUNIT
								
								if mo.numattacks%3 == 0
									DF_initWhipSmash(mo)
								
								elseif rng == 0
									DF_initBurst(mo)
								elseif rng == 1
									DF_initDoubleCross(mo)
								elseif rng == 2
									DF_initWhipSmash(mo)
								elseif rng == 3
									DF_initBeam(mo)
								end
							else
								
								if mo.numattacks%3 == 0
									
									if rng & 1
										DF_initWhipSmash(mo)
									else
										DF_initWhipSlash(mo)
									end
									
								elseif rng == 0
									DF_initBeam(mo)
								elseif rng == 1
									DF_initDoubleCross(mo)
								elseif rng == 2
									DF_initWhipSmash(mo)
								elseif rng == 3
									DF_initWhipSlash(mo)
								end
							end
						end
						
						mo.waittimer = 0
					else
						DF_whipSetNeutral(mo)
						DF_walk(mo)
					end
				end
			else
				mo.frame = A
				mo.momx = 0
				mo.momy = 0
			end

			DF_whipUpdate(mo)
			DF_whipSpline(mo)			
			
		end,
}