-- enemy definitions
-- all stats are at level 1
-- enemies will automatically scale up with enemy level / difficulty

-- dummy state for enemies to go in so we won't be bothered by external behaviours.
freeslot("S_PSO_ENEMYDUMMY")
states[S_PSO_ENEMYDUMMY] = {SPR_NULL, A, -1, nil, 0, 0, S_PSO_ENEMYDUMMY}

PSO.info.enemies = {}

-- build lookup table for stats scaling:
local stat_scaling_res = 100
local stat_scaling_start = 108
local stat_scaling = {stat_scaling_res}

for i = 2, 200
	local start = stat_scaling[i-1] * stat_scaling_start
	stat_scaling[i] = stat_scaling[i-1] * stat_scaling_start / min(stat_scaling_start-1, (stat_scaling_res + (i/10)))
	--print("scale ("..i.."): "..(30*stat_scaling[i]/stat_scaling_res))
end

-- mapload: give stats to all the enemies
PSO.lib.setupEnemies = function()
	--print("setupEnemies")
	-- count players at setup time...
	local numplayers = 0
	for p in players.iterate do
		numplayers = $+1
	end
	
	for m in mobjs.iterate()	-- why does this one need ()
		
		if m and m.valid and m.flags & MF_ENEMY
			PSO.lib.setupEnemy(m, numplayers)
		end
	end
end

PSO.lib.setupEnemy = function(m, numplayers)
	if not numplayers
		numplayers = 0
		for p in players.iterate do
			if not p.bot
				numplayers = $+1
			end
		end	
	end
	
	m.pso = {
			
		maxhp = 100,
		hp = 100,
		satk = 20,
		ratk = 20,
		tatk = 20,
		def = 15,
		exp = 3,
		weakness = ELEM_FIRE,
	}
			
	local lv = 1

	-- get level if applicable
	if server.questdata
				
		local q = PSO.info.quests[server.questdata.map][server.questdata.index]
		lv = q.levels[server.questdata.difficulty] or server.pso.level
	end
			
	-- now apply the base stats and level scaling!
	-- we will also scale HP specifically with playercount
	
	if PSO.info.enemies[m.type]
		local info = PSO.info.enemies[m.type]
				
		m.pso.maxhp = info.bhp *stat_scaling[lv]/stat_scaling_res
		
		if numplayers > 1
			if info.boss
				m.pso.maxhp = $ + m.pso.maxhp*numplayers/3
			else
				m.pso.maxhp = $ + m.pso.maxhp*numplayers/5
			end
		end
		
		m.pso.hp = m.pso.maxhp
		m.pso.satk = info.satk *stat_scaling[lv]/stat_scaling_res
		m.pso.ratk = info.ratk *stat_scaling[lv]/stat_scaling_res
		m.pso.tatk = info.tatk *stat_scaling[lv]/stat_scaling_res
		m.pso.def = (info.def *stat_scaling[lv]/stat_scaling_res)/3
		m.pso.weight = info.weight or 1
		m.pso.exp = (info.exp or $) *stat_scaling[lv]/stat_scaling_res
		m.pso.weakness = info.weakness or $
		m.pso.level = lv
		m.pso.boss = info.boss
		m.pso.waitclear = info.waitclear
		
		-- validate drops!
		if info.drops
			for i = 1, #info.drops
				local d = info.drops[i]
				
				local name = d[1]
				
				if type(name) ~= "number"
					
					if not PSO.info.items[name]
						print("\x82".."WARNING: ".."\x80".."Enemy "..info.name..": Invalid drop \'"..name.."\'")
					end
				end
			end
		end
	end
end

PSO.lib.spawnEnemyFromEnemy = function(ref, x, y, z, newtype)
	local mo = P_SpawnMobj(x, y, z, newtype)
	PSO.lib.setupEnemy(mo)
	
	-- add that enemy in the same room as the ref
	table.insert(server.psorooms[ref.roomid].waves[ref.waveid], mo)
	
	local tin = P_SpawnMobj(mo.x, mo.y, mo.z, MT_PSO_TELEPORTIN)
	tin.target = mo
	
	return mo
end

/*
	Enemy stats:
		
	name:
		Enemy display name
	
	sprite:
		SPR_ sprite to use by default. This can be overwritten at runtime but is used by some default functions.
	
	frame:
		frame to use by default. This can be overwritten at runtime but is used by some default functions.
		
	painframe:
		frame used by default functions for stagger, knockback and blowdown.
		Will fallback to 'frame' if it doesn't exist. Can also be left nil if you overwrite all behaviours.
	
	bhp:
		Enemy max HP at Lv1
	
	satk:
		Enemy striking attack at Lv1
	
	ratk:
		Enemy ranged attack at Lv1
		
	tatk:
		Enemy technique attack at Lv1
	
	def:
		Enemy def at Lv1
	
	exp:
		Enemy EXP at Lv1
		
	weakness:
		Enemy weakness
		
	drops:
		Enemy drop table
		{
			{"item", min. level, max. level, ratelow, ratehigh},
				^ "item" drops if enemy is between min. level and max. level at a rate of ratelow/ratehigh
				
			{"item2", .....},
			{...},
			...
		}
		
	weight:
		Enemy weight class, determines if an enemy can be staggered, knocked back and blown down
		1: Lightweight:
			- Stagger: Yes
			- Knockback: Yes
			- Blowdown: Yes
	
		2: Normal:
			- Stagger: No
			- Knockback -> Stagger
			- Blowdown -> Knockback
			
		3: Heavyweight:
			- Stagger: No
			- Knockback: No
			- Blowdown -> Stagger
		
		4: Boss:
			- Stagger: No
			- Knockback: No
			- Blowdown: No
		
		
	thinker: function(mobj_t mo)
		Function ran each frame, calling mo as the enemy argument.
		This function does not run during stagger, knockback or blowdown.
	
	stagger: function(mobj_t mo)
		Function to call when enemy is staggered. 
		Enemy stagger stops when the function returns true.
		If nil, uses a default behaviour.
		
	knockback: function(mobj_t mo)
		Function to call when enemy is knocked back.
		Enemy knockback ends when the function returns true.
		If nil, uses a default behaviour.
		
	blowdown: function(mobj_t mo)
		Function to call when the enemy is blown down.
		Enemy blowdown ends when the function returns true.
		If nil, uses a default behaviour.
*/	

-- drop items after killing an enemy.
PSO.lib.enemyDropItems = function(mo)
	
	local info = PSO.info.enemies[mo.type]
	local boxdrops
	local boxes
	local drops
	
	if mo.options & 1	-- drop items from boxes exclusively
		--print("mo is a boss")
		-- check if we CAN
		local room = server.psorooms[mo.roomid]
		
		if room and room.boxes and room.boxes[room.wavenum] and #room.boxes[room.wavenum]
			boxes = room.boxes[room.wavenum]
			--print("boxes "..tostring(boxes))
		end
		
		-- then just drop nothing lol
		if not boxes
			return
		end
	end
	
	if info and info.drops
		if boxes
			for i = 1, #boxes
				local box = boxes[i]
				if not (box.options & 4)
					box.drops = PSO.lib.calcDrops(info.drops, mo.pso.level)
				end
				--box.state = S_PSO_CRATE	-- this will make them appear
				--print(box.drops)
				local tpin = P_SpawnMobj(box.x, box.y, box.z, MT_PSO_TELEPORTIN)
				tpin.target = box
				tpin.nopush = true
			end
		else
			drops = PSO.lib.calcDrops(info.drops, mo.pso.level)
		end
	end
	
	-- iterate drops and check for meseta amount, if so, make sure to scale it apropriately.
	if drops
		for k,v in pairs(drops)
			if type(v) == "number"	-- meseta
				--local mult = FixedMul( FixedSqrt((mo.pso.level or 1)<<FRACBITS), FixedSqrt((mo.pso.maxhp or 1)<<FRACBITS) /FRACUNIT
				local mult = (mo.pso.level or 1) * max(1, (mo.pso.maxhp)/8)
				local meseta = (mult/3)*v/100
				meseta = $ + P_RandomRange(-meseta/10, meseta/10)	-- variance
				PSO.lib.dprint(meseta)
				drops[k] = meseta
			end
		end
	
		local drop = P_SpawnMobj(mo.x, mo.y, mo.z + mo.height/2, MT_PSO_DROP)
		local ang = P_RandomRange(0, 359)*ANG1
		--P_InstaThrust(drop, ang, P_RandomRange(0, 4)*drop.scale)
		drop.momz = drop.scale*8
		drop.psoitems = drops
		
		if consoleplayer
			local mydrop = drop.psoitems[#consoleplayer+1]
			if mydrop
			and type(mydrop) ~= "number"
				local minfo = PSO.lib.itemInfo(mydrop)
				if minfo.rarity >= 7
					PSO.lib.startRareDropAnim(consoleplayer, minfo.rarity)
				end
			end
		end
	end
end

-- @TODO:
-- Manage the enemy's target depending on aggro.
-- Aggro changes depending on damage dealt and distance
PSO.lib.enemyManageAggro = function(mo)
	
	local bestdist = 4096*mo.scale
	
	if not mo.target or not mo.target.valid
	or R_PointToDist2(mo.x, mo.y, mo.target.x, mo.target.y) > bestdist
	or (mo.target.player and mo.target.player.pso and mo.target.player.pso.hp <= 0)	-- blud is dead
	or leveltime % (TICRATE*10) == 0
		
		mo.target = nil
		for p in players.iterate do
			if p.mo and p.mo.valid
			and p.pso
			and p.pso.hp
			and p.pso.type_initialized	-- don't attempt to target new joiners, they're in limbo.
				
				local dist = R_PointToDist2(mo.x, mo.y, p.mo.x, p.mo.y)
				if dist < bestdist
					mo.target = p.mo
					bestdist = dist
				end
			end
		end
	end
end

PSO.lib.enemyStaggerDefault = function(mo)
	
	local info = PSO.info.enemies[mo.type]
	local stagger_duration = info.stagger_duration or TICRATE/3
	
	mo.rollangle = 0
	mo.sprite = info.sprite
	if info.painframe ~= nil
		mo.frame = info.painframe
	else
		mo.frame = info.frame or A
	end
	
	if mo.pso.stagger == 1
		mo.momz = mo.scale*2
	end
	
	mo.pso.stagger = $+1
	if mo.pso.stagger > stagger_duration
		mo.pso.stagger = 0
	end
end

PSO.lib.enemyKnockbackDefault = function(mo)
	
	mo.pso.knockback = $+1
	mo.rollangle = 0
	
	if not P_IsObjectOnGround(mo)
		mo.momx = FixedMul(mo.momx, mo.friction)
		mo.momy = FixedMul(mo.momy, mo.friction)
	else
		local rad = mo.radius/FRACUNIT
		local d = P_SpawnMobj(mo.x + P_RandomRange(-rad, rad)*FRACUNIT, mo.y + P_RandomRange(-rad, rad)*FRACUNIT, mo.z, MT_SPINDUST)
		d.momz = P_RandomRange(1, 4)*mo.scale
	end
	
	mo.momx = FixedMul(mo.momx, mo.friction)
	mo.momy = FixedMul(mo.momy, mo.friction)
	
	if FixedHypot(mo.momx, mo.momy) < mo.scale
		mo.pso.knockback = 0
	end
	
end

PSO.lib.enemyBlowdownDefault = function(mo)
	
	mo.pso.blowdown = $+1
	
	if not P_IsObjectOnGround(mo)
		mo.blowdownmomz = mo.momz
		mo.rollangle = $ + ANG1*16
		mo.z = $ + mo.momz
	elseif mo.pso.blowdown > 4
		
		if FixedHypot(mo.momx, mo.momy) < mo.scale*4
			mo.pso.blowdown = nil
			mo.rollangle = 0
		else
		
			mo.momz = -mo.blowdownmomz/3
			mo.momx = $/2
			mo.momy = $/2
			
			for i = 1, 8
				local d = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SPINDUST)
				d.angle = (360/8)*i*ANG1
				P_InstaThrust(d, d.angle, mo.blowdownmomz*3)
				d.momz = -mo.blowdownmomz/3
			end	
		end
	end
end

-- calls enemy thinkers and other things

local status_funcs = {
	{"stagger", PSO.lib.enemyStaggerDefault},
	{"knockback", PSO.lib.enemyKnockbackDefault},
	{"blowdown", PSO.lib.enemyBlowdownDefault},
}

PSO.lib.enemyThinker = function(mo)
	
	local info = PSO.info.enemies[mo.type]
	if info and mo.pso.hp and mo.health
		
		local ran_reset
		
		for i = 1, #status_funcs
			local s = status_funcs[i]
			if mo.pso[s[1]]
				
				if not ran_reset
					if info.reset
						local run, res = pcall(info.reset, mo)

						if not run
							print("\x82".."WARNING".."\x80"..": enemyThinker (reset): "..res) 
						end
					end
				end
				
				local fn = info[s[1]] or s[2]
				local run, res = pcall(fn, mo)
					
				if not run
					print("\x82".."WARNING".."\x80"..": enemyThinker ("..s[1].."): "..res) 
				end
					
				return	
			end
		end
		
		if info.thinker
			local run, res = pcall(info.thinker, mo)
			
			if not run
				print("\x82".."WARNING".."\x80"..": enemyThinker: "..res) 
			end
		end
		
		-- bosses and music: play music for players that get near us!
		if info.boss and info.bossmusic
			
			local myroom = mo.roomid
			local room = server.psorooms[myroom]
			
			if room
			
				mo.playersmusic = $ or {}
				for p in players.iterate do
					
					if p.mo and p.mo.valid and p.pso
					
						if not mo.playersmusic[p.mo]
							if p.pso.hp
							and not p.pso.teleport	-- medbay tp
							and PSO.lib.objectInRoom(room, p.mo)
								if mapheaderinfo[gamemap].bossrush
									S_ChangeMusic(info.bossrushmusic, true, p)
								else
									S_ChangeMusic(info.bossmusic, true, p, 0, 0, 300)
								end
								mo.playersmusic[p.mo] = true
								
								-- start the emergency!
								PSO.lib.startEmergency(p, "EMERGENCY CODE", "(DUEL)", sfx_emrgy, sfx_cduel, TICRATE*3/2)
								
							end
						else
							
							if p.pso.hp <= 0
								mo.playersmusic[mo] = nil
							end
						end
					end
				end
				
				-- check if players have died or smth
				
			end
		end
	end
end

-- part creator:
PSO.lib.enemyCreatePart = function(source, x, y, z, parttype)
	
	local part = P_SpawnMobj(x, y, z, parttype)
	part.pso = {}
	
	for k,v in pairs(source.pso)
		part.pso[k] = v
	end
	
	if PSO.info.enemies[parttype]
	and PSO.info.enemies[parttype].part
		--print("B")
		part.pso.maxhp = max(1, source.pso.maxhp * PSO.info.enemies[parttype].bhp / 100)
		part.pso.hp = part.pso.maxhp
		part.pso.exp = 0
		part.pso.part = true
		if PSO.info.enemies[parttype].partlinkdamage
			--print("links damage")
			part.pso.partlinkdamage = true
		end
	end
	
	part.tracer = source
	
	source.parts = $ or {}
	source.parts[#source.parts+1] = part
	
	return part
end

-- enemies shouldn't collide between one another
local function enemy_collide(mo1, mo2)
	if mo1 and mo1.valid and mo2 and mo2.valid
	and mo1.pso	
		if mo1.flags & MF_ENEMY and mo2.flags & MF_ENEMY
		or (mo1.pso.knockback or mo1.pso.blowdown) and mo2.player
			return false
		end
	end
end

addHook("MobjCollide", enemy_collide)
addHook("MobjMoveCollide", enemy_collide)


PSO.lib.positionInvalid = function(mo, tx, ty)
	local blocked
	local function blockcheck(mo, mo2)
		if mo2 ~= mo and mo2 and mo2.valid and mo2.flags & MF_SOLID
		and mo2.type ~= MT_PLAYER	-- ramming this is always fine :)
		and not (mo2.flags & MF_ENEMY) 
								
			if tx >= mo2.x - mo2.radius
			and tx <= mo2.x + mo2.radius
			and ty >= mo2.y - mo2.radius
			and ty <= mo2.y + mo2.radius
			and mo.z >= mo2.z
			and mo.z <= mo2.z+mo.height
				blocked = true
			end
		end	
	end
						
	local x1 = mo.x - 512*FRACUNIT
	local x2 = mo.x + 512*FRACUNIT
	local y1 = mo.y - 512*FRACUNIT
	local y2 = mo.y + 512*FRACUNIT
						
	searchBlockmap("objects", blockcheck, mo, x1, x2, y1, y2)
	
	return blocked
end

PSO.lib.enemyMovementInvalid = function(mo)
	
	local spd = FixedHypot(mo.momx, mo.momy)
	local a = R_PointToAngle2(0, 0, mo.momx, mo.momy)
	if not spd return false end
	
	local bspd = spd
	
	for i = 1, 4
		
		local tx = mo.x + FixedMul(spd, cos(a))
		local ty = mo.y + FixedMul(spd, sin(a))
		
		if PSO.lib.positionInvalid(mo, tx, ty)
			return true
		end
		spd = $+bspd
	end
	
	-- if we made it here the movement wasn't blocked.
	return false
end

-- enemy push hitbox: pushes the player following momentum or pushes them away
freeslot("MT_PSO_PUSHBOX")
mobjinfo[MT_PSO_PUSHBOX] = {
	doomednum = -1,
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	radius = 16<<FRACBITS,
	height = 32<<FRACBITS,
	flags = MF_NOGRAVITY|MF_SOLID,
}

local reactivation_delay = 5
addHook("MobjThinker", function(mo)
	
	if not mo.target or not mo.target.valid
		P_RemoveMobj(mo)
		return
	end
	
	P_MoveOrigin(mo, mo.target.x, mo.target.y, mo.target.z)
	mo.momx = mo.target.momx
	mo.momy = mo.target.momy
	mo.momz = mo.target.momz
	
	mo.radius = mo.target.radius	--*11/10
	mo.height = mo.target.height
	
	if mo.target.disablepushbox
		mo.delay = reactivation_delay
		mo.flags = $ & ~MF_SOLID
	elseif mo.delay
		mo.delay = $-1
		if not mo.delay
			mo.flags = $|MF_SOLID
		end
	end
	
	--print(mo.radius/FRACUNIT)
	
	--P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
	
	mo.tics = 2
end, MT_PSO_PUSHBOX)

local function pushbox_collide(mo1, mo2)
	
	if not mo1.target or not mo1.target.valid return false end
	if mo1.target.disablepushbox return false end
	
	if mo2.type != MT_PLAYER return false end
	
	if mo2.player.pso.blowdown or mo2.player.pso.stagger or mo2.player.pso.knockback return false end
end

addHook("MobjCollide", pushbox_collide, MT_PSO_PUSHBOX)
addHook("MobjMoveCollide", pushbox_collide, MT_PSO_PUSHBOX)

local function pushbox_blocked(mo1, mo2)
	if not mo2 or not mo2.valid return end	-- line block
	
	-- push mo2 the fuck out
	mo2.momx = mo1.momx
	mo2.momy = mo2.momy
	return true
end

addHook("MobjMoveBlocked", pushbox_blocked, MT_PSO_PUSHBOX)

local function player_enemyTopCollide(mo, mo2)
	
	if mo2 and mo2.valid and mo2.flags & MF_ENEMY
		
		if mo.z >= mo2.z+mo2.height/2	-- standing on it?
		and mo.z <= mo2.z+mo2.height*3/2
		and (mo.momz < 0 or (mo.momz == 0 and mo.z >= mo2.z+mo2.height))
		-- falling or standing above the hitbox
			P_InstaThrust(mo, mo2.angle, FRACUNIT*16)
			return false
		end
	end
end

addHook("MobjCollide", player_enemyTopCollide, MT_PLAYER)
addHook("MobjMoveCollide", player_enemyTopCollide, MT_PLAYER)