-- IceStabber
/*
	Peaks area boss
	
	Uses a series of stabs, jumps, and projectile attacks.
	Some attacks may freeze players
	
*/

freeslot("MT_PSO_ICESTABBER", "S_PSO_ICESTABBER", "S_PSO_ICESTABBER_DEAD", "SPR_ISTC", "S_PSO_ICESTABBER_SLASH")
freeslot("sfx_sparks")

mobjinfo[MT_PSO_ICESTABBER] = {
	doomednum = 2552,
	spawnhealth = 1000,
	deathstate = S_FACESTABBER_DIE1,	-- todo: change
	spawnstate = S_THOK,
	height = 120<<FRACBITS,
	radius = 50<<FRACBITS,
	flags = MF_ENEMY|MF_SOLID|MF_SHOOTABLE,
}

freeslot("MT_PSO_ICESTABBER_NABARTA")
mobjinfo[MT_PSO_ICESTABBER_NABARTA] = {
	doomednum = -1,
	spawnhealth = 1000,
	spawnstate = S_THOK,
	deathstate = S_NULL,
	height = 128<<FRACBITS,
	radius = 32<<FRACBITS,
	flags = MF_RUNSPAWNFUNC,
}

addHook("MobjThinker", function(mo)
	if not mo.cvmem
		mo.sprite = SPR_USPK
		mo.frame = A
		mo.colorized = true
		mo.color = SKINCOLOR_CYAN
		mo.rollangle = ANG1*180
		mo.momz = -FRACUNIT*32
		mo.tics = -1
		
		if P_IsObjectOnGround(mo)
			mo.cvmem = 1
		end
	else
		
		if mo.cvmem == 1
			
			mo.rollangle = 0
			mo.sprite = SPR_FHZI
			mo.scale = 1
			mo.destscale = FRACUNIT
			mo.scalespeed = FRACUNIT/3
			mo.frame = A
			mo.fuse = TICRATE*8
			mo.tics = mo.fuse
			mo.colorized = false		
			mo.spriteyscale = $*4 + P_RandomRange(-FRACUNIT/2, FRACUNIT/2 -1)
		end
		
		mo.cvmem = $+1
		
		mo.frame = $ & FF_FRAMEMASK
		if mo.fuse < 10
			local tflag = 10 - mo.fuse
			mo.frame = $| (tflag<<FF_TRANSSHIFT)
		end

		if mo.cvmem % 10 == 0
			local hb = PSO.lib.spawnHurtBox(mo.x, mo.y, mo.z - mo.scale*4, 64<<FRACBITS, 256<<FRACBITS, mo.target, 300, DMT_TECHNIQUE, 10, RT_BLOWDOWN)
			hb.pso.reactionradialangle = true
			hb.pso.reactionstrength = 14		
		end
	end
end, MT_PSO_ICESTABBER_NABARTA)

function A_IcestabberSlash(actor)
	
	local potency = 250
	
	if actor.extravalue1
	and actor.target and actor.target.valid
	and leveltime%3 == 0
		for i = 1, 2
			
			local ang = actor.angle + ANG1*90 + ANG1*180*i
			
			local msl = PSO.lib.spawnMissile(actor.target, MT_BARTA, ang, 12*FRACUNIT, potency, DMT_TECHNIQUE, 0, 0, 0)
			P_MoveOrigin(msl, actor.x, actor.y, actor.floorz)
			
			P_InstaThrust(msl, msl.angle, FRACUNIT*20)	
			msl.pso.element = ELEM_ICE
			msl.fuse = TICRATE*5
		end
	end
end

states[S_PSO_ICESTABBER_SLASH] = {SPR_ISTC, A, 1, A_IcestabberSlash, 0, 0, S_PSO_ICESTABBER_SLASH}

for i = 1, 10 do
	freeslot("S_ICESPARK"..i)
end

for i = 1, 10 do
	states[S_ICESPARK1+(i-1)] = {SPR_ISTC, (i)|FF_FULLBRIGHT, 1, nil, 0, 0, i < 9 and S_ICESPARK1+i or S_NULL}
end

local ISS_NEUTRAL = 0
local ISS_LEAP = 1
local ISS_STAB = 2
local ISS_SLASH = 3
local ISS_ICICLES = 4

local ISS_RAIN = 5

local function is_neutral(mo)
	return not mo.isstate
end

local function is_movement(mo)
	local speed_closein = FRACUNIT*10
	local melee_dist = FRACUNIT*512

	-- do different actions depending on how far the player is
	local dist = R_PointToDist2(mo.x, mo.y, mo.target.x, mo.target.y)
				
	local movespeed = (dist - melee_dist*3/4) / 128
	if movespeed < 0	-- make it faster when backpedalling
		movespeed = $*2
	end
	-- calculate movespeed based on distance to player
	-- clamp it between the target speeds
	movespeed = min($, speed_closein)
	movespeed = max($, -speed_closein)				

	mo.angle = R_PointToAngle2(mo.x, mo.y, mo.target.x, mo.target.y)
	P_InstaThrust(mo, mo.angle, movespeed)
	
	mo.frame = leveltime % (G*2) /2
end

local function is_lowhp(mo)
	return mo.pso.hp <= mo.pso.maxhp*3/4 or PSO.lib.questDifficulty() >= DIFF_HARD
end

local function is_preprain(mo)
	mo.dorain = 1
	mo.isstate = ISS_RAIN
end

-- prepares a leap
-- above half hp, the leap is stationary
-- otherwise, targets the player

local function is_prepleap(mo)
	
	local leap_time = TICRATE*2
	
	mo.doleap = 1
	mo.leapcount = 1
	mo.leapend = 0
	mo.leapfailsafe = 0
	
	mo.isstate = ISS_LEAP
	
	-- set target
	if mo.doleap == 1
					
		-- leap towards target and knock everyone around back
		if is_lowhp(mo)
			for p in players.iterate 
				if p and p.mo and p.mo.valid
				and R_PointToDist2(p.mo.x, p.mo.y, mo.x, mo.y) < 320*p.mo.scale
								
					local ang = R_PointToAngle2(mo.x, mo.y, p.mo.x, p.mo.y)
					local dist = R_PointToDist2(p.mo.x, p.mo.y, mo.x, mo.y)
					-- magic numbers but they work for now and there's no need to change that...
					P_InstaThrust(p.mo, ang, 20*p.mo.scale - (dist/17))
				end
			end

			for i = 1, 8
				local dust = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SPINDUST)
				dust.angle = (360/8)*i*ANG1
				P_InstaThrust(dust, dust.angle, FRACUNIT*24)
			end
						
			mo.angle = R_PointToAngle2(mo.x, mo.y, mo.target.x, mo.target.y)
			local ldist = R_PointToDist2(mo.x, mo.y, mo.target.x, mo.target.y)
						
			P_InstaThrust(mo, mo.angle, ldist / (leap_time*3/4))
			mo.leaptarg = {mo.target.x, mo.target.y}
		else
			mo.leaptarg = {mo.x, mo.y}
			P_InstaThrust(mo, 0, 0)
		end
		mo.momz = FRACUNIT*32
		mo.doleap = 2
					
		S_StartSound(mo, sfx_beflap)
	end
end

local function is_prepslash(mo)
	
	mo.isstate = ISS_SLASH
	mo.slashtimer = 1
	
	P_InstaThrust(mo, 0, 0)
	mo.slashpos = {mo.x, mo.y, mo.z}
	
	S_StartSound(mo, sfx_s3k80)
end


local function is_prepstab(mo)
	
	mo.isstate = ISS_STAB
	mo.stabtimer = 1
	
	S_StartSound(mo, sfx_s3kc5s)
	P_InstaThrust(mo, 0, 0)	
end

local function is_prepicicles(mo)
	mo.isstate = ISS_ICICLES
	mo.icicletimer = 1
	mo.flags = $ & ~MF_NOGRAVITY
	S_StartSound(mo, sfx_s3k60)
end

local function ls_doleap(mo)

	local gibarta_potency = 300
	local barta_potency = 200
	local diff = PSO.lib.questDifficulty()
				
	-- reach target
	if mo.doleap == 2
			
		if leveltime & 1
			P_SpawnGhostMobj(mo)
		end
					
		mo.momz = max(0, $)
		
		local minspeed = max(FixedHypot(mo.momx, mo.momy)*8, FRACUNIT*24)
		
		mo.leapfailsafe = $+1
		
		if R_PointToDist2(mo.x, mo.y, mo.leaptarg[1], mo.leaptarg[2]) < minspeed
		and mo.momz <= 0
		or (mo.leapfailsafe and mo.leapfailsafe > TICRATE*2)
			mo.doleap = 3
			mo.leapfailsafe = nil
			P_InstaThrust(mo, 0, 0)
		end
				
	-- SLAM!!!
	elseif mo.doleap == 3
					
		mo.momz = -32*FRACUNIT
					
		if P_IsObjectOnGround(mo)
			local hb = PSO.lib.spawnHurtBox(mo.x, mo.y, mo.z - mo.scale*4, 320<<FRACBITS, 40<<FRACBITS, mo, leap_potency, DMT_STRIKE, 8, RT_BLOWDOWN)
			hb.pso.reactionradialangle = true
			hb.pso.reactionstrength = 14
						
			for i = 1, 3
				local dist = 64*i
				
				for k = 1, 16
					local ang = (360/16)*k*ANG1
					local x = mo.x + dist*cos(ang)
					local y = mo.y + dist*sin(ang)
					
					local e = P_SpawnMobj(x, y, mo.z, MT_THOK)
					e.sprite = SPR_FHZI
					e.scale = 1
					e.destscale = FRACUNIT
					e.scalespeed = FRACUNIT/2 /i
					e.frame = A
					e.fuse = TICRATE/3 + i*8
					e.tics = e.fuse
					
					if i == 3
						e.spriteyscale = $*4 + P_RandomRange(-FRACUNIT/2, FRACUNIT/2 -1)
					end
				end
			end

			-- gibarta
			local h = PSO.lib.spawnHurtBox(mo.x, mo.y, mo.z, 355<<FRACBITS, 128<<FRACBITS, mo, gibarta_potency, DMT_TECHNIQUE, 4, RT_BLOWDOWN)
			h.pso.whitelist = $ or {}
			h.pso.reactionradialangle = true
			h.pso.element = ELEM_ICE
			
			-- bartas
			local numbartas = 8
			
			if is_lowhp(mo)
				numbartas = 24
			end
			
			if diff >= DIFF_HARD
				numbartas = $*2
			end
			
			for i = 1, numbartas
				local a = 360/numbartas*i *ANG1
				local msl = PSO.lib.spawnMissile(mo, MT_BARTA, a, 12*FRACUNIT, barta_potency, DMT_TECHNIQUE, 0, 0, 0)
				P_InstaThrust(msl, msl.angle, FRACUNIT*20)	
				msl.pso.element = ELEM_ICE
				msl.fuse = TICRATE*5
			end
			
			S_StartSound(mo, sfx_pstop)
						
			mo.doleap = 4
		end
	elseif mo.doleap == 4
		
		mo.leapend = $+1
		if mo.leapend >= TICRATE*2
			
			mo.doleap = nil
			if is_lowhp(mo)
			and not mo.followup
				if diff >= DIFF_HARD
					is_preprain(mo)
				else
					is_prepslash(mo)
				end
				mo.followup = true
			else
				mo.isstate = ISS_NEUTRAL
			end
		end	
		
	end
end


local function is_doslash(mo)
	
	local slash_charge = TICRATE
	local slash_swing = 2
	local slash_wait = TICRATE*3/4
	
	local slash_potency = 350
	
	if mo.slashtimer
		
		mo.slashtimer = $+1
		
		if mo.slashtimer < slash_charge
			
			mo.frame = I
			
			mo.flags = $|MF_NOGRAVITY|MF_NOCLIPHEIGHT
			P_MoveOrigin(mo, mo.slashpos[1] + P_RandomRange(8, 8)*FRACUNIT, mo.slashpos[2] + P_RandomRange(8, 8)*FRACUNIT, mo.slashpos[3] + P_RandomRange(8, 8)*FRACUNIT)
		elseif mo.slashtimer < slash_charge  + slash_wait
			
			if mo.slashtimer == slash_charge
				
				S_StartSound(mo, sfx_s3ka8)
				
				local a = R_PointToAngle2(mo.x, mo.y, mo.target.x, mo.target.y)
				local msl = PSO.lib.spawnMissile(mo, MT_ENERGYBALL, a, 21*FRACUNIT, slash_potency, DMT_TECHNIQUE, 0, 0, 0)
				--P_InstaThrust(msl, msl.angle, FRACUNIT*20)
				
				if is_lowhp(mo)
				and not mo.followup
					msl.extravalue1 = 1
					msl.state = S_PSO_ICESTABBER_SLASH
				else
					msl.sprite = SPR_ISTC
					msl.frame = A
					msl.tics = -1
				end
				
				msl.color = SKINCOLOR_ICY
				msl.scale = $*3/2
				msl.flags = $|MF_NOGRAVITY
				msl.pso.element = ELEM_ICE
				msl.fise = TICRATE*8
				
				PSO.lib.aimMissile(msl, mo.target)
			end
			
			if mo.slashtimer < slash_charge + slash_swing
				mo.frame = J
			end
			
			P_MoveOrigin(mo, mo.slashpos[1], mo.slashpos[2], mo.slashpos[3])
			mo.flags = $ & ~(MF_NOGRAVITY|MF_NOCLIPHEIGHT)
		else
			mo.isstate = nil
			mo.slashtimer = 0
		end
		
	end
end

local function is_dostab(mo)
	
	local windup = TICRATE
	local charge_duration = TICRATE*3
	local charge_speed = FRACUNIT*40
	local charge_potency = 450
	local barta_potency = 350
	
	if mo.stabtimer
		
		mo.stabtimer = $+1
		mo.frame = H
		
		if mo.stabtimer < windup
			P_InstaThrust(mo, mo.angle, -FRACUNIT*3)
		
		elseif mo.stabtimer == windup
		
			local hx = mo.x + 32*cos(mo.angle)
			local hy = mo.y + 32*sin(mo.angle)

			local hb = PSO.lib.spawnHurtBox(hx, hy, mo.z, 128<<FRACBITS, 40<<FRACBITS, mo, charge_potency, DMT_STRIKE, charge_duration, RT_BLOWDOWN)
			mo.hitbox = hb
			hb.angle = mo.angle
			hb.pso.reactionstrength = 10
			
			if mo.followup
				
				local targetx = mo.target.x + mo.target.momx*TICRATE*3/4
				local targety = mo.target.y + mo.target.momy*TICRATE*3/4
				
				mo.angle = R_PointToAngle2(mo.x, mo.y, targetx, targety)
				
			end
		else
			
			if leveltime & 1
				P_SpawnGhostMobj(mo)
			end
			P_InstaThrust(mo, mo.angle, charge_speed)

			if is_lowhp(mo)
				local e = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
				e.sprite = SPR_FHZI
				e.scale = 1
				e.destscale = FRACUNIT
				e.scalespeed = FRACUNIT/4
				e.frame = A
				e.fuse = TICRATE*2
				e.tics = e.fuse
				e.spriteyscale = $*4 + P_RandomRange(-FRACUNIT/2, FRACUNIT/2 -1)	

				local hb = PSO.lib.spawnHurtBox(e.x, e.y, e.z, 80<<FRACBITS, 256<<FRACBITS, mo, barta_potency, DMT_TECHNIQUE, e.fuse, RT_STAGGER)				
			end
			
			if mo.hitbox and mo.hitbox.valid

				local hx = mo.x + 32*cos(mo.angle)
				local hy = mo.y + 32*sin(mo.angle)
				P_MoveOrigin(mo.hitbox, hx, hy, mo.z)
				
			end
			
			local frontx = mo.x + 256*cos(mo.angle)
			local fronty = mo.y + 256*sin(mo.angle)
			local tester = P_SpawnMobj(frontx, fronty, mo.z, MT_THOK)
			tester.state = S_INVISIBLE
			
			if not P_CheckSight(mo, tester)
			and mo.stabtimer < windup + charge_duration
				mo.stabtimer = windup + charge_duration
			end
			
			if mo.stabtimer > windup + charge_duration
				
				mo.stabtimer = 0
				if is_lowhp(mo)
				and not mo.followup
					is_prepleap(mo)
					mo.followup = true
				else
					
					mo.momx = $/4
					mo.momy = $/4
					mo.isstate = ISS_NEUTRAL
					
				end
			end
		end
	end
end

local function is_doicicles(mo)
	
	local num_icicles = 3
	local icicle_start = TICRATE/3
	local icicle_delay = TICRATE/2
	local barta_potency = 350
	
	if mo.icicletimer
		
		if mo.icicletimer == 1
			
			-- if we can't jump backwards, do so forwards and higher!
			local tx = mo.x - 192*cos(mo.angle)
			local ty = mo.y - 192*sin(mo.angle)
			local tester = P_SpawnMobj(tx, ty, mo.z, MT_THOK)
			tester.state = S_INVISIBLE
			
			mo.z = $+8	-- so that we don't touch the ground
			if not P_CheckSight(mo, tester)
				mo.momz = FRACUNIT*20
				P_InstaThrust(mo, mo.angle, FRACUNIT*10)				
			else
				mo.momz = FRACUNIT*10
				P_InstaThrust(mo, mo.angle, -FRACUNIT*18)
			end
			mo.icicletimer = $+1
		end
		
		if P_IsObjectOnGround(mo)
			
			if mo.icicletimer == 2
			and not P_RandomRange(0, 4)
				is_prepslash(mo)
				mo.icicletimer = nil
				return
			end
			
			P_InstaThrust(mo, 0, 0)
			mo.icicletimer = $+1
			mo.frame = H
			
			mo.angle = R_PointToAngle2(mo.x, mo.y, mo.target.x, mo.target.y)
			
			if mo.icicletimer >= icicle_start
			and mo.icicletimer <= icicle_start + icicle_delay*(num_icicles)
				local curr = mo.icicletimer - icicle_start
				
				if curr%icicle_delay == 0
					local msl = PSO.lib.spawnMissile(mo, MT_BARTA, mo.angle, 12*FRACUNIT, barta_potency, DMT_TECHNIQUE, 0, 0, 0)
					P_InstaThrust(msl, msl.angle, FRACUNIT*26)	
					msl.pso.element = ELEM_ICE
					S_StartSound(mo, sfx_barta)
					msl.fuse = TICRATE*5
				end
			end
			
			if mo.icicletimer > icicle_start + icicle_delay*num_icicles + TICRATE/2
				
				if is_lowhp(mo)
				and not mo.followup
					is_prepstab(mo)
					mo.followup = true
				else
					mo.isstate = ISS_NEUTRAL
				end
				mo.icicletimer = 0
			end
		end
		
	end
	
end

local function is_dorain(mo)
	
	local rain_charge = TICRATE
	local rain_swing = 2
	local rain_duration = TICRATE*6
	local rain_wait = TICRATE*3/4
	
	local rain_potency = 350
	
	if mo.dorain
		
		mo.dorain = $+1
		
		if mo.dorain < rain_charge
			
			mo.frame = I
			if leveltime%4 < 2
				mo.color = SKINCOLOR_WHITE
			end
			
		elseif mo.dorain < rain_charge  + rain_duration + rain_wait
			
			if mo.dorain >= rain_charge
			and mo.dorain < rain_charge + rain_duration
				
				if mo.dorain == rain_charge
					for i = 1, 16
						local d = P_SpawnMobj(mo.target.x, mo.target.y, mo.target.z, MT_SPINDUST)
						d.colorized = true
						d.color = SKINCOLOR_CYAN
						P_InstaThrust(d, 360/16 *i*ANG1, FRACUNIT*24)
					end
				end
				
				if mo.dorain%4 == 0
					local msl = PSO.lib.spawnMissile(mo, MT_PSO_ICESTABBER_NABARTA, a, 0, rain_potency, DMT_TECHNIQUE, 0, 0, 0)
					msl.momx = 0
					msl.momy = 0
					P_MoveOrigin(msl, mo.target.x, mo.target.y, mo.target.z + FRACUNIT*768)
					
					S_StartSound(mo.target, sfx_barta)
				end
			end
			
			if mo.dorain < rain_charge + rain_swing
				mo.frame = J
			end
		else
			mo.isstate = nil
			mo.dorain = 0
		end
		
	end
end

local function is_showoff(mo)
	
	if not mo.introd
		mo.introtimer = $ and $+1 or 1
		
		for p in players.iterate do
		
			local pmo = p.mo
			if pmo and pmo.valid and R_PointToDist2(mo.x, mo.y, pmo.x, pmo.y) < FRACUNIT*1536
				if mo.introtimer == TICRATE*3
					p.powers[pw_nocontrol] = 0
				else
					p.powers[pw_nocontrol] = 32767
				end
			end
		end
		
		if mo.introtimer == TICRATE/2
			P_StartQuake(FRACUNIT*30, TICRATE/2)
			S_StartSound(mo, sfx_barta)
			S_StartSound(mo, sfx_s3k80)

			for i = 1, 3
				local dist = 64*i
				
				for k = 1, 16
					local ang = (360/16)*k*ANG1
					local x = mo.x + dist*cos(ang)
					local y = mo.y + dist*sin(ang)
					
					local e = P_SpawnMobj(x, y, mo.z, MT_THOK)
					e.sprite = SPR_FHZI
					e.scale = 1
					e.destscale = FRACUNIT
					e.scalespeed = FRACUNIT/2 /i
					e.frame = A
					e.fuse = TICRATE/3 + i*8
					e.tics = e.fuse
					
					if i == 3
						e.spriteyscale = $*4 + P_RandomRange(-FRACUNIT/2, FRACUNIT/2 -1)
					end
				end
			end
		end
			
			
		if mo.introtimer >= TICRATE/2
		and mo.introtimer < TICRATE
			mo.frame = I
			--print("I")
		elseif mo.introtimer >= TICRATE
		and mo.introtimer <= TICRATE+2
			mo.frame = J
			--print("J")
		elseif mo.introtimer > TICRATE+2
			mo.frame = H
			--print("H")
		end
			
		if mo.introtimer == TICRATE+4
				
			local ang = mo.angle - ANG1*30
			local dist = 80
				
			local sx = mo.x + dist*cos(ang)
			local sy = mo.y + dist*sin(ang)
			local sz = mo.z + FRACUNIT*30
				
			local s = P_SpawnMobj(sx, sy, sz, MT_THOK)
			s.state = S_ICESPARK1
			s.color = SKINCOLOR_WHITE
			s.scale = FRACUNIT*3
			
			S_StartSound(mo, sfx_sparks)
		end	
		
		if mo.introtimer == TICRATE*3
			mo.introd = true
		end
	end	
end

PSO.info.enemies[MT_PSO_ICESTABBER] = {
		
		name = "Ice Stabber",
		sprite = SPR_CBFS,
		frame = A,
		painframe = A,
		
		bhp = 4500,
		satk = 35,
		ratk = 35,
		tatk = 35,
		def = 25,
		exp = 300,
		weight = 4,

		boss = true,		-- enemy is a major boss!
		bossmusic = "BOSS3",		
		bossrushmusic = "BOSSR3",
		
		
		drops = {
			-- Lv1-29
			{"pine", 1, 29, 1, 100},
			{"claymore", 1, 29, 1, 100},
			{"alva pine", 1, 29, 1, 100},
			{"alva claymore", 1, 29, 1, 100},
			
			{"doubliss", 1, 29, 1, 100},
			{"doublecutter", 1, 29, 1, 100},
			{"alva doubliss", 1, 29, 1, 100},
			{"alva doublecutter", 1, 29, 1, 100},	
			
			{"shooter", 1, 29, 1, 100},
			{"burster", 1, 29, 1, 100},
			{"alva shooter", 1, 29, 1, 100},
			{"alva burster", 1, 29, 1, 100},				
			
			{"shot", 1, 29, 1, 100},
			{"concentrator", 1, 29, 1, 100},	
			{"alva shot", 1, 29, 1, 100},
			{"alva concentrator", 1, 29, 1, 100},	
			
			{"crest", 1, 29, 1, 100},
			{"crescent", 1, 29, 1, 100},
			{"alva crest", 1, 29, 1, 100},
			{"alva crescent", 1, 29, 1, 100},
				
			{"claw", 1, 29, 1, 100},
			{"cutter", 1, 29, 1, 100},
			{"alva claw", 1, 29, 1, 100},
			{"alva cutter", 1, 29, 1, 100},			
			
			{"beamgun", 1, 29, 1, 100},
			{"peacekeeper", 1, 29, 1, 100},
			{"alva beamgun", 1, 29, 1, 100},
			{"alva peacekeeper", 1, 29, 1, 100},			
					
			{"hammer", 1, 29, 1, 100},
			{"scepter", 1, 29, 1, 100},
			{"alva hammer", 1, 29, 1, 100},
			{"alva scepter", 1, 29, 1, 100},
			
			{"icicle frame", 1, 29, 1, 20},
			{"knight frame", 1, 29, 1, 20},
			
			{"unit_hp_2", 1, 29, 2, 50},
			{"unit_pp_2", 1, 29, 2, 50},
			{"unit_satk_2", 1, 29, 2, 50},
			{"unit_tatk_2", 1, 29, 2, 50},
			{"unit_ratk_2", 1, 29, 2, 50},
			
			{"monogrinder", 1, 29, 5, 50},
			{"digrinder", 1, 29, 5, 50},
			
			-- Lv30-50
			{"alva pine", 30, 50, 1, 100},
			{"alva claymore", 30, 50, 1, 100},
			{"vita pine", 30, 50, 1, 100},
			{"vita claymore", 30, 50, 1, 100},
			
			{"alva doubliss", 30, 50, 1, 100},
			{"alva doublecutter", 30, 50, 1, 100},	
			{"vita doubliss", 30, 50, 1, 100},
			{"vita doublecutter", 30, 50, 1, 100},	
			
			{"vita shooter", 30, 50, 1, 100},
			{"vita burster", 30, 50, 1, 100},
			{"alva shooter", 30, 50, 1, 100},
			{"alva burster", 30, 50, 1, 100},				
			
			{"vita shot", 30, 50, 1, 100},
			{"vita concentrator", 30, 50, 1, 100},	
			{"alva shot", 30, 50, 1, 100},
			{"alva concentrator", 30, 50, 1, 100},	
			
			{"vita crest", 30, 50, 1, 100},
			{"vita crescent", 30, 50, 1, 100},
			{"alva crest", 30, 50, 1, 100},
			{"alva crescent", 30, 50, 1, 100},
				
			{"vita claw", 30, 50, 1, 100},
			{"vita cutter", 30, 50, 1, 100},
			{"alva claw", 30, 50, 1, 100},
			{"alva cutter", 30, 50, 1, 100},			
			
			{"vita beamgun", 30, 50, 1, 100},
			{"vita peacekeeper", 30, 50, 1, 100},
			{"alva beamgun", 30, 50, 1, 100},
			{"alva peacekeeper", 30, 50, 1, 100},			
					
			{"vita hammer", 30, 50, 1, 100},
			{"vita scepter", 30, 50, 1, 100},
			{"alva hammer", 30, 50, 1, 100},
			{"alva scepter", 30, 50, 1, 100},
			
			{"icicle frame", 30, 50, 1, 20},
			{"knight frame", 30, 50, 1, 20},
			{"fehler frame", 30, 50, 3, 20},
			
			{"unit_ppsatk_2", 30, 50, 3, 50},
			{"unit_pptatk_2", 30, 50, 3, 50},
			{"unit_ppratk_2", 30, 50, 3, 50},
			{"unit_rainbow_2", 30, 50, 3, 50},
			
			{"orakl sword", 30, 50, 1, 400},
			{"orakl doublesaber", 30, 50, 1, 400},
			{"orakl soaringblade", 30, 50, 1, 400},
			{"orakl rifle", 30, 50, 1, 400},
			{"orakl shot", 30, 50, 1, 400},
			{"orakl orphan", 30, 50, 1, 400},
			{"orakl rod", 30, 50, 1, 400},
			{"orakl wand", 30, 50, 1, 400},
			{"ice stabberrod", 30, 50, 1, 50},
			{"digrinder", 30, 50, 5, 50},
			{"trigrinder", 30, 50, 4, 50},


			{10000, nil, nil, 1, 9},
		},
		
		stagger_duration = TICRATE/2,
		
		weakness = ELEM_FIRE,
		
		reset =	function(mo)
			mo.actiontimer = 0	-- reset action if we get staggered etc.
		end,
		
		thinker = function(mo)
			
			local diff = PSO.lib.questDifficulty()
			PSO.lib.enemyManageAggro(mo)
			
			mo.sprite = SPR_CBFS
			mo.colorized = true
			mo.color = SKINCOLOR_ICY
			mo.scale = FRACUNIT*3/2
			mo.shadowscale = FRACUNIT*2
			
			if not mo.introd
				is_showoff(mo)
				return
			end
			mo.frame = A
			
			local attack_wait = TICRATE*7
			
			if mo.target and mo.target.valid
				mo.waittimer = $ or 1
				
				if not is_neutral(mo)
				
					-- let em attack...
					ls_doleap(mo)	-- will only work if in the proper state
					is_doslash(mo)
					is_dostab(mo)
					is_doicicles(mo)
					is_dorain(mo)
					
				
				elseif mo.waittimer < attack_wait
					-- move similar to crawla commander
					is_movement(mo)
					
					mo.followup = nil	-- reset pinch behaviour til next attack
					mo.waittimer = $+1
				else
					
					local dist = R_PointToDist2(mo.x, mo.y, mo.target.x, mo.target.y)
					local melee_range = FRACUNIT*384
					local mid_range = FRACUNIT*600
					

						--is_preprain(mo)

					if dist <= melee_range
						--print("melee")
						-- in melee range, either do a leap or go back and throw icicles
						if P_RandomRange(0, 1)
							is_prepleap(mo)
						else
							is_prepicicles(mo)
						end
					else
						--print("mid")
						if P_RandomRange(0, 1)
							is_prepstab(mo)
						else
							is_prepslash(mo)
						end					
					end

					
					mo.waittimer = 0
				end
			end
		end,	
}