-- Unidus
/*
	When near players, tries to stay near them to continuously hurt them.
	When afar, throws their balls periodically until they run out
*/

addHook("MobjSpawn", function(mo)
	return true	-- override normal behaviour
end, MT_UNIDUS)

freeslot("MT_PSO_UNIBALL", "S_PSO_UNIBALL")
mobjinfo[MT_PSO_UNIBALL] = {
	
	doomednum = -1,
	spawnstate = S_PSO_UNIBALL,
	flags = MF_NOGRAVITY|MF_MISSILE,
	height = 48<<FRACBITS,
	radius = 24<<FRACBITS,
	spawnhealth = 1000,
}

states[S_PSO_UNIBALL] = {SPR_UNID, B, -1, nil, 0, 0, S_PSO_UNIBALL}

addHook("MobjThinker", function(mo)
	
	mo.angle = $ + ANG1*10
	if leveltime & 1
		P_SpawnGhostMobj(mo)
	end
end, MT_PSO_UNIBALL)

PSO.info.enemies[MT_UNIDUS] = {
		
		name = "Unidus",
		sprite = SPR_UNID,
		frame = A,
		painframe = A,
		
		bhp = 160,
		satk = 40,
		ratk = 40,
		tatk = 10,
		def = 25,
		exp = 8,
		weight = 4,
		
		
		drops = {
			{300, 1, 29, 1, 15},
			{"breaker", 1, 29, 1, 150},
			{"doublegladius", 1, 29, 1, 150},
			{"laser", 1, 29, 1, 150},
			{"canon", 1, 29, 1, 150},
			{"ulrod", 1, 29, 1, 150},
			{"dimate", 1, 29, 1, 25},

			-- Lv30-50
			{650, 30, 50, 1, 15},
			{"scepter", 30, 50, 1, 50},
			{"alva scepter", 30, 50, 1, 75},
			{"vita scepter", 30, 50, 1, 100},
			{"asterod", 30, 50, 1, 200},
			{"rostig frame", 30, 50, 1, 30},
			{"fehler frame", 30, 50, 1, 100},
			{"digrinder", 30, 50, 1, 25},
			{"dimate", 30, 50, 1, 35},			
		},
		
		stagger_duration = TICRATE/2,
		
		weakness = ELEM_EARTH,
		
		reset =	function(mo)
			if not mo.pso.blowdown
				mo.flags = $|MF_NOGRAVITY
				mo.momz = 0
			end
		end,
		
		thinker = function(mo)
			
			local diff = PSO.lib.questDifficulty()
			
			local ball_potency = 70
			local ball_throw_potency = 220
			
			local targetz = mo.floorz + mo.scale*12
			local rate = FRACUNIT*8/10
			
			local numballs = (diff >= DIFF_HARD and 6 or 3)
			
			local distance_threshold = 768
			local ballthrow_wait = TICRATE*2
			
			mo.sprite = SPR_UNID
			mo.frame = A
			
			PSO.lib.enemyManageAggro(mo)
			
			mo.flags = $ & ~(MF_FLOAT|MF_NOGRAVITY)
			
			if not mo.spawnedballs
				
				mo.balls = {}
				local angdiff = (360 / numballs) *ANG1
				for i = 1, numballs
					
					local ball = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
					ball.angle = mo.angle + angdiff*i
					ball.tics = -1
					ball.fuse = -1
					ball.sprite = SPR_UNID
					ball.frame = B
					mo.balls[#mo.balls+1] = ball
				end
				
				mo.spawnedballs = true
			end
			
			if mo.balls and #mo.balls
				
				local ball_dist = 64
				local hitboxes = {}
				
				for i = 1, #mo.balls do
					
					local ball = mo.balls[i]
					local bx = mo.x + ball_dist*cos(ball.angle)
					local by = mo.y + ball_dist*sin(ball.angle)
					
					P_MoveOrigin(ball, bx, by, mo.z)
					
					ball.angle = $ + ANG1*2
					ball.tics = 2
					
					if leveltime % 10 == 0
						
						local hb = PSO.lib.spawnHurtBox(bx, by, mo.z, 40<<FRACBITS, 40<<FRACBITS, mo, ball_potency, DMT_STRIKE, 2, RT_NONE)
						hitboxes[#hitboxes+1] = hb
					end
				end
				
				for i = 1, #hitboxes
					hitboxes[i].pso.links = hitboxes	-- make sure they don't all hurt us at once
				end
			end
			
			if mo.target and mo.target.valid
				local dist = R_PointToDist2(mo.x, mo.y, mo.target.x, mo.target.y)
				mo.angle = R_PointToAngle2(mo.x, mo.y, mo.target.x, mo.target.y)
				
				P_InstaThrust(mo, mo.angle, FRACUNIT*(diff >= DIFF_HARD and 4 or 2))
				
				if dist > distance_threshold*FRACUNIT
					mo.waittimer = $ and $+1 or 1
					
					if mo.waittimer > ballthrow_wait
						
						if mo.balls and #mo.balls
							
							local ball = mo.balls[#mo.balls]

							local msl = PSO.lib.spawnMissile(mo, MT_PSO_UNIBALL, mo.angle, 20*FRACUNIT, ball_throw_potency, DMT_RANGED, 0, 0, 0)
							P_MoveOrigin(msl, ball.x, ball.y, ball.z)
							msl.angle = R_PointToAngle2(msl.x, msl.y, mo.target.x, mo.target.y)
							P_InstaThrust(msl, msl.angle, FRACUNIT*20)
							
							PSO.lib.aimMissile(msl, mo.target)
							
							P_RemoveMobj(ball)
							mo.balls[#mo.balls] = nil
						end
						
						mo.waittimer = 0
					end
				else
					mo.waittimer = 0
				end
				
			end
			
			if mo.z < targetz
				mo.momz = $ + rate
			elseif mo.momz < 0
				mo.momz = min(-mo.scale, $ + rate*2)
			end
		end,
}