-- handles doors, lasers and unlock cards

freeslot("MT_PSO_UNLOCKCARD", "SPR_ULCK", "MT_PSO_CARDREADER", "SPR_CDRD", "MT_PSO_DOOR", "MT_DOOR_HITBOX", "SPR_DOOR")
freeslot("MT_PSO_LASERTOWER", "SPR_LSRT")

-- for colours.
local DOOR_SEALED = 0
local DOOR_LOCKED = 1
local DOOR_UNLOCKED = 2
local DOOR_USED = 3

mobjinfo[MT_PSO_LASERTOWER] = {
	doomednum = 2504,
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	radois = 16<<FRACBITS,
	height = 64<<FRACBITS,
	flags = MF_SOLID
}

mobjinfo[MT_PSO_CARDREADER] = {
	doomednum = 2503,
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	radius = 16<<FRACBITS,
	height = 64<<FRACBITS,
	flags = MF_SOLID,
}

mobjinfo[MT_PSO_UNLOCKCARD] = {
	doomednum = 2502,
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	radius = 48<<FRACBITS,
	height = 48<<FRACBITS,
	flags = MF_NOGRAVITY,
}

mobjinfo[MT_PSO_DOOR] = {
	doomednum = 2501,
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	radius = 127<<FRACBITS,
	height = 320<<FRACBITS,
	flags = MF_NOGRAVITY,
}

freeslot("S_PSO_INVISIBLE")
states[S_PSO_INVISIBLE] = {SPR_NULL, A, -1, nil, 0, 0, S_PSO_INVISIBLE}

mobjinfo[MT_DOOR_HITBOX] = {
	doomednum = -1,
	spawnstate = S_PSO_INVISIBLE,
	deathstate = S_PSO_INVISIBLE,
	spawnhealth = 1000,
	radius = 64<<FRACBITS,
	height = 1024<<FRACBITS,
	flags = MF_SOLID,
}

-- laser tower thinker
-- lasers can be set up independently from rooms,
-- but putting a laser in a room w/ extrainfo allows it to be disabled with a keycard & and a reader of matching extrainfo.
addHook("MobjThinker", function(mo)
	if not mo.extravalue1
		
		local dist = 12
		local ang = 0	-- literally does not matter.
		
		mo.id = mo.extrainfo	-- get the id.
		mo.mobjs = {}
		mo.sides = {}
		
		mo.color = SKINCOLOR_RED
		
		for i = 1, 4
			
			local x = mo.x + dist*cos(ang)
			local y = mo.y + dist*sin(ang)
			
			local s = P_SpawnMobj(x, y, mo.z, MT_THOK)
			s.tics = -1
			s.fuse = -1
			s.sprite = SPR_LSRT
			s.frame = A | FF_PAPERSPRITE
			s.angle = ang - ANG1*90
			s.color = mo.color
			ang = $+ANG1*90		
			
			mo.sides[#mo.sides+1] = s
		end
		
		-- now for the funny, if we're the FIRST LASER IN OUR CHAIN (MTF_EXTRA)...
		if mo.options & 1
			mo.first = true	-- helps identify us faster
			-- now we scan for every other laser, woo!!!
			-- that means several iterations...
			local lasers = {}
			for m in mobjs.iterate() do
				-- get lasers with a higher spawnpoint angle than us.
				if m and m.valid and m ~= mo and m.type == MT_PSO_LASERTOWER and m.mtangle > mo.mtangle
					lasers[#lasers+1] = m
					--PSO.lib.dprint("Found new laser")
				end
			end
			
			table.sort(lasers, function(a, b) return a.mtangle < b.mtangle end)
			
			local final_lasers = {}
			local myang = mo.mtangle +1
			-- sort the table by angle num...
			for i = 1, #lasers do
				local l = lasers[i]
				if l.mtangle == myang
					local index = #final_lasers
					
					-- this is used mostly by the minimap to optimize rendering
					if final_lasers[index]
						final_lasers[index].hnext = l
					else
						mo.hnext = l
					end
					
					final_lasers[index+1] = l		
					PSO.lib.dprint("Found laser with ang "..myang)
					myang = $+1
				else
					break	-- we're finished here.
				end
			end
						
			-- now spawn the laser visuals
			if final_lasers[1]
				
				mo.lasers = final_lasers
				
				if mo.options & 4	-- special: loop laser
					local index = #final_lasers
					
					final_lasers[index].hnext = mo
					final_lasers[index+1] = mo	-- this makes it loop!
				end
			
				local currlaser = mo				
				for i = 1, #final_lasers
					
					local nextlaser = final_lasers[i]
					
					local ang = R_PointToAngle2(currlaser.x, currlaser.y, nextlaser.x, nextlaser.y)
					local dist = R_PointToDist2(currlaser.x, currlaser.y, nextlaser.x, nextlaser.y)
					local ox = (currlaser.x/FRACUNIT + nextlaser.x/FRACUNIT)*FRACUNIT/2
					local oy = (currlaser.y/FRACUNIT + nextlaser.y/FRACUNIT)*FRACUNIT/2
					
					local lsr = P_SpawnMobj(ox, oy, mo.z, MT_THOK)
					lsr.color = SKINCOLOR_RED
					lsr.sprite = SPR_LSRT
					lsr.frame = B|FF_PAPERSPRITE|FF_ADD|FF_FULLBRIGHT
					lsr.color = SKINCOLOR_RED
					lsr.angle = ang
					lsr.spritexscale = (dist - 12*mo.scale)/2
					lsr.tics = -1
					lsr.fue = -1
					mo.mobjs[#mo.mobjs+1] = lsr
					
					-- put solid hitboxes from the start to finish...
					local ndist = 0
					dist = $/FRACUNIT
					for i = 1, dist/32
						local sx = currlaser.x + ndist*cos(ang)
						local sy = currlaser.y + ndist*sin(ang)
						local hb = P_SpawnMobj(sx, sy, mo.z, MT_DOOR_HITBOX)
						--hb.sprite = SPR_THOK
						hb.radius = $/4
						
						ndist = $+32
						mo.mobjs[#mo.mobjs+1] = hb
					end
					
					currlaser = nextlaser
				end
			end
		end
		
		mo.extravalue1 = 1
	end
	
	-- disable everything
	if mo.extravalue2 == 1
		
		-- make sure we do that.
		if not mo.lasers
			mo.lasers = {mo}
		else
			mo.lasers[#mo.lasers+1] = mo
		end
		
		for i = 1, #mo.lasers
			local l = mo.lasers[i]
			l.extravalue2 = 2
			for j = 1, #l.sides
				l.sides[j].color = SKINCOLOR_EMERALD
			end
		end
		
		if mo.mobjs
			for i = 1, #mo.mobjs
				mo.mobjs[i].fuse = 1
			end
			mo.mobjs = nil
		end
		mo.extravalue2 = 2
	end
end, MT_PSO_LASERTOWER)

-- card reader thinker
addHook("MobjThinker", function(mo)
	
	if not mo.extravalue1	-- set up the visuals...
		
		local ang = mo.angle
		local frames = {A, B, C, B}
		local dist = 12
		
		mo.id = mo.extrainfo	-- get the id.
		
		for i = 1, 4
			
			local x = mo.x + dist*cos(ang)
			local y = mo.y + dist*sin(ang)
			
			local s = P_SpawnMobj(x, y, mo.z, MT_THOK)
			s.tics = -1
			s.fuse = -1
			s.sprite = SPR_CDRD
			s.frame = frames[i] | FF_PAPERSPRITE
			s.angle = ang - ANG1*90
			ang = $+ANG1*90
			
			if not mo.colorside
				mo.colorside = s
				s.color = SKINCOLOR_RED
			end
		end
		mo.extravalue1 = 1
	end
end, MT_PSO_CARDREADER)

PSO.lib.activateCardReader = function(p, reader)
	local keycards = server.keycards or 0
	
	if keycards & (2^(reader.id-1))
		
		-- get the room:
		assert(server.psorooms, "Card reader in map with no rooms.")
		local room = server.psorooms[reader.roomid]
		assert(room, "Card reader not in a room.")
		
		reader.colorside.color = SKINCOLOR_EMERALD
		reader.extravalue2 = 1	-- activated
		
		for i = 1, #room.objects
			
			local mo = room.objects[i]
			if mo and mo.valid
				
				if mo.type == MT_PSO_DOOR
				and mo.id == reader.id
				and mo.extravalue2 == DOOR_LOCKED
					
					-- unlock the door!
					mo.extravalue2 = DOOR_UNLOCKED
					PSO.lib.dprint("Door unlocked")
				
				elseif mo.type == MT_PSO_LASERTOWER
				and mo.first
				and mo.id == reader.id
					
					mo.extravalue2 = 1
					PSO.lib.dprint("Laser deactivated")
				end
			end
		end
		
		server.keycards = $ & ~(2^(reader.id-1))
		
		-- set this card reader as the "checkpoint" to teleport a player to if they somehow die.
		server.keycheckpoint = reader
		
		return true
	else
		PSO.lib.dprint("Card missing")
		return false
	end
end

-- card thinker
addHook("MobjThinker", function(mo)
	
	if mo.extravalue1	-- once it's activated.
		
		if mo.id == nil
			mo.id = mo.mtangle
		end
		
		mo.sprite = SPR_ULCK
		mo.frame = A|FF_PAPERSPRITE|FF_ADD|FF_FULLBRIGHT
		mo.color = SKINCOLOR_RED
		mo.angle = leveltime*ANG1*2
		
		if leveltime%5 == 0
			local spark = P_SpawnMobj(mo.x + P_RandomRange(-32, 32)*mo.scale, mo.y + P_RandomRange(-32, 32)*mo.scale, mo.z, MT_SUPERSPARK)
			spark.momz = P_RandomRange(1, 8)*mo.scale
			spark.scale = mo.scale
		end
	end
end, MT_PSO_UNLOCKCARD)

PSO.lib.playerGetKeyCard = function(p, card)
	server.keycards = $ or 0
	
	for i = 1, 8
		local spark = P_SpawnMobj(card.x, card.y, card.z+card.height/2, MT_SUPERSPARK)
		spark.momx = P_RandomRange(-32, 32)*card.scale
		spark.momy = P_RandomRange(-32, 32)*card.scale
		spark.momz = P_RandomRange(-32, 32)*card.scale
	end
	S_StartSound(p.mo, sfx_pikup)
	
	server.keycards = $|(2^(card.id-1))
	P_RemoveMobj(card)
end

-- door thinker
local doorcolours = {
	[DOOR_SEALED] = SKINCOLOR_GREY,
	[DOOR_LOCKED] = SKINCOLOR_RED,
	[DOOR_UNLOCKED] = SKINCOLOR_CYAN,
	[DOOR_USED] = SKINCOLOR_EMERALD,
}

addHook("MobjThinker", function(mo)
	
	-- find players close to us so that we know when to open
	local playerinrange = false
	if mo.extravalue2 >= DOOR_UNLOCKED
		
		searchBlockmap("objects", function(ref, found)
			
			if found and found.valid and found.type == MT_PLAYER
				playerinrange = true
				return false
			end
		end, mo)
	end
	
	-- make the door...
	if not mo.extravalue1	-- not set
		
		mo.doors = {}
		mo.hitboxes = {}
		mo.extravalue2 = DOOR_LOCKED
		mo.id = mo.extrainfo
		
		local ang = mo.angle + ANG1*90
		
		for i = 1, 2
			local x = mo.x + (320/4 -12)*cos(ang)
			local y = mo.y + (320/4 -12)*sin(ang)
			local door = P_SpawnMobj(x, y, mo.z, MT_THOK)
			door.tics = -1
			door.fuse = -1
			door.color = doorcolours[mo.extravalue2]
			door.sprite = SPR_DOOR
			door.frame = B|FF_PAPERSPRITE
			door.flags = MF_NOGRAVITY
			door.radius = 100*FRACUNIT
			door.height = 320*FRACUNIT
			door.angle = InvAngle(ang)
			
			mo.doors[#mo.doors+1] = door
			
			ang = $+ANG1*180
		end
		
		local hbrange = 32
		for i = 1, 6
			
			local x = mo.x + (hbrange)*cos(ang)
			local y = mo.y + (hbrange)*sin(ang)			
			
			local hb = P_SpawnMobj(x, y, mo.z, MT_DOOR_HITBOX)
			mo.hitboxes[#mo.hitboxes+1] = hb
			
			x = mo.x - (hbrange)*cos(ang)
			y = mo.y - (hbrange)*sin(ang)				

			hb = P_SpawnMobj(x, y, mo.z, MT_DOOR_HITBOX)
			mo.hitboxes[#mo.hitboxes+1] = hb
			
			hbrange = $+32
		end
		
		mo.extravalue1 = 1
	end
	
	-- open the door!
	if playerinrange
		
		if not mo.cusval
			-- disable the hitboxes:
			for i = 1, #mo.hitboxes
				mo.hitboxes[i].flags = $|MF_NOCLIPTHING
			end
			PSO.lib.dprint("door opening")
		end
		
		mo.cusval = min(TICRATE/2, $ + 1)
	else	-- close it.
		
		-- rennable hitboxes
		if mo.cusval == 1
			for i = 1, #mo.hitboxes
				mo.hitboxes[i].flags = $ & ~MF_NOCLIPTHING
			end	
			PSO.lib.dprint("door closed")
		end
		
		mo.cusval = max(0, $-1)
	end
		
	local ang = mo.angle + ANG1*90
	for i = 1, #mo.doors
		local door = mo.doors[i]
		local dist = (320/4 -12) + mo.cusval*16
		
		door.color = doorcolours[mo.extravalue2] or SKINCOLOR_NONE
		
		local x = mo.x + dist*cos(ang)
		local y = mo.y + dist*sin(ang)
		P_MoveOrigin(door, x, y, mo.z)
		ang = $+ ANG1*180
	end
	
end, MT_PSO_DOOR)

-- remove keycards on map change.
-- it creates too many bugs otherwise 
addHook("MapChange", do
	if server and server.valid
		server.keycards = 0
	end
end)