-- Drop box item

freeslot("sfx_pikup", "sfx_full")
for i = 0, 4
	freeslot("sfx_rdrp"..i)
end

freeslot("MT_PSO_DROP", "S_PSO_DROP", "S_PSO_RAINBOWDROP", "SPR_RDRP", "SPR_RDRB", "SPR_MSTA", "S_PSO_MESETA")
mobjinfo[MT_PSO_DROP] = {
	doomednum = -1,
	spawnstate = S_PSO_DROP,
	radius = 16 << FRACBITS,
	height = 32 << FRACBITS,
	spawnhealth = 1000,
	flags = MF_RUNSPAWNFUNC,
}
states[S_PSO_DROP] = {SPR_RDRP, FF_ANIMATE|FF_FULLBRIGHT, -1, nil, 5, 4, S_PSO_DROP}
states[S_PSO_MESETA] = {SPR_MSTA, FF_ANIMATE|FF_FULLBRIGHT, -1, nil, 5, 4, S_PSO_MESETA}
states[S_PSO_RAINBOWDROP] = {SPR_RDRP, G|FF_ANIMATE|FF_FULLBRIGHT, -1, nil, 5, 4, S_PSO_RAINBOWDROP}

freeslot("S_PSO_RDRAURA")
states[S_PSO_RDRAURA] = {SPR_RDRB, FF_ANIMATE|FF_ADD|FF_FULLBRIGHT, ((TICRATE/2) *4), nil, 3, TICRATE/2, S_NULL}

local USE_WEAPONDROPMODEL = true

local weapon_col = {
	[WPT_SWORD] = SKINCOLOR_ORANGE,
	[WPT_DOUBLESABER] = SKINCOLOR_ORANGE,
	[WPT_RIFLE] = SKINCOLOR_SAPPHIRE,
	[WPT_SHOTGUN] = SKINCOLOR_SAPPHIRE,
	[WPT_ROD] = SKINCOLOR_YELLOW,
	[WPT_SABER] = SKINCOLOR_ORANGE,
	[WPT_HANDGUN] = SKINCOLOR_SAPPHIRE,
	[WPT_WAND] = SKINCOLOR_YELLOW,
}

local weapon_dropangle = {
	[WPT_SWORD] = ANG1*90,
	[WPT_DOUBLESABER] = ANG1*90,
	[WPT_RIFLE] = ANG1*90,
	[WPT_SHOTGUN] = ANG1*90,
	[WPT_ROD] = ANG1*90,
	[WPT_SABER] = ANG1*90,
	[WPT_HANDGUN] = ANG1*90,
	[WPT_WAND] = ANG1*90,
}

local weapon_offsets = {
	[WPT_SWORD] = -48,
	[WPT_DOUBLESABER] = -48,
	[WPT_RIFLE] = -24,
	[WPT_SHOTGUN] = -24,
	[WPT_ROD] = -48,
	[WPT_SABER] = -32,
	[WPT_HANDGUN] = -12,
	[WPT_WAND] = -32,
}


-- returns what drops should be obtained by every player for any given roll in a table.
PSO.lib.calcDrops = function(droptable, level)
	
	-- find the max rate:
	local drops = {}	-- drops per player
	local maxrate = 0
	local curr_odd = 1
	local final_droptable = {}
	
	-- pass 1: figure out the largest number in our drop table
	for i = 1, #droptable
		local drop = droptable[i]
		
		if not level
		or (not drop[2] or level >= drop[2]) and (not drop[3] or level <= drop[3])
			maxrate = max($, drop[5])
		end
	end

	-- pass 2: build the usable drop table using that max rate
	for i = 1, #droptable

		local drop = droptable[i]

		if not level
		or (not drop[2] or level >= drop[2]) and (not drop[3] or level <= drop[3])
				
			local odd = drop[4] * maxrate / drop[5]
				
			curr_odd = $ + odd	
			final_droptable[#final_droptable+1] = {drop[1], curr_odd}	
			--print("Table["..i.."]: {"..drop[1]..", "..curr_odd.."}")
		end
	end
	
	final_droptable[#final_droptable+1] = {nil, maxrate}
	--print("Table["..#final_droptable.."]: {nil, "..maxrate.."}")
	
	-- pass 3: and now we just pick random shit in the table!
	for p in players.iterate
		local num = P_RandomRange(1, maxrate)
		--print("player "..(#p+1)..": "..num)
		-- figure out which item we got!
		for i = 1, #final_droptable-1
			
			local drop = final_droptable[i]
			if num <= drop[2]
				--print("Get "..drop[1])
				if type(drop[1]) == "number"	-- meseta
					drops[#p +1] = drop[1]
				else
					drops[#p +1] = PSO.lib.makeItem(drop[1])	-- item
				end
				break
			end
		end	
	end
	return drops
end

-- make a player pick up a drop!
-- if possible, that is.
PSO.lib.playerPickUpDrop = function(p, drop)
	
	if drop.collected
		if drop.collected & (2^(#p)) 
			return	-- player already collected that drop!!
		end
	end
	
	if not drop.psoitem and not drop.psoitems
		return  -- there's.... nothing.
	end
	
	local item = drop.psoitem
	if drop.psoitems
		item = drop.psoitems[#p+1]
	end
	if not item return end	-- there's nothing for you, buddy
	
	local info, take
	
	if type(item) ~= "number"	-- numbers are for meseta
		info = PSO.lib.itemInfo(item)
		take = PSO.lib.playerGetItem(p, item, drop.psoqty)
	else
		take = true
		-- add the meseta to the player's count!
		p.pso.meseta = $+item
		
	end
		
	local globaldrop = (drop.psoitem)	-- if this isn't set then it's a local drop
	
	if not take
		S_StartSound(nil, sfx_full, p)
		return 	-- player couldn't take the item
	else
		if type(take) == "number" and drop.psoqty and take < drop.psoqty
			-- took a certain amount
			drop.psoqty = $ - take
		else	-- we took everything
			
			for i = 1, 8
				local s = P_SpawnMobj(drop.x, drop.y, drop.z, MT_SUPERSPARK)
				s.momx = P_RandomRange(-16, 16)*drop.scale
				s.momy = P_RandomRange(-16, 16)*drop.scale
				s.momz = P_RandomRange(-16, 16)*drop.scale
				s.destscale = 1
				s.scalespeed = s.scale/12
				
				-- don't show them to other players!
				if p ~= consoleplayer
				and not globaldrop
					s.flags2 = $|MF2_DONTDRAW
				end
			end
			
			if globaldrop
				P_RemoveMobj(drop)
			else
				-- has everyone taken us?
				-- this player collected us!
				drop.collected = $ | (2^(#p))
				
				local numplayers = 0
				local numcollected = 0
				for pl in players.iterate do
					if drop.collected & (2^(#pl))
					or drop.psoitems and not drop.psoitems[#pl+1]	-- null item also counts as "collected"
						numcollected = $+1
					end
					numplayers = $+1
				end
				
				PSO.lib.dprint(numcollected.."/"..numplayers)
				if numcollected >= numplayers
					PSO.lib.dprint("collected by everyone, vanish")
					P_RemoveMobj(drop)
				end
			end
		end
		
		S_StartSound(p.mo, sfx_pikup, not globaldrop and p or nil)
		return true
	end
end


-- mo.psoitem: item ref for global drops
-- mo.psoqty: # of a given item (may be nil) can only be set for global drops.
-- mo.psoitems[]: item ref PER PLAYER


addHook("MobjThinker", function(mo)

	-- first hit the ground...
	
	local rarity = 1
	local typ = 1
	local col = SKINCOLOR_GREEN
	local mapobjectscale = FRACUNIT
	
	local dropsprite
	local dropframe
	local dropangle
	local soffs
	local mypnum = consoleplayer and (#consoleplayer+1) or 1
	
	mo.collected = $ or 0
	
	local itref = mo.psoitem
	if mo.psoitems
		itref = mo.psoitems[mypnum]
		--print(itref)
		--print(itref.ref)
	end

	local dontdraw = 0
	local meseta = 0

	if not itref or mo.collected & (2^(mypnum-1))
		dontdraw = MF2_DONTDRAW
	end
	
	if itref and type(itref) == "number"
		meseta = MF2_DONTDRAW
		itref = nil
	end
	
	if itref
		
		local info
		
		if type(itref) == "string"
			info = PSO.info.items[itref]
		else
			info = PSO.info.items[itref.ref]
		end
		
		rarity = info.rarity or 1
		typ = info.type or 1
		
		if rarity < 7
			
			if typ == IT_WEAPON
				col = SKINCOLOR_YOGURT
				
				if USE_WEAPONDROPMODEL
					col = weapon_col[info.weapontype]
				end
				
			elseif typ == IT_ARMOR
			or typ == IT_UNIT
				col = SKINCOLOR_RASPBERRY
			
			else
				col = SKINCOLOR_GREEN
			end
			
		elseif rarity < 13
			col = SKINCOLOR_RED
		else
			col = SKINCOLOR_CARBON
		end
		
		if USE_WEAPONDROPMODEL
		and typ == IT_WEAPON
			
			dropsprite = info.sprite
			dropframe = info.frame
			dropangle = weapon_dropangle[info.weapontype]
			soffs = weapon_offsets[info.weapontype]
			
		end
	end
	
	if (P_IsObjectOnGround(mo)) and not mo.extravalue2
	
		P_InstaThrust(mo, 0, 0)
		mo.extravalue2 = 1
		mo.scale = 1
		mo.destscale = mapobjectscale
		
		mo.scalespeed = mapobjectscale/4
		mo.flags2 = $ & ~MF2_DONTDRAW
		mo.flags = $ & ~MF_NOCLIPTHING
		mo.rollangle = dropangle or 0
		
		if meseta
			mo.state = S_PSO_MESETA
			mo.height = 16*mo.scale
			
		elseif rarity >= 13
			mo.state = S_PSO_RAINBOWDROP
		end
		if not dontdraw
			S_StartSound(mo, sfx_rdrp0)
		end


		-- drop circle aura
		local ang = leveltime*ANG1*2
		local m = mo
		for i = 1, 8 do
			local x = mo.x + FixedMul(16*mapobjectscale, cos(ang))
			local y = mo.y + FixedMul(16*mapobjectscale, sin(ang))
			local z = mo.z

			local a = P_SpawnMobj(x, y, z, MT_THOK)
			--print("A")
			a.sprite = SPR_RDRP
			a.frame = M|FF_ADD|FF_PAPERSPRITE|FF_FULLBRIGHT
			a.scale = mapobjectscale/2
			a.tics = -1
			a.colorized = true
			-- colour:
			a.color = col

			a.angle = ang + ANG1*90


			-- build hnext chain
			m.hnext = a
			m = a
			ang = $ + (360/8)*ANG1
		end
	elseif mo.extravalue2
	and dropsprite
		
		mo.sprite = dropsprite
		mo.frame = dropframe|FF_PAPERSPRITE
		mo.angle = leveltime*ANG1*2
		
		mo.rollangle = dropangle
		mo.spriteyoffset = -soffs*FRACUNIT
		
		if R_PointToAngle(mo.x, mo.y, mo.z) - mo.angle > 0
		and render_hack == "software"
			mo.rollangle = -dropangle	-- this is retarded
		end
		
	end
	
	mo.flags2 = $|dontdraw
	
	local spin = mo.hnext
	local ang = leveltime*ANG1*2
	while spin
		
		local x = mo.x + FixedMul(16*mapobjectscale, cos(ang))
		local y = mo.y + FixedMul(16*mapobjectscale, sin(ang))
		local z = mo.z		
		
		P_MoveOrigin(spin, x, y, z)
		spin.angle = ang + ANG1*90
		spin.flags2 = $|dontdraw|meseta
		spin.tics = 2
		
		ang = $ + (360/8)*ANG1
		spin = spin.hnext
	end

	-- while in the air, move twice as fast
	local gravity = P_GetMobjGravity(mo)
	if mo.extravalue2 == 0	-- in the air
		
		for i = 1, 3
			mo.z = $+mo.momz
			P_TryMove(mo, mo.x+mo.momx, mo.y+mo.momy, true)
			--P_MoveOrigin(mo, mo.x+mo.momx, mo.y+mo.momy, mo.z+mo.momz)
			mo.momz = $ + gravity
			mo.flags2 = $|MF2_DONTDRAW
			local s = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SUPERSPARK)
			s.flags2 = $|dontdraw
		end	
	elseif mo.extravalue2 == 1	-- on the ground ready to be picked up!
		
		-- respawning:
		if mo.cvmem
			mo.cvmem = $-1
			if not mo.cvmem
				--mo.flags = $ | MF_SPECIAL
				mo.flags2 = $ & ~MF2_DONTDRAW
			else
				return
			end	
		end
		
		if mo.fuse == 2 and mo.hnext
			-- remove hnext list
			local m = mo.hnext
			while m
				local buf = m
				m = buf.hnext
				P_RemoveMobj(buf)
			end
		end
		
		if not meseta
			mo.colorized = true
		end
		
		if rarity <= 12
			mo.color = col
			if rarity >= 10 and rarity <= 12
			
				if leveltime % 15 == 0
					local spark = P_SpawnMobj(mo.x + P_RandomRange(-32, 32)*mo.scale, mo.y + P_RandomRange(-32, 32)*mo.scale, mo.z, MT_SUPERSPARK)
					spark.momz = mo.scale*3
					spark.tics = $*3
					spark.flags2 = $|dontdraw
				end
			end
		else	-- 13+

			local rainbow = {
				SKINCOLOR_PINK,
				SKINCOLOR_PEACHY,
				SKINCOLOR_SUNSET,
				SKINCOLOR_YELLOW,
				SKINCOLOR_LIME,
				SKINCOLOR_MINT,
				SKINCOLOR_AQUA,
				SKINCOLOR_SKY,
				SKINCOLOR_PASTEL,
				SKINCOLOR_RASPBERRY
			}

			local colour = rainbow[leveltime % (10*(#rainbow)) / 10 +1]
			local tcolour = rainbow[(leveltime+10) % (10*(#rainbow)) / 10 +1]
			local trans = (9 - (leveltime % 10)) << V_ALPHASHIFT

			mo.color = colour
			local t = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
			t.sprite = mo.sprite
			t.tics = 2
			t.frame = mo.frame | (trans << FF_TRANSSHIFT) | FF_FULLBRIGHT
			t.scale = mo.scale
			t.flags2 = mo.flags2
			t.angle = mo.angle
			t.colorized = true
			t.color = tcolour
			t.spriteyoffset = mo.spriteyoffset
			t.rollangle = mo.rollangle
			t.flags2 = $|dontdraw

			if leveltime % 5 == 0
				local spark = P_SpawnMobj(mo.x + P_RandomRange(-32, 32)*mo.scale, mo.y + P_RandomRange(-32, 32)*mo.scale, mo.z, MT_SUPERSPARK)
				spark.momz = mo.scale*6
				spark.tics = $*3

				local aura = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
				aura.momz = 8*mapobjectscale
				aura.scale = mo.scale/2
				aura.state = S_PSO_RDRAURA
				aura.colorized = true
				aura.color = mo.color
				aura.scalespeed = mapobjectscale/(TICRATE*2)
				aura.flags2 = $|dontdraw

				if rarity < 15
					aura.destscale = 1
				else
					aura.scalespeed = mapobjectscale*2
				end
			end

			if rarity >= 15
			and leveltime & 1
				local spark = P_SpawnMobj(mo.x + P_RandomRange(-12, 12)*mo.scale, mo.y + P_RandomRange(-12, 12)*mo.scale, mo.z, MT_SUPERSPARK)
				spark.momz = mo.scale*16
				spark.scale = $*3/2
				spark.flags2 = $|dontdraw
			end
		end
	end
end, MT_PSO_DROP)

-- debug command
COM_AddCommand("psod_drop", function(p, item)

	if not server.psocheats
		print("Enable cheats with 'psod_debugcheats' to use this command")
		return 
	end

	local drop = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_PSO_DROP)
	local selected = PSO.lib.makeItem(item)
	
	drop.psoitem = selected	-- global drop
end)

