-- Room system
freeslot("MT_PSO_ROOMANCHOR")

mobjinfo[MT_PSO_ROOMANCHOR] = {
	doomednum = 2500,
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	flags = MF_NOCLIP|MF_NOGRAVITY|MF_NOCLIPHEIGHT,
}

PSO.lib.objectInRoom = function(room, mo)
	return mo.x >= room.x
		and mo.x <= room.x + room.w
		and mo.y <= room.y
		and mo.y >= room.y - room.h
end

-- creates the rooms, their bounds and associated objects.
local function makeRooms()
	if not server return end
	
	PSO.lib.setupEnemies()
	
	server.psorooms = {}
	
	local room_objects = {}		-- room anchors
	local other_objects = {}	-- other items we may have to put within rooms
	
	for m in mobjs.iterate()
		
		if m and m.valid
			if m.type == MT_PSO_ROOMANCHOR
				m.id = m.mtangle
				room_objects[#room_objects+1] = m
			
			elseif m.flags & MF_ENEMY
			or m.type == MT_PSO_UNLOCKCARD
			or m.type == MT_PSO_CARDREADER
			or m.type == MT_PSO_DOOR
			or m.type == MT_PSO_LASERTOWER
			or m.type == MT_PSO_CRATE
			or m.type == MT_PSO_TELEPORTER
				other_objects[#other_objects+1] = m
				
			end
		end
	end
	
	-- sort room objects by angle to make them easier to process
	table.sort(room_objects, function(a, b) return a.id < b.id end)
	
	-- now create the rooms by going 2 by 2
	local lastid
	for i = 1, #room_objects-1, 2
		
		local p1 = room_objects[i]
		local p2 = room_objects[i+1]
		
		assert(p1, "Missing reference point setting room #"..(lastid or "0"))
		
		local id = p1.id
		
		assert(p2, "room #"..(id).." is missing a 2nd anchor point")
		local id2 = p2.id
		
		assert(id == id2, "room #"..(id)..": mismatched IDs ("..id2..") More than 2 points / Missing 2nd point")
		
		-- if we made it here, we are sure that both the ID points are okay.
		local x, y, w, h
		
		local leftpoint = p1.x < p2.x and p1 or p2
		local toppoint = p1.y > p2.y and p1 or p2
		local rightpoint = leftpoint == p1 and p2 or p1
		local bottompoint = toppoint == p1 and p2 or p1
		
		local lastroom = ((p1.options & 1) or (p2.options & 1))
		
		x = leftpoint.x
		y = toppoint.y
		w = rightpoint.x - leftpoint.x
		h = toppoint.y - bottompoint.y
		
		-- and like that we have our boundaries
		local dta = {
			id = id,
			p1 = p1,
			p2 = p2,
			x = x,
			y = y,
			w = w,
			h = h,
			active = false,		-- true when room has been entered once
			cleared = false,	-- true when room is cleared
			lastroom = lastroom, -- if true, finish the quest when the room is cleared
			objects = {},		-- all objects in the room for convenience. (! SOME MAY BE INVALID!!)
			waves = {},			-- enemy waves (contains references to the objects directly) (! DEAD INVALID ENEMIES ARE KEPT)
			bosswaves = 0,		-- adds a power of 2 for each wave that contains a boss.
			cards = {},			-- cards spawned per wave. ONLY ONE.
			boxes = {},			-- boxes to spawn per wave
			teleporters = {},	-- teleporters to spawn per wave
			
			wavedrops = {},		-- items to put in the boxes for this wave.
								-- only set if the wave's boxes has items for this wave.
								
			wavenum = 0,		-- current wave number
		}
		
		/*print("room: ")
		print(x/FRACUNIT)
		print(y/FRACUNIT)
		print(w/FRACUNIT)
		print(h/FRACUNIT)*/
		server.psorooms[#server.psorooms+1] = dta
	end
	
	-- now that we have our rooms, figure out where objects belong
	for i = 1, #server.psorooms
		
		local room = server.psorooms[i]
		local j = 1
		while other_objects[j]
			
			local mo = other_objects[j]
			
			if PSO.lib.objectInRoom(room, mo)
				PSO.lib.dprint(j.." "..tostring(mo).." added to room #"..i)
				room.objects[#room.objects+1] = mo
				mo.roomid = i	-- assign object to a room, this helps it find other objects within the same room.
				table.remove(other_objects, j)	-- remove from table, we *know* the object is in here. 
			else
				j = $+1
			end
		end
		
		-- build enemy waves
		for k = 1, #room.objects
			local m = room.objects[k]
			local id = m.extrainfo +1
			
			if m.type == MT_PSO_UNLOCKCARD
				assert(room.cards[id] == nil, "Room "..i..": multiple unlock cards spawning on wave "..id..", there can only be one.")
				room.cards[id] = m
				PSO.lib.dprint("Room "..i.." Card "..id..": "..tostring(m))
			
			elseif m.type == MT_PSO_CRATE
			and (m.options & 1)
				room.boxes[id] = $ or {}
				table.insert(room.boxes[id], m)
				
				-- make the crate invisible and intangible...
				m.state = S_INVISIBLE
				
				PSO.lib.dprint("Room "..i.." Crate for wave "..id..": "..tostring(m))
			
			elseif m.type == MT_PSO_TELEPORTER
			and m.extrainfo
				-- teleporters are a bit hacky, they take extrainfo -1 so that extrainfo 0 is no wave dependency
				room.teleporters[id-1] = $ or {}
				table.insert(room.teleporters[id-1], m)
				
				m.hold = true
				
				PSO.lib.dprint("Room "..i.." Teleporter for wave "..(id-1)..": "..tostring(m))
				PSO.lib.dprint(id-1)
				
			elseif m.flags & MF_ENEMY
			
				room.waves[id] = $ or {}
				table.insert(room.waves[id], m)
				m.roomid = i
				m.waveid = id
				
				-- mark waves with bosses
				if m.options & 1
					room.bosswaves = $ | (2^(id-1))
					PSO.lib.dprint("boss waves: "..room.bosswaves)
				end
				
				PSO.lib.dprint("Room "..i.." Wave "..id..": "..tostring(m))
			end
		end
	end
end

addHook("MapLoad", makeRooms)

addHook("MapChange", do
	if server and server.valid
		server.psorooms = nil
	end
end)

local function handleRooms()
	if not server.psorooms return end	-- wait til these are generated
	
	-- iterate in another order in case goofy shit happens with the way the mapper sets rooms up
	for i = 1, #server.psorooms
		local room = server.psorooms[i]
		
		for j = 1, #room.waves
			
			if j ~= room.wavenum	-- dormant wave
				-- "disable" all of these enemies.
				for k = 1, #room.waves[j]
					local en = room.waves[j][k]
					
					if en and en.valid
						if PSO.debug
							P_SpawnMobj(en.x, en.y, en.z, MT_THOK)
						end
						
						en.state = S_INVISIBLE
						en.tics = -1
						en.fuse = -1
						en.flags = MF_NOTHINK|MF_NOCLIP|MF_NOGRAVITY
					end
				end
			else	-- active wave
				
				local deadcount = 0
				local totalcount = 0
				
				-- "enable" all of this wave's enemy.
				for k = 1, #room.waves[j]
					local en = room.waves[j][k]
					
					totalcount = $+1
					if en.valid
					
						if en.flags & MF_NOTHINK
						and not (server.questdata and server.questdata.complete)
							
							-- if opt 4 is checked or the enemy is a boss, spawn instantly
							if en.options & 4
							or en.pso.boss
								en.flags = mobjinfo[en.type].flags|MF_SOLID & (~MF_SPECIAL)
								en.state = S_PSO_ENEMYDUMMY
								en.spawned = true
							else	-- else, use the normal thing
								local tin = P_SpawnMobj(en.x, en.y, en.z, MT_PSO_TELEPORTIN)
								en.flags = $ & ~MF_NOTHINK	-- remove that for the time being
								tin.target = en
							end	
						end
						
						-- call ongoing enemy thinkers!
						if en.spawned
						and en.pso
						and en.pso.hp
							PSO.lib.enemyThinker(en)
						end
					else
						-- do something with dead enemies here
						deadcount = $+1
					end
				end
				
				if deadcount >= totalcount
					room.wavenum = $+1
					
					-- spawn the keycard
					if room.cards[j] and room.cards[j].valid
						room.cards[j].extravalue1 = 1
					end
					
					-- spawn teleporters if need be!
					if room.teleporters[j]
						for k = 1, #room.teleporters[j]
							room.teleporters[j][k].hold = false	-- let em spawn!
						end
					end
					
					-- spawn boxes that aren't tied to bosses
					--if not (room.bosswaves & (2^(j-1)))
						--print("room is not a boss wave, attempt to spawn boxes")
					if room.boxes[j]
						--print("do boxes")
						for k = 1, #room.boxes[j]
							local box = room.boxes[j][k]
							
							if not box or not box.valid continue end	--!?
							
							if box.options & 4
							or not (room.bosswaves & (2^(j-1)))
								
								--print("spawn box...")
								local tpin = P_SpawnMobj(box.x, box.y, box.z, MT_PSO_TELEPORTIN)
								tpin.target = box
								tpin.nopush = true
							end
						end
					end
					--end
					
					if room.wavenum > #room.waves
						room.cleared = true
						
						if room.lastroom
							-- finish the quest!
							PSO.lib.clearQuest()
						end
					end
				end
			end
		end
		
		-- if a player gets in the room we need to activate it!
		if not room.active
		and not (server.questdata and server.questdata.complete)	-- do not activate rooms when the quest is completed
			for p in players.iterate do
				if p.mo and p.mo.valid
				and PSO.lib.objectInRoom(room, p.mo)
					
					room.active = true
					room.wavenum = 1	-- begin enemy assault!
					
					PSO.lib.dprint("Room "..i.." is now active")
				end
			end
		end
	end
end

PSO.lib.registerFrameFunc(handleRooms)