-- Makes entities it's attached to "appear" via a teleporting fashion.
freeslot("sfx_telein", "MT_PSO_TELEPORTIN", "SPR_SPHX")

sfxinfo[sfx_telein].flags = $|SF_X4AWAYSOUND

mobjinfo[MT_PSO_TELEPORTIN] = {
	doomednum = -1,
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	radius = 1,
	height = 1,
	flags = MF_NOCLIPTHING|MF_NOGRAVITY|MF_NOCLIP,
}

addHook("MobjThinker", function(mo)
	
	local tgt = mo.target
	if not mo.target or not mo.target.valid
		P_RemoveMobj(mo)
		return
	end	-- bye
	
	local tinfo = mobjinfo[mo.target.type]
	local pushradius = tinfo.radius*8
	
	if not mo.extravalue1
		S_StartSound(mo, sfx_telein)
		mo.fuse = TICRATE*2
		mo.extravalue1 = 1
	end
	
	local fadelevel = 9 - (TICRATE*2 - mo.fuse)/8
	
	fadelevel = max(0, $)
	fadelevel = min(9, $)
	fadelevel = $ << FF_TRANSSHIFT
	
	if tinfo.flags & MF_ENEMY	-- target is an enemy
		local einfo = PSO.info.enemies[tgt.type]
		tgt.state = S_PSO_ENEMYDUMMY
		
		if einfo
			tgt.sprite = einfo.sprite or $
			tgt.frame = einfo.frame or A
			tgt.scale = einfo.scale or FRACUNIT
		end
		
		tgt.frame = $ & ~FF_TRANSMASK
		tgt.frame = $|fadelevel
		P_InstaThrust(tgt, 0, 0)
		tgt.momz = 0
		
		-- we're about to vanish!
		if mo.fuse == 1
			tgt.flags = mobjinfo[tgt.type].flags|MF_SOLID & (~MF_SPECIAL)
			tgt.spawned = true
		end
	
	elseif tgt.type == MT_PSO_CRATE	-- hacky for this one
		
		if not tgt.hnext
			A_SetupCrate(tgt)
			tgt.flags = MF_NOCLIP|MF_NOGRAVITY
		end
		
		local cur = tgt.hnext
		while cur
			cur.frame = $ & ~FF_TRANSMASK
			cur.frame = $|fadelevel
			cur = cur.hnext
		end
		
		if mo.fuse == 1
			tgt.flags = MF_SOLID|MF_SHOOTABLE
		end
	end
	
	-- visual:
	if leveltime%10 == 0
		
		local ang = mo.angle
		local dist = 64
		for i = 1, 6
			
			local x = mo.x + dist*cos(ang)
			local y = mo.y + dist*sin(ang)
			local s = P_SpawnMobj(x, y, mo.z, MT_THOK)
			s.fuse = TICRATE/2
			s.tics = TICRATE/2
			s.sprite = SPR_SPHX
			s.frame = A|FF_ADD|FF_PAPERSPRITE
			s.color = SKINCOLOR_SAPPHIRE
			s.momz = FRACUNIT*16
			s.angle = ang + ANG1*90
			ang = $ + (360/6)*ANG1
		end
		
	end
	
	-- push players away from the radius
	if mo.nopush return end	-- don't push players
	for p in players.iterate
		
		local pmo = p.mo
		if pmo and pmo.valid
			local dist = R_PointToDist2(pmo.x, pmo.y, mo.x, mo.y)
			
			if dist < pushradius
			and abs(pmo.z - mo.z) < FRACUNIT*256
				local pushforce = 20*FRACUNIT
				local pushfac = FixedDiv(pushforce, pushradius)
				local finalpush = pushforce - FixedMul(dist, pushfac)
				local ang = R_PointToAngle2(mo.x, mo.y, pmo.x, pmo.y)
				
				P_Thrust(pmo, ang, finalpush/4)
			end
		end
	end
	
end, MT_PSO_TELEPORTIN)