
PSO.info.items = {}

freeslot("SPR_SWD1", "SPR_RIF1", "SPR_ROD1")

rawset(_G, "HAND_BOTH", 0)
rawset(_G, "HAND_RIGHT", 2)
rawset(_G, "HAND_LEFT", 1)

-- list of available photon arts per weapon

local technique_list = {
	"foie",
	"gifoie",
	"rafoie",
	"shifta",
	"jellen",

	"barta",
	"gibarta",
	"rabarta",
	"deband",
	"zallure",
	
	"zonde",
	"gizonde",
	"razonde",
	
	"diga",
	"gidiga",
	"radiga",
	
	"grants",
	"gigrants",
	"ragrants",
	"resta",
	
	"megid",
	"gimegid",
	"ramegid",
}

PSO.info.weaponPhotonArts = {
	
	[WPT_SWORD] = {
		"sword_normal",
		"sword_pa_spindrop",
		"sword_pa_extendcutter",
		"sword_pa_spiralbreak",
	},
	
	[WPT_RIFLE] = {
		"rifle_normal",
		"rifle_charge",
		"rifle_pa_piercinground",
		"rifle_pa_slideshot",
	},
	
	--[WPT_ROD] = technique_list,
	
	[WPT_DOUBLESABER] = {
		"dsaber_normal",
		"dsaber_pa_deadlyarcher",
		"dsaber_pa_spinadvance",
		"dsaber_pa_tornadodance",
	},
	
	[WPT_SHOTGUN] = {
		"shotgun_normal",
		"shotgun_pa_zerodistance",
		"shotgun_pa_impactrecoil",
		"shotgun_pa_gloriousrain"
	},
	
	[WPT_SABER] = {
		"saber_normal",
		"saber_pa_crushingleap",
	},
	
	[WPT_HANDGUN] = {
		"handgun_normal",
		"handgun_pa_charge",
	},
	
	--[WPT_WAND] = technique_list,
	techniques = technique_list,
}

PSO.info.weaponDefaultActions = {
	
	[WPT_SWORD] = {
		"sword_normal",
		"sword_pa_spindrop",
	},
	
	[WPT_RIFLE] = {
		"rifle_normal",
		"rifle_charge",
	},
	
	[WPT_ROD] = {
		"barta",
		"foie",
		"zonde",
		"diga",
	},
	
	[WPT_DOUBLESABER] = {
		"dsaber_normal",
		"dsaber_pa_deadlyarcher",
	},
	
	[WPT_SHOTGUN] = {
		"shotgun_normal",
		"shotgun_pa_zerodistance",
	},
	
	[WPT_SABER] = {
		"saber_normal",
		"saber_pa_crushingleap",
	},
	
	[WPT_HANDGUN] = {
		"handgun_normal",
		"handgun_pa_charge",
	},
	
	[WPT_WAND] = {
		"barta",
		"foie"
	}
}

PSO.info.defaultSheathePosition = {
	
	[WPT_SWORD] = {
		16,	-- x (behind player)
		16,	-- y (behind player)
		100,	-- height % of player
		
		ANG1*90,	-- angle
		-ANG1*110,	-- rollangle	
	},
	
	[WPT_DOUBLESABER] = {
		16,
		16,
		70,
		
		ANG1*90,
		-ANG1*45,
	},
	
	[WPT_RIFLE] = {
		16,	-- x (behind player)
		16,	-- y (behind player)
		33,	-- height % of player
		
		ANG1*90,	-- angle
		ANG1*30,	-- rollangle
	},
	
	[WPT_SHOTGUN] = {
		16,	-- x (behind player)
		16,	-- y (behind player)
		33,	-- height % of player
		
		ANG1*90,	-- angle
		ANG1*30,	-- rollangle
	},	
	
	[WPT_ROD] = {
		16,	-- x (behind player)
		16,	-- y (behind player)
		75,	-- height % of player
		
		ANG1*90,	-- angle
		ANG1*110,	-- rollangle
	},

	[WPT_SABER] = {
		16,	-- x (behind player)
		16,	-- y (behind player)
		50,	-- height % of player
		
		ANG1*90,	-- angle
		-ANG1*60,	-- rollangle	
	},

	[WPT_HANDGUN] = {
		24,	-- x (behind player)
		24,	-- y (behind player)
		50,	-- height % of player
		
		ANG1*90,	-- angle
		ANG1*30,	-- rollangle
	},

	[WPT_WAND] = {
		16,	-- x (behind player)
		16,	-- y (behind player)
		55,	-- height % of player
		
		ANG1*90,	-- angle
		ANG1*110,	-- rollangle
	},	
	
}

-- default cast "art" for each weapon type
PSO.info.castArt = {
	[WPT_SWORD] = "rod_cast",
	[WPT_DOUBLESABER] = "rod_cast",
	[WPT_RIFLE] = "rod_cast",
	[WPT_SHOTGUN] = "rod_cast",
	[WPT_ROD] = "rod_cast",
	[WPT_SABER] = "rod_cast",
	[WPT_HANDGUN] = "rod_cast",
	[WPT_WAND] = "wand_cast",
}

PSO.info.weaponHand = {
	[WPT_SWORD] = HAND_BOTH,
	[WPT_DOUBLESABER] = HAND_BOTH,
	[WPT_RIFLE] = HAND_BOTH,
	[WPT_SHOTGUN] = HAND_BOTH,
	[WPT_ROD] = HAND_BOTH,
	[WPT_SABER] = HAND_RIGHT,
	[WPT_HANDGUN] = HAND_LEFT,
	[WPT_WAND] = HAND_RIGHT,
}

-- true if weapon should use techs
PSO.info.isTechWeapon = {
	[WPT_ROD] = true,
	[WPT_WAND] = true,
}

PSO.lib.itemInfo = function(it)
	if type(it) == "string"
		return PSO.info.items[it]
	elseif it
		return PSO.info.items[it.ref]
	else
		return nil
	end
end

PSO.lib.itemPrice = function(it)
	local info = PSO.lib.itemInfo(it)
	
	if info.cost
		return info.cost
	end
	
	if info.type == IT_WEAPON
		return it.atk*100*it.rarity/2
	
	elseif info.type == IT_ARMOR
		return it.def*100*it.rarity/2
	end
	
	return it.rarity*1000 or 1
end

-- In the context of getting a weapon or an armour, we need to create a new instance of it.
-- dta is an extra table:
-- edta[element, element value] for weapons and armours
PSO.lib.makeItem = function(it, edta)
	
	local dta = PSO.info.items[it]
	
	-- oops
	if not dta
		print("\x82".."WARNING:".." Item "..it.." doesn't exist")
		return it
	end
	
	if dta.type == IT_WEAPON
		
		local ret = {}
		
		ret.type = dta.type
		ret.weapontype = dta.weapontype
		ret.rarity = dta.rarity
		ret.atk = dta.atk
		ret.req = dta.req
		
		ret.range = dta.range
		ret.angle = dta.angle
		
		ret.pa_cost = dta.pa_cost or 100
		ret.pa_damage = dta.pa_damage or 100
		
		ret.level = 1	
		ret.ref = it	-- keep a reference
		ret.element = edta and edta[1] or ELEM_FIRE
		ret.elementv = edta and edta[2] or 0
		ret.grind = 0
		
		if not edta
			if dta.elem
				ret.element = dta.elem
				ret.elementv = P_RandomRange(min(60, ret.rarity*5), min(60, ret.rarity*10))
			else
				ret.element = P_RandomRange(ELEM_FIRE, ELEM_DARK)
				ret.elementv = P_RandomRange(0, min(60, ret.rarity*10))
			end
		end
		
		ret.actions = {}
		local actionlist = PSO.info.weaponDefaultActions[ret.weapontype]
		
		for i = 1, #actionlist
			ret.actions[i] = actionlist[i]
		end
		
		return ret
	elseif dta.type == IT_ARMOR
		
		local ret = {}
		ret.type = dta.type
		ret.rarity = dta.rarity
		ret.def = dta.def
		ret.req = dta.req
		
		ret.slots = {}
		ret.ref = it	-- keep a reference
		ret.element = edta and edta[1] or ELEM_FIRE
		ret.elementv = edta and edta[2] or 0
		ret.grind = 0

		if not edta
			if dta.element
				ret.element = dta.element
			else
				ret.element = P_RandomRange(ELEM_FIRE, ELEM_DARK)
			end
			
			ret.elementv = P_RandomRange(0, max(60, ret.rarity*10))
		end
		
		return ret
	elseif dta.type == IT_UNIT
		local ret = {}
		for k, v in pairs(dta)
			if k == "desc" continue end
			ret[k] = v
		end
		
		ret.ref = it
		
		return ret
	end
	
	return it	-- the string is fine.
end

PSO.lib.grindItem = function(it, grind_amount)
	local grindcount = 0
	if it.grind ~= nil
		for i = 1, (grind_amount or 1)
			if it.grind < PSO.info.MAXGRIND	
				
				local info = PSO.lib.itemInfo(it)
				local usestat = info.atk and "atk" or "def"
				
				it[usestat] = $ + 1 + (info.rarity-1)/3
				it.grind = $+1
				grindcount = $+1
			end
		end
		
		return grindcount > 0
	end
	return false	-- couldn't grind
end

PSO.lib.getItemIcon = function(it)
	
	local info = PSO.lib.itemInfo(it)
	
	if info.icon
		return info.icon
	else
		
		if info.type == IT_WEAPON
			return "H_WICO"..PSO.info.weaponnums[(info.weapontype or 1)]
		else
			return "H_IICO"..(info.type or 1)
		end
	end
end

PSO.lib.getItemName = function(it)
	
	if it.grind
		return PSO.lib.itemInfo(it).name.."+"..it.grind
	end
	
	return PSO.lib.itemInfo(it).name or "INVALID"
end	

PSO.info.weaponToIcon = {
	[WPT_SWORD] = "H_SWRD",
	[WPT_DOUBLESABER] = "H_DSABER",
	[WPT_RIFLE] = "H_RIFL",
	[WPT_SHOTGUN] = "H_SHOTG",
	[WPT_ROD] = "H_ROD",
	[WPT_SABER] = "H_SABR",
	[WPT_HANDGUN] = "H_HGUN",
	[WPT_WAND] = "H_WAND",
}

PSO.info.weaponDamageType = {
	[WPT_SWORD] = DMT_STRIKE,
	[WPT_DOUBLESABER] = DMT_STRIKE,
	[WPT_RIFLE] = DMT_RANGED,
	[WPT_SHOTGUN] = DMT_RANGED,
	[WPT_ROD] = DMT_TECHNIQUE,
	[WPT_SABER] = DMT_STRIKE,
	[WPT_HANDGUN] = DMT_RANGED,
	[WPT_WAND] = DMT_TECHNIQUE,
}

PSO.info.weaponDesc = {
	[WPT_SWORD] = "Two handed melee weapon. Large sword that excels in crowd control while remaining powerful in single combat.",
	[WPT_DOUBLESABER] = "Two handed melee weapon. A fast double bladed saber that excels at building chains and single target combat.",
	[WPT_RIFLE] = "Two handed ranged weapon. A medium range firearm that excels at building chains in single target combat.",
	[WPT_SHOTGUN] = "Two handed ranged weapon. A close range firearm with a wide bullet spread that excels at taking on groups of enemies.",
	[WPT_ROD] = "Two handed technique weapon. Allows using a versatile, highly potent array of Techniques.",
	[WPT_SABER] = "Right handed melee weapon. Allows using fast flurries of slashes. It's less powerful than a sword but much faster.",
	[WPT_HANDGUN] = "Left handed ranged weapon. Less powerful but more efficient than a Rifle, it's great for medium range combat.",
	[WPT_WAND] = "Right handed technique weapon. Allows casting techniques faster than Rods can, although they will be far less potent.",
}