
freeslot("sfx_sasfx1", "sfx_sasfx2", "sfx_sasfx3", "sfx_sasfx4", "sfx_sasfx5", "sfx_sasfx6", "sfx_sasfx7", "sfx_sasfx8", "sfx_sasfx9", "sfx_sasfxa", "sfx_sasfxb") 

PSO.info.SASoundNames = {
	"Normal",
	"Joy",
	"Anger",
	"Sorrow",
	"Unease",
	"Surprised",
	"Doubt",
	"Help",
	"Whistle",
	"Embarrassment",
	"Nailed it"
}

-- validates the payload
local function validatePayload(index, payload)

	if not payload.file
		print("\x85".."ERROR: ".."\x80".."No file for symbol art #"..index..", thus it cannot be added.")	
		return false
	end

	if not payload.name
		payload.name = "SYMBOL"..index
		print("\x82".."WARNING: ".."\x80".."No name for symbol art #"..index..", name SYMBOL"..index.." will be used instead.")
	end
	
	if not payload.sound
		payload.sound = sfx_sasfx1
		print("\x82".."WARNING: ".."\x80".."No sound for symbol art #"..index..", sound 1 will be used instead.")	
	elseif tonumber(payload.sound) < 0 or tonumber(payload.sound) > 11
		payload.sound = sfx_sasfx1
		print("\x82".."WARNING: ".."\x80".."Sound # out of range (1-11) for symbol art #"..index..", sound 1 will be used instead.")	
	else
		payload.sound = (tonumber($)-1) + sfx_sasfx1
	end
	
	return true
end

local function readSAFile(file)
	if not file
		return
	end
	
	local sa_read = {}
	
	local line = file:read()
	
	local index = 0
	local payload = {}
	
	local word_to_attr = {
		
		["NAME"] = "name",
		["FILE"] = "file",
		["SOUND"] = "sound",
	}
	
	while line
		
		-- kills leading/trailing spaces
		line = line:gsub("^%s*(.-)%s*$", "%1")
		--print(line)
		
		if line == "SYMBOL"
			if index
			and validatePayload(index, payload)
				sa_read[#sa_read+1] = payload
			end
			
			index = $+1
			
			if index > PSO.info.MAXSYMBOLARTS
				print("\x82".."WARNING: ".."\x80".."Cannot load more than "..PSO.info.MAXSYMBOLARTS.." symbol arts, aborting.")
				break
			end
			
			print("Reading symbol "..index.."...")
			payload = {}
		end
		
		for k,v in pairs(word_to_attr)
			if (line:sub(1, k:len()+1)) == k.." "
				local val = line:sub(k:len()+1, line:len()):gsub("^%s*(.-)%s*$", "%1")
				--print(k.." -> "..tostring(val))
				if val
					payload[v] = val
				end
			end
		end
		
		line = file:read()
	end
	
	-- validate the last SA
	if validatePayload(index, payload)
		sa_read[#sa_read+1] = payload
	end
	
	for i = 1, #sa_read
		print("SYMBOL "..i)
		print("=====================")
		local sa = sa_read[i]
		for k,v in pairs(sa)
			print(tostring(k).." :: "..tostring(v))
		end
		print("=====================")
	end
	
	server.sa_data = sa_read
end

-- loads SA config
PSO.lib.loadSymbolArtConf = function()
	io.open(PSO.info.SAFile, "r", readSAFile)
end

local bmp_data = {}

local function sa_readFile(f)
	
	local index = #bmp_data+1
	server.sa_data[index].initialized = true
	
	print("Loading image for SA #"..index.." ("..server.sa_data[index].name..")...")
	
	-- bruv!
	if not f
		local cdta = server.sa_data[index]
		local file = cdta.file
		
		print("\x82".."WARNING: ".."Couldn't load Symbol Art #"..(#bmp_data+1).."'s image ("..tostring(file).."): File does not exist.")
		bmp_data[index] = nil
		server.sa_data[index].bmp = nil
		return
	end
	
	local dta
	
	local run, res = pcall(function()
		dta = loadBMP(f)
	end)
	
	if not run
		local cdta = server.sa_data[index]
		local file = cdta.file	
		print("\x82".."WARNING: ".."Eerror(s) parsing Symbol Art #"..(#bmp_data+1).."'s image ("..tostring(file)..")")
		print(res)
		bmp_data[index] = nil
		server.sa_data[index].bmp = nil
	else
	
		bmp_data[index] = dta
		server.sa_data[index].bmp = dta
	end
end

local function serverLoadSymbolArts()
	
	if not server return end
	if not netgame return end	-- don't load SAs outside of netgames
	
	if not server.init_sa
		print("Loading symbol arts...")
		PSO.lib.loadSymbolArtConf()
		server.init_sa = true
	end
	
	if server.sa_data
	and not server.init_sadata
		
		-- empty this, we need this because we can't pass other args to the fucking callback function......
		bmp_data = {}
		
		for i = 1, #server.sa_data
			
			local sa = server.sa_data[i]
			
			if sa.file
				io.open("client/srb2i/symbolarts/"..sa.file, "rb", sa_readFile)
			end
		end
		
		server.init_sadata = true
	elseif server.sa_data
		local i = #server.sa_data
		while i
			-- rremove uninitialized SA
			local sa = server.sa_data[i]
			
			if sa.initialized	-- this is set when the file is read
			and not sa.bmp
				table.remove(server.sa_data, i)
			end
			
			i = $-1
		end
	end
end

PSO.lib.registerFrameFunc(serverLoadSymbolArts)

PSO.lib.drawSymbolArt = function(v, x, y, num)
	local dta = server.sa_data[num]
	
	if not dta return end
	if not dta.bmp return end
	
	drawBMP(v, x, y, dta.bmp)
end

-- test
/*hud.add(function(v, p, c)
	if server.sa_data
		for i = 1, #server.sa_data
			local dta = server.sa_data[i]
			
			if dta and dta.bmp
				drawBMP(v, 100, (i-1)*45, dta.bmp)
			end
			
		end
	end
end)*/