-- Rifle photon arts

freeslot("MT_PSO_RIFLESHOT")
freeslot("S_PSO_RIFLESHOT", "S_PSO_RIFLESHOT_EXPLODE")
mobjinfo[MT_PSO_RIFLESHOT] = {
	doomednum = -1,
	spawnstate = S_PSO_RIFLESHOT,
	deathstate = S_PSO_RIFLESHOT_EXPLODE,
	xdeathstate = S_PSO_RIFLESHOT_EXPLODE,
	height = 64*FRACUNIT,
	radius = 32*FRACUNIT,
	flags = MF_NOGRAVITY|MF_MISSILE,
	spawnhealth = 1000,
}

function A_RifleShotVisual(actor, var1, var2)
	
	if actor.extravalue1
		
		-- spawn some particle trail...
		local x = actor.x + P_RandomRange(-32, 32)*actor.scale
		local y = actor.y + P_RandomRange(-32, 32)*actor.scale
		local z = actor.z + P_RandomRange(-32, 32)*actor.scale
		
		local trail = P_SpawnMobj(x, y, z, MT_THOK)
		trail.scale = actor.scale*3/4
		trail.scalespeed = actor.scale/12
		trail.tics = 10
		trail.fuse = 10
		trail.frame = A
		trail.color = actor.color
	end
	
	if actor.extravalue2
	and leveltime & 1
		
		local trail = P_SpawnMobj(actor.x, actor.y, actor.z, MT_THOK)
		trail.color = SKINCOLOR_WHITE
		trail.scale = actor.scale*3/2
		trail.tics = 2
		trail.fuse = 2
		trail.frame = A|FF_FULLBRIGHT
		
	end
	
	if actor.prevpos
		for i = 0, 5
			local x = PSO.lib.lerp(actor.prevpos[1], actor.x, FRACUNIT/5 *i)
			local y = PSO.lib.lerp(actor.prevpos[2], actor.y, FRACUNIT/5 *i)
			local z = PSO.lib.lerp(actor.prevpos[3], actor.z, FRACUNIT/5 *i)
			
			local trail = P_SpawnMobj(x, y, z, MT_THOK)
			trail.scale = actor.scale*3/4
			trail.destscale = 1
			trail.scalespeed = actor.scale/12
			trail.tics = 10
			trail.fuse = 10
			trail.frame = A|TR_TRANS50
			trail.color = actor.color
		end
	end
	
	actor.prevpos = {actor.x, actor.y, actor.z}
end

function A_RifleShotExplode(actor, var1, var2)
	
	if not actor.extravalue2
		for i = 1, 8
			local t = P_SpawnMobj(actor.x, actor.y, actor.z, MT_THOK)
			t.color = actor.color
			t.scale = actor.scale*3/4
			t.destscale = 1
			t.scalespeed = actor.scale/12
			t.momx = P_RandomRange(-16, 16)*actor.scale
			t.momy = P_RandomRange(-16, 16)*actor.scale
			t.momz = P_RandomRange(-16, 16)*actor.scale
			t.frame = A|TR_TRANS10
			t.fuse = 20
			t.tics = 20
		end
	else
		for i = 1, 24
			local t = P_SpawnMobj(actor.x, actor.y, actor.z, MT_THOK)
			t.color = i & 1 and SKINCOLOR_WHITE or actor.color
			t.scale = actor.scale
			t.destscale = 1
			t.scalespeed = actor.scale/12
			t.momx = P_RandomRange(-16, 16)*actor.scale
			t.momy = P_RandomRange(-16, 16)*actor.scale
			t.momz = P_RandomRange(-16, 16)*actor.scale
			t.frame = A|TR_TRANS10
			t.fuse = 20
			t.tics = 20
		end	
		
		if not actor.cusval
			-- spawn an explosion hitbox:
			local h = PSO.lib.spawnHurtBox(actor.x, actor.y, actor.z, 200<<FRACBITS, 64<<FRACBITS, actor.target, actor.pso.potency-500, DMT_RANGED, 4)
			h.pso.whitelist = $ or {}
			h.pso.combobreaker = true
		
			-- search for the enemy we hit:
			local enemies = {}
			searchBlockmap("objects", 
				function(ref, found)
					
					if found and found.valid and found.flags & MF_SHOOTABLE
						enemies[#enemies+1] = found
						
					end
				end,
			actor)
			
			table.sort(enemies, 
				function(a, b)
					return R_PointToDist2(actor.x, actor.y, a.x, a.y) < R_PointToDist2(actor.x, actor.y, b.x, b.y)
				end
			)
			
			if enemies[1] and enemies[1].valid
				PSO.lib.dprint(enemies[1])
				h.pso.whitelist[#h.pso.whitelist+1] = enemies[1]
			end
		end
	end
end

states[S_PSO_RIFLESHOT] 		= {SPR_THOK, A|TR_TRANS30, 1, A_RifleShotVisual, 0, 0, S_PSO_RIFLESHOT}
states[S_PSO_RIFLESHOT_EXPLODE] = {SPR_THOK, A|TR_TRANS30, 1, A_RifleShotExplode, 0, 0, S_NULL}

PSO.info.photonArts["rifle_normal"] = {
		name = "Shot",
		desc = "A fast moving photon bullet that deals damage on contact.",
		normal = true,
		cost = 2,
		potency = 140,
		speed = 30,		-- speed % multiplier.
						-- 100: full speed
						-- 0: stand still
		
		icon = "H_RSHOT1",
		combo = TICRATE/2,		-- can be reused after 17 frames on the timer!
		
		-- p: player
		-- timer: how long the animation has played for
		-- hold: if true, button is being held down, if not, it's been released
		-- input: timer on how long the button has been held for
		-- combo: non-zero if the attack was performed from a combo.
					-- useful to create attacks that are faster/different if used repeatedly.
		
		-- return true to end the animation
		
		use = 	function(p, timer, hold, input, combo)
					
					local mo = p.mo
					-- rifle is always weapon slot 1
					local pos = p.pso.weaponpos[1]
					local potency = PSO.info.photonArts[p.pso.action].potency
					local wep = PSO.lib.getPlayerActiveWeapon(p)
					local winfo = PSO.lib.itemInfo(wep)
					
					--print(timer)
					-- set new weapon pos:
					local ang = p.drawangle - ANG1*30
					local x = 24*cos(ang)
					local y = 24*sin(ang)
					
					pos[2] = x
					pos[3] = y
					pos[5] = ANG1*10	-- lat. angle
					
					if timer == 1
						
						if combo
							pos[5] = FixedDiv(mo.height, mo.scale)/2
							pos[6] = 0
							
						else	-- we've just started that attack
							local z = FixedDiv(mo.height, mo.scale)
							pos[4] = z
								
							-- set angle
							pos[6] = -ANG1*90 -- vert. angle
						end
					end
					
					if timer < 5
					and not combo
						pos[6] = $ - ANG1*20
						pos[4] = $ - FixedDiv(mo.height, mo.scale)/10
					elseif timer == 5
						pos[6] = 0
					end
					
					
					if timer == 9
						pos[6] = ANG1*30
						
						-- fire the bullet!
						local msl = PSO.lib.spawnMissile(mo, MT_PSO_RIFLESHOT, p.drawangle, 65*FRACUNIT, potency, DMT_RANGED, pos[2], pos[3], mo.scale*30)
						msl.scale = $/3
						msl.color = PSO.info.elementColours[wep.element]
						msl.pso.potency = potency	-- normals shouldn't benefit from PA%
												
						if p.pso.lockon and p.pso.lockon.valid
							PSO.lib.aimMissile(msl, p.pso.lockon)
						end

						if winfo.sound
							S_StartSound(mo, winfo.sound)
						end

						-- make the missile fade after it gets out of the range.
						local range = wep.range or 512
						msl.fuse = FixedDiv(range*FRACUNIT, FixedHypot(msl.momx, msl.momy))/FRACUNIT
						
					elseif timer > 9 and timer < 13
						pos[6] = $ - ANG1*10
					end
					
					if timer == TICRATE
						return true
					end
				end,
}

PSO.info.photonArts["rifle_pa_slideshot"] = {
		name = "Slide Shot",
		desc = "A dodge manoeuver followed by a volley of 3 bullets towards the target",
		cost = 25,
		potency = 390,
		speed = 100,		-- speed % multiplier. (hack in this case because we'll set it to 0 mid-action)
						-- 100: full speed
						-- 0: stand still
		
		icon = "H_RSHOT4",
		
		-- p: player
		-- timer: how long the animation has played for
		-- hold: if true, button is being held down, if not, it's been released
		-- input: timer on how long the button has been held for
		-- combo: non-zero if the attack was performed from a combo.
					-- useful to create attacks that are faster/different if used repeatedly.
		
		-- return true to end the animation
		
		use = 	function(p, timer, hold, input, combo)
					
					local mo = p.mo
					-- rifle is always weapon slot 1
					local pos = p.pso.weaponpos[1]
					local potency = PSO.info.photonArts[p.pso.action].potency
					local wep = PSO.lib.getPlayerActiveWeapon(p)
					local winfo = PSO.lib.itemInfo(wep)
					
					--print(timer)
					-- set new weapon pos:
					local ang = p.drawangle - ANG1*30
					local x = 24*cos(ang)
					local y = 24*sin(ang)
					
					pos[2] = x
					pos[3] = y
					pos[5] = ANG1*10	-- lat. angle
					
					if timer == 1
						
						-- propel us depending on our forwardmove/sidemove
						local defaultang = ANG1*180
						local ang = mo.angle
						
						if abs(p.cmd.sidemove) < 25
							ang = mo.angle + defaultang
						else
							if p.cmd.sidemove >= 25
								ang = mo.angle - ANG1*90
							elseif p.cmd.sidemove <= -25
								ang = mo.angle + ANG1*90
							end
						end
						
						mo.z = $+2
						mo.momz = FRACUNIT*4
						mo.state = S_PLAY_FALL
						P_InstaThrust(mo, ang, FRACUNIT*32)
						p.pso.actionspeed = 0
						
						local z = FixedDiv(mo.height, mo.scale)
						pos[4] = z
								
						-- set angle
						pos[6] = ANG1*90 -- vert. angle
					end
					
					if timer < 5
					and not combo
						pos[6] = $ - ANG1*20
						pos[4] = $ - FixedDiv(mo.height, mo.scale)/10
					elseif timer == 5
						pos[6] = 0
					end
					
					if not P_IsObjectOnGround(mo)
						if leveltime & 1
							P_SpawnGhostMobj(mo)
						end
						mo.momx = FixedMul(mo.momx, mo.friction)
						mo.momy = FixedMul(mo.momy, mo.friction)
					end
					
					for i = 1, 3
					
						if timer == 6 + (i-1)*5
							pos[6] = ANG1*30
							
							-- fire the bullet!
							local msl = PSO.lib.spawnMissile(mo, MT_PSO_RIFLESHOT, p.drawangle, 65*FRACUNIT, potency/3, DMT_RANGED, pos[2], pos[3], mo.scale*30)
							msl.scale = $/3
							msl.color = PSO.info.elementColours[wep.element]
							msl.pso.potency = potency/3	-- normals shouldn't benefit from PA%
							msl.pso.combobreaker = true
													
							if p.pso.lockon and p.pso.lockon.valid
								PSO.lib.aimMissile(msl, p.pso.lockon)
							end

							if winfo.sound
								S_StartSound(mo, winfo.sound)
							end

							-- make the missile fade after it gets out of the range.
							local range = wep.range or 512
							msl.fuse = FixedDiv(range*FRACUNIT, FixedHypot(msl.momx, msl.momy))/FRACUNIT
							
						elseif timer > 6 + (i-1)*5 and timer < 10 + (i-1)*5
							pos[6] = $ - ANG1*10
						end
					end
					
					if timer == TICRATE-5
						return true
					end
				end,
}

-- PIERCINGROUND
freeslot("MT_PIERCINGROUND", "S_PIERCINGROUND", "SPR_SAIR")
mobjinfo[MT_PIERCINGROUND] = {
	doomednum = -1,
	spawnstate = S_PIERCINGROUND,
	deathstate = S_NULL,
	xdeathstate = S_NULL,
	radius = 48<<FRACBITS,
	height = 24<<FRACBITS,
	flags = MF_MISSILE|MF_NOGRAVITY,
}

-- disable collision for barta
addHook("MobjCollide", function(mo1, mo2) return false end, MT_PIERCINGROUND)
addHook("MobjMoveCollide", function(mo1, mo2) return false end, MT_PIERCINGROUND)

function A_PiercingRound(mo)
	
	mo.friction = FRACUNIT	-- no slowdown
	if not mo.hurtbox
		mo.hurtbox = PSO.lib.spawnHurtBox(mo.x, mo.y, mo.z, 96<<FRACBITS, 64<<FRACBITS, mo.target, mo.pso.potency, DMT_RANGED, TICRATE*999)
		mo.hurtbox.pso.combobreaker = true
	else
		mo.hurtbox.fuse = 3
		P_MoveOrigin(mo.hurtbox, mo.x, mo.y, mo.z)
	end
	
	if leveltime%5 == 0
		local t = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
		t.sprite = SPR_SAIR
		t.frame = A
		t.fuse = 15
		t.tics = 10
		t.scale = mo.scale
		t.destscale = mo.scale*8
		t.scalespeed = mo.scale/4
		t.color = SKINCOLOR_GREY
	end
	
	mo.coloorized = true
	mo.color = SKINCOLOR_GREY
	
	A_RifleShotVisual(mo)
end

states[S_PIERCINGROUND] = {SPR_TRLS, A, 1, A_PiercingRound, 0, 0, S_PIERCINGROUND}

PSO.info.photonArts["rifle_pa_piercinground"] = {
		name = "Piercing Round",
		desc = "Fires a sharp metal shell that can penetrate targets",
		cost = 15,
		potency = 300,
		speed = 30,		-- speed % multiplier.
						-- 100: full speed
						-- 0: stand still
		
		icon = "H_RSHOT3",
		combo = TICRATE/2,		-- can be reused after 17 frames on the timer!
		
		-- p: player
		-- timer: how long the animation has played for
		-- hold: if true, button is being held down, if not, it's been released
		-- input: timer on how long the button has been held for
		-- combo: non-zero if the attack was performed from a combo.
					-- useful to create attacks that are faster/different if used repeatedly.
		
		-- return true to end the animation
		
		use = 	function(p, timer, hold, input, combo)
					
					local mo = p.mo
					-- rifle is always weapon slot 1
					local pos = p.pso.weaponpos[1]
					local potency = PSO.info.photonArts[p.pso.action].potency
					local wep = PSO.lib.getPlayerActiveWeapon(p)
					local winfo = PSO.lib.itemInfo(wep)
					
					--print(timer)
					-- set new weapon pos:
					local ang = p.drawangle - ANG1*30
					local x = 24*cos(ang)
					local y = 24*sin(ang)
					
					pos[2] = x
					pos[3] = y
					pos[5] = ANG1*10	-- lat. angle
					
					if timer == 1
						
						if combo
							pos[5] = FixedDiv(mo.height, mo.scale)/2
							pos[6] = 0
							
						else	-- we've just started that attack
							local z = FixedDiv(mo.height, mo.scale)
							pos[4] = z
								
							-- set angle
							pos[6] = ANG1*90 -- vert. angle
						end
					end
					
					if timer < 5
					and not combo
						pos[6] = $ - ANG1*20
						pos[4] = $ - FixedDiv(mo.height, mo.scale)/10
					elseif timer == 5
						pos[6] = 0
					end
					
					
					if timer == 9
						pos[6] = ANG1*30
						
						-- fire the bullet!
						local msl = PSO.lib.spawnMissile(mo, MT_PIERCINGROUND, p.drawangle, 65*FRACUNIT, potency, DMT_RANGED, pos[2], pos[3], mo.scale*30)
						msl.scale = $/3
						msl.color = PSO.info.elementColours[wep.element]
						msl.pso.potency = potency	-- normals shouldn't benefit from PA%
												
						if p.pso.lockon and p.pso.lockon.valid
							PSO.lib.aimMissile(msl, p.pso.lockon)
						end

						if winfo.sound
							S_StartSound(mo, winfo.sound)
						end

						-- make the missile fade after it gets out of the range.
						local range = wep.range or 512
						msl.fuse = FixedDiv(range*FRACUNIT, FixedHypot(msl.momx, msl.momy))/FRACUNIT
						
					elseif timer > 9 and timer < 13
						pos[6] = $ - ANG1*10
					end
					
					if timer == TICRATE
						return true
					end
				end,
}

PSO.info.photonArts["rifle_charge"] = {
		name = "Charge Shot",
		desc = "Fire a powerful bullet. Charge and release to power the attack up and make it explode on impact",
		cost = 20,
		potency = 400,
		speed = 20,
		
		icon = "H_RSHOT2",
		
		use = 	function(p, timer, hold, input)
					
					local mo = p.mo
					-- rifle is always weapon slot 1
					local pos = p.pso.weaponpos[1]
					local potency = PSO.info.photonArts[p.pso.action].potency
					local wep = PSO.lib.getPlayerActiveWeapon(p)
					local winfo = PSO.lib.itemInfo(wep)
					--print(timer)
					-- set new weapon pos:
					local ang = p.drawangle - ANG1*30
					local x = 24*cos(ang)
					local y = 24*sin(ang)
					
					local chargethreshold = PSO.lib.hasSkill(p, "ra_quickload") and TICRATE*2/3 or TICRATE
					
					pos[2] = x
					pos[3] = y
					pos[5] = ANG1*10	-- lat. angle
					
					if timer == 1
						local z = FixedDiv(mo.height, mo.scale)
						pos[4] = z
							
						-- set angle
						pos[6] = ANG1*90 -- vert. angle
					end
					
					if timer < 5
						pos[6] = $ - ANG1*20
						pos[4] = $ - FixedDiv(mo.height, mo.scale)/10
					elseif timer == 5
						pos[6] = 0
					end
					
					if timer >= 5
					and timer < chargethreshold
						
						-- show the charge...
						local cang = p.drawangle - ANG1*5
						local cx = 50*cos(cang)
						local cy = 50*sin(cang)
						
						if timer == 5
							S_StartSound(mo, sfx_s3ka3)
						end
						
						if leveltime & 1
							local t = P_SpawnMobj(mo.x + FixedMul(cx, mo.scale), mo.y + FixedMul(cy, mo.scale), mo.z + mo.height*2/3, MT_THOK)
							t.tics = 2
							t.fuse = 2
							t.scale = mo.scale/3
							t.frame = A
							t.color = PSO.info.elementColours[wep.element]
							
							local t2 = P_SpawnMobj(t.x + P_RandomRange(-64, 64)*mo.scale, t.y + P_RandomRange(-64, 64)*mo.scale, t.z + P_RandomRange(-64, 64)*mo.scale, MT_THOK)
							t2.scale = mo.scale/4
							t2.destscale = mo.scale/8
							t2.scalespeed = mo.scale/12
							t2.frame = A
							t2.color = PSO.info.elementColours[wep.element]
							
							t2.momx = (t.x - t2.x)/10
							t2.momy = (t.y - t2.y)/10
							t2.momz = (t.z - t2.z)/10
							
							t2.fuse = 10
							t2.tics = 10
							
						end
					end
					
					if timer >= chargethreshold*2/3
					and timer < TICRATE*20
					and not hold
						-- just let it go.
						S_StopSound(mo)
						p.pso.actiontimer = timer < chargethreshold and TICRATE*20 or TICRATE*30	-- first firing state.
						pos[6] = timer < chargethreshold and ANG1*40 or ANG1*90
						
						local msl = PSO.lib.spawnMissile(mo, MT_PSO_RIFLESHOT, p.drawangle, 70*FRACUNIT, potency, DMT_RANGED, pos[2], pos[3], pos[4])
						msl.pso.combobreaker = true	-- charge shots are always combo breakers
						msl.extravalue1 = 1
						msl.color = PSO.info.elementColours[wep.element]
						
						if p.pso.lockon and p.pso.lockon.valid
							PSO.lib.aimMissile(msl, p.pso.lockon)
						end

						local range = wep.range or 512
						msl.fuse = FixedDiv(range*FRACUNIT, FixedHypot(msl.momx, msl.momy))/FRACUNIT

						if winfo.sound
							S_StartSound(mo, winfo.sound)
						end
						S_StartSound(mo, sfx_s3k83)

						if timer >= chargethreshold
							msl.pso.potency = $+200
							msl.extravalue2 = 1
							
							P_InstaThrust(mo, p.drawangle, -10*FRACUNIT)
							p.skidtime = 20
							mo.saveang = p.drawangle
							p.pso.actionspeed = 0	-- remove all cmd prompts.
							p.drawangle = mo.angle
							mo.state = S_PLAY_WALK
						else
							msl.scale = $ / 2
						end
					end
					
					-- if we keep holding we'll eventually reach full charge!
					if timer == chargethreshold
						S_StopSound(mo)
						local g = P_SpawnGhostMobj(mo)
						g.destscale = mo.scale*4
						g.scalespeed = mo.scale/12
						S_StartSound(mo, sfx_s3ka8)
					end
					
					if timer >= TICRATE*20
					and timer < TICRATE*20 + 5
						pos[6] = $ - ANG1*10
					elseif timer == TICRATE*21
						return true
					end
					
					
					if timer >= TICRATE*30
					and timer < TICRATE*31
					and mo.saveang
						p.drawangle = mo.saveang
						p.cmd.angleturn = mo.saveang/FRACUNIT
					end	
					
					if timer >= TICRATE*30
					and timer < TICRATE*30 + 10
						pos[6] = $ - ANG1*10
					elseif timer == TICRATE*31
						P_InstaThrust(mo, 0, 0)
						p.drawangle = mo.saveang
						return true
					end
					
					-- hack to prevent us from reaching firing states.
					if p.pso.actiontimer > TICRATE*10
					and p.pso.actiontimer < TICRATE*20	-- if we're past here, don't do it.
						p.pso.actiontimer = TICRATE*10
					end
					
					
					/*if timer == 9
						pos[6] = -ANG1*30
						
						-- fire the bullet!
						PSO.lib.spawnMissile(mo, MT_ROCKET, p.drawangle, 40*FRACUNIT, potency, DMT_RANGED, pos[2], pos[3], pos[4])
						
					elseif timer > 9 and timer < 13
						pos[6] = $ + ANG1*10
					end
					
					if timer == TICRATE/2
						return true
					end*/
				end,
		
}