-- Rod cast and base technique definition

freeslot("sfx_foie", "sfx_barta", "sfx_zonde", "sfx_shifta", "sfx_deband")

PSO.info.photonArts["foie"] = {
		-- PA is a technique
		technique = true,
		name = "Foie",
		desc = "Fire Technique. Launches a fireball that explodes on impact with an enemy",
		cost = 14,
		potency = 310,
		icon = "H_TFOIE",
		casttime = TICRATE*3/4,	-- how long it takes before the tech weapon swings to cast.
		
		sound = sfx_foie,
		
		aim = true,				-- tech can be aimed up/down
		reaction = RT_KNOCKBACK, -- foie does knockback
		element = ELEM_FIRE,
		object = MT_FOIE,		-- object to spawn
		speed = 40,				-- projectile speed.
		actionspeed = 40,
		spawndist = 32,			-- distance to spawn from the player if spawntarget is false
		spawntarget = false,	-- if true, spawns ontop of the target, or somewhere in front of the player if no target is detected.
		spawnheight = 50,		-- height % to spawn the object at
		
}

PSO.info.photonArts["gifoie"] = {
		-- PA is a technique
		technique = true,
		name = "Gifoie",
		desc = "Fire Technique. Creates a ring of fire around the caster that knocks enemies away.",
		cost = 22,
		actionspeed = 20,
		potency = 270,
		icon = "H_TGFOIE",
		casttime = TICRATE,	-- how long it takes before the tech weapon swings to cast.
		
		sound = sfx_foie,

		reaction = RT_KNOCKBACK, -- foie does knockback
		element = ELEM_FIRE,
		spawndist = 0,			-- distance to spawn from the player if spawntarget is false
		spawntarget = false,	-- if true, spawns ontop of the target, or somewhere in front of the player if no target is detected.

		object = MT_THOK,
		
		spawnfunc = function(msl)
			for i = 1, 24
				local ang = (360/24)*i*ANG1
				local e = P_SpawnMobj(msl.x, msl.y, msl.z, MT_SONIC3KBOSSEXPLODE)
				e.scale = msl.scale*3/2
				P_InstaThrust(e, ang, FRACUNIT*48)
			end

			-- spawn an explosion hitbox:
			local h = PSO.lib.spawnHurtBox(msl.x, msl.y, msl.z, 455<<FRACBITS, 128<<FRACBITS, msl.target, msl.pso.potency, DMT_TECHNIQUE, 4, RT_KNOCKBACK)
			h.pso.whitelist = $ or {}
			h.pso.reactionradialangle = true
			h.pso.element = ELEM_FIRE
			h.pso.combobreaker = msl.pso.combobreaker

			P_RemoveMobj(msl)
		end,
	
}

PSO.info.photonArts["rafoie"] = {
		-- PA is a technique
		technique = true,
		name = "Rafoie",
		desc = "Fire Technique. Cretaes a flame explosion at the target's location",
		cost = 16,
		actionspeed = 35,
		potency = 250,
		icon = "H_TRFOIE",
		casttime = TICRATE*2/3,	-- how long it takes before the tech weapon swings to cast.
		
		sound = sfx_foie,
		
		reaction = RT_KNOCKBACK, -- foie does knockback
		element = ELEM_FIRE,
		spawndist = 128,		-- distance to spawn from the player if spawntarget is false
		spawntarget = true,	-- if true, spawns ontop of the target, or somewhere in front of the player if no target is detected.
		spawnheight = 50,		-- height % to spawn the object at

		object = MT_THOK,
		
		spawnfunc = function(msl)
			for i = 1, 24
				local e = P_SpawnMobj(msl.x, msl.y, msl.z, MT_SONIC3KBOSSEXPLODE)
				e.scale = msl.scale*3/2
				e.destscale = 1
				e.scalespeed = msl.scale/12
				e.momx = P_RandomRange(-32, 32)*msl.scale
				e.momy = P_RandomRange(-32, 32)*msl.scale
				e.momz = P_RandomRange(-32, 32)*msl.scale
			end

			for i = 1, 10
				local e = P_SpawnMobj(msl.x + P_RandomRange(-64, 64)*msl.scale, msl.y + P_RandomRange(-64, 64)*msl.scale, msl.z, MT_SONIC3KBOSSEXPLODE)
				e.scale = msl.scale*3/2
				e.destscale = 1
				e.scalespeed = msl.scale/12
				e.momx = P_RandomRange(-6, 6)*msl.scale
				e.momy = P_RandomRange(-6, 6)*msl.scale
				e.momz = P_RandomRange(24, 32)*msl.scale
			end

			-- spawn an explosion hitbox:
			local h = PSO.lib.spawnHurtBox(msl.x, msl.y, msl.z, 200<<FRACBITS, 64<<FRACBITS, msl.target, msl.pso.potency, DMT_TECHNIQUE, 4, RT_KNOCKBACK)
			h.angle = msl.angle
			h.pso.element = ELEM_FIRE
			h.pso.combobreaker = msl.pso.combobreaker

			P_RemoveMobj(msl)
		end,
	
}

PSO.info.photonArts["barta"] = {
		-- PA is a technique
		technique = true,
		name = "Barta",
		desc = "Ice Technique. Launches an icicle that pierces through enemies.",
		cost = 10,
		actionspeed = 30,
		potency = 170,
		icon = "H_TBARTA",
		casttime = TICRATE/3,
		
		sound = sfx_barta,
		
		object = MT_BARTA,
		element = ELEM_ICE,
		speed = 30,	-- projectile speed
		spawndist = 32,			-- distance to spawn from the player if spawntarget is false
}

PSO.info.photonArts["gibarta"] = {
		-- PA is a technique
		technique = true,
		name = "Gibarta",
		desc = "Ice Technique. Surround yourself in sharp ice spikes, damaging enemies.",
		cost = 19,
		actionspeed = 30,
		potency = 270,
		icon = "H_TGBART",
		casttime = TICRATE/2,
		
		sound = sfx_barta,
		
		object = MT_THOK,
		element = ELEM_ICE,
		spawndist = 0,			-- distance to spawn from the player if spawntarget is false

		spawnfunc = function(msl)
			for i = 1, 3
				local dist = 64*i
				
				for k = 1, 16
					local ang = (360/16)*k*ANG1
					local x = msl.x + dist*cos(ang)
					local y = msl.y + dist*sin(ang)
					
					local e = P_SpawnMobj(x, y, msl.z, MT_THOK)
					e.sprite = SPR_FHZI
					e.scale = 1
					e.destscale = FRACUNIT
					e.scalespeed = FRACUNIT/2 /i
					e.frame = A
					e.fuse = TICRATE/3 + i*8
					e.tics = e.fuse
					
					if i == 3
						e.spriteyscale = $*4 + P_RandomRange(-FRACUNIT/2, FRACUNIT/2 -1)
					end
				end
			end

			-- spawn an explosion hitbox:
			local h = PSO.lib.spawnHurtBox(msl.x, msl.y, msl.z, 455<<FRACBITS, 128<<FRACBITS, msl.target, msl.pso.potency, DMT_TECHNIQUE, 4, RT_STAGGER)
			h.pso.whitelist = $ or {}
			h.pso.reactionradialangle = true
			h.pso.element = ELEM_ICE
			h.pso.combobreaker = msl.pso.combobreaker

			P_RemoveMobj(msl)
		end,		
}

PSO.info.photonArts["rabarta"] = {
		-- PA is a technique
		technique = true,
		name = "Rabarta",
		desc = "Ice technique. Creates an icicle below the targetted enemy",
		cost = 16,
		actionspeed = 35,
		potency = 250,
		icon = "H_TRBART",
		casttime = TICRATE/2,	-- how long it takes before the tech weapon swings to cast.
		
		sound = sfx_barta,
		
		reaction = RT_KNOCKBACK, -- foie does knockback
		element = ELEM_ICE,
		spawndist = 128,		-- distance to spawn from the player if spawntarget is false
		spawntarget = true,	-- if true, spawns ontop of the target, or somewhere in front of the player if no target is detected.

		object = MT_THOK,
		
		spawnfunc = function(msl)
			
			for i = 1, 4
				
				local offs = {0, 0, 0}
				if i > 1
					offs = {P_RandomRange(-48, 48), P_RandomRange(-48, 48), 0}
				end
				
				local e = P_SpawnMobj(msl.x + offs[1]*msl.scale, msl.y + offs[2]*msl.scale, msl.z, MT_THOK)
				e.sprite = SPR_FHZI
				e.scale = 1
				e.destscale = FRACUNIT
				e.scalespeed = FRACUNIT/3 /i
				e.fuse = TICRATE + (i-1)*6
				e.tics = e.fuse
				e.frame = A
				e.spriteyscale = FRACUNIT*6 / i
			end
					
			-- spawn an explosion hitbox:
			local h = PSO.lib.spawnHurtBox(msl.x, msl.y, msl.z, 128<<FRACBITS, 64<<FRACBITS, msl.target, msl.pso.potency, DMT_TECHNIQUE, 4, RT_STAGGER)
			h.pso.element = ELEM_ICE
			h.pso.combobreaker = msl.pso.combobreaker

			P_RemoveMobj(msl)
		end,
	
}



PSO.info.photonArts["zonde"] = {
		-- PA is a technique
		technique = true,
		name = "Zonde",
		desc = "Electric Technique. Summons a thunderbolt on the targetted enemy.",
		cost = 9,
		potency = 200,
		actionspeed = 30,
		icon = "H_TZONDE",
		casttime = TICRATE/3,
		sound = sfx_zonde,
		
		spawntarget = true,
		element = ELEM_ELEC,
		spawndist = 48,
		speed = 30,
		
		object = MT_THOK,
		
		spawnfunc = function(msl)
			local zonde = P_SpawnMobj(msl.x, msl.y, msl.z, MT_THOK)
			zonde.state = S_ZONDE
			zonde.color = SKINCOLOR_GOLD
			local hb = PSO.lib.spawnHurtBox(zonde.x, zonde.y, zonde.z, 32<<FRACBITS, 180<<FRACBITS, msl.target, msl.pso.potency, DMT_TECHNIQUE, 4)
			hb.pso.combobreaker = msl.pso.combobreaker
			hb.pso.element = ELEM_ELEC

			P_RemoveMobj(msl)

			return zonde
		end,
}

PSO.info.photonArts["gizonde"] = {
		-- PA is a technique
		technique = true,
		name = "Gizonde",
		desc = "Electric Technique. Electrifies the target with a current that spreads to up to 5 nearby enemies, damaging all of them.",
		cost = 18,
		potency = 200,
		actionspeed = 30,
		icon = "H_TGZOND",
		casttime = TICRATE/3,
		sound = sfx_zonde,
		
		spawntarget = true,
		element = ELEM_ELEC,
		spawndist = 48,
		speed = 30,
		
		object = MT_THOK,
		
		spawnfunc = function(msl)
			
			local curr_target = msl.target.player.pso.target
			local check_radius = 256
			msl.targets = {}
			
			if curr_target
				
				while curr_target and #msl.targets < 5
					msl.targets[#msl.targets+1] = curr_target

					local hb = PSO.lib.spawnHurtBox(curr_target.x, curr_target.y, curr_target.z, 32<<FRACBITS, 180<<FRACBITS, msl.target, msl.pso.potency, DMT_TECHNIQUE, 4)
					hb.pso.combobreaker = msl.pso.combobreaker
					hb.pso.locks = 1
					hb.pso.element = ELEM_ELEC

					for i = 1, 8
						local s = P_SpawnMobj(curr_target.x, curr_target.y, curr_target.z, MT_SUPERSPARK)
						s.colorized = true
						s.scale = $*3/2
						s.color = SKINCOLOR_GOLD
						s.momx = P_RandomRange(-32, 32)*FRACUNIT
						s.momy = P_RandomRange(-32, 32)*FRACUNIT
						s.momz = P_RandomRange(-32, 32)*FRACUNIT
					end
					
					local newtarget
					local x1 = curr_target.x - check_radius*FRACUNIT
					local x2 = curr_target.x + check_radius*FRACUNIT
					local y1 = curr_target.y - check_radius*FRACUNIT
					local y2 = curr_target.y + check_radius*FRACUNIT
					
					local targets = {}
					
					local function searchTarget(ref, found)
						if ref != found
						and found and found.valid	
							
							if found.flags & MF_SHOOTABLE
								
								local dupe
								for i = 1, #msl.targets
									--print(tostring(msl.targets[i]).."/"..tostring(found))
									if msl.targets[i] == found
										dupe = true
										break
									end
								end
								
								if not dupe
									--print("add: "..tostring(found))
									targets[#targets+1] = found
								end
							end
						end
					end
					
					searchBlockmap("objects", searchTarget, curr_target, x1, x2, y1, y2)
					
					-- sort the target list by distance
					table.sort(targets, function(a, b)
						return R_PointToDist2(curr_target.x, curr_target.y, a.x, a.y) < R_PointToDist2(curr_target.x, curr_target.y, b.x, b.y)
					end)
					
					-- if we have no target to begin with this will be nil and end the loop.
					local newtarget = targets[1]
					if newtarget and newtarget.valid
						-- spawn a chain between curr_target and the newtarget
						
						--local dist = R_PointToDist2(curr_target.x, curr_target.y, newtarget.x, newtarget.y)
						for i = 1, 5
							
							local x = PSO.lib.lerp(curr_target.x, newtarget.x, FRACUNIT/5 *i)
							local y = PSO.lib.lerp(curr_target.y, newtarget.y, FRACUNIT/5 *i)
							local z = PSO.lib.lerp(curr_target.z, newtarget.z, FRACUNIT/5 *i)
							
							local s = P_SpawnMobj(x, y, z, MT_SUPERSPARK)
							s.colorized = true
							s.color = SKINCOLOR_GOLD
							s.momx = P_RandomRange(-4, 4)*FRACUNIT
							s.momy = P_RandomRange(-4, 4)*FRACUNIT
							s.momz = P_RandomRange(-4, 4)*FRACUNIT
						end
					end
					
					curr_target = newtarget
				end
			else
				for i = 1, 8
					local s = P_SpawnMobj(msl.x, msl.y, msl.z, MT_SUPERSPARK)
					s.colorized = true
					s.color = SKINCOLOR_GOLD
					s.momx = P_RandomRange(-32, 32)*FRACUNIT
					s.momy = P_RandomRange(-32, 32)*FRACUNIT
					s.momz = P_RandomRange(-32, 32)*FRACUNIT
				end
			end

			P_RemoveMobj(msl)
		end,
}

PSO.info.photonArts["razonde"] = {
		-- PA is a technique
		technique = true,
		name = "Razonde",
		desc = "Electric Technique. Summons a field at the target location that periodically electrifies all enemies caught within until it disspates.",
		cost = 25,
		potency = 200,
		actionspeed = 30,
		icon = "H_TRZOND",
		casttime = TICRATE/3,
		sound = sfx_zonde,
		
		spawntarget = true,
		element = ELEM_ELEC,
		spawndist = 0,
		speed = 30,
		
		object = MT_RAZONDE,
		
		spawnfunc = function(msl)
			-- make sure razonde cannot have more than 1 instance per player
			local mo = msl.target
			
			if mo.lastrazonde
			and mo.lastrazonde.valid
				P_RemoveMobj(mo.lastrazonde)	-- don't stack!!
			end
			
			mo.lastrazonde = msl
			msl.tics = TICRATE*10 + TICRATE/3
		end,
}


PSO.info.photonArts["diga"] = {
		-- PA is a technique
		technique = true,
		name = "Diga",
		desc = "Earth Technique. Sends a slow, powerful rock flying towards the enemy.",
		cost = 16,
		potency = 410,
		icon = "H_TDIGA",
		casttime = TICRATE,	-- how long it takes before the tech weapon swings to cast.
		
		sound = sfx_rocks2,
		
		--aim = true,				-- tech can be aimed up/down
		reaction = RT_BLOWDOWN,
		element = ELEM_EARTH,
		object = MT_DIGA,		-- object to spawn
		speed = 20,				-- projectile speed.
		actionspeed = 40,
		spawndist = 32,			-- distance to spawn from the player if spawntarget is false
		spawntarget = false,	-- if true, spawns ontop of the target, or somewhere in front of the player if no target is detected.
		spawnheight = 50,		-- height % to spawn the object at
		
		spawnfunc = function(msl)
			
			local mo = msl.target
			msl.color = SKINCOLOR_ORANGE
			msl.colorized = true
			msl.scale = $*3/2
			
			if mo and mo.valid and mo.player
				
				local p = mo.player
				if p.pso.target and p.pso.target.valid
					PSO.lib.lobMissile(msl, p.pso.target)
				else
					msl.momz = FRACUNIT*12
				end
			end
			
		end,
		
}


PSO.info.photonArts["gidiga"] = {
		-- PA is a technique
		technique = true,
		name = "Gidiga",
		desc = "Earth Technique. Creates a localized earthquake around the user, dealing high damage and sending nearby enemies flying.",
		cost = 27,
		potency = 360,
		icon = "H_TGDIGA",
		casttime = TICRATE + TICRATE/5,	-- how long it takes before the tech weapon swings to cast.
		
		sound = sfx_rocks4,
		
		aim = true,				-- tech can be aimed up/down
		reaction = RT_BLOWDOWN,
		element = ELEM_EARTH,
		object = MT_THOK,
		speed = 20,				-- projectile speed.
		actionspeed = 10,
		spawndist = 0,			-- distance to spawn from the player if spawntarget is false
		spawntarget = false,	-- if true, spawns ontop of the target, or somewhere in front of the player if no target is detected.
		spawnheight = 0,		-- height % to spawn the object at
		
		spawnfunc = function(msl)
			
			local mo = msl.target
			local range = 512
			
			if mo and mo.valid and mo.player
				
				if mo.player == consoleplayer
					P_StartQuake(FRACUNIT*30, 8)
				end
				
				for i = 1, 64
						
					local x = mo.x + P_RandomRange(-range, range)*mo.scale
					local y = mo.y + P_RandomRange(-range, range)*mo.scale
					
					local t = P_SpawnMobj(x, y, mo.z, MT_THOK)
					t.state = S_ROCKCRUMBLEA
					t.fuse = P_RandomRange(10, 20)
					t.scale = mo.scale/3
					
					t.momz = P_RandomRange(2, 16)*mo.scale
					t.momx = P_RandomRange(-2, 2)*mo.scale
					t.momy = P_RandomRange(-2, 2)*mo.scale
					
					t.color = SKINCOLOR_ORANGE
					t.colorized = true
				end
				
				
				local x1 = mo.x - range*FRACUNIT
				local x2 = mo.x + range*FRACUNIT
				local y1 = mo.y - range*FRACUNIT
				local y2 = mo.y + range*FRACUNIT
				
				local fn = function(mo, found)
					
					if found and found.valid
					and R_PointToDist2(mo.x, mo.y, found.x, found.y) < range*FRACUNIT
						
						local hb = PSO.lib.spawnHurtBox(found.x, found.y, found.z, 32<<FRACBITS, 180<<FRACBITS, msl.target, msl.pso.potency, DMT_TECHNIQUE, 4, RT_BLOWDOWN)
						hb.pso.combobreaker = msl.pso.combobreaker
						hb.pso.reactionradialangle = true
						hb.pso.element = ELEM_EARTH
					end
				end
				
				searchBlockmap("objects", fn, mo, x1, x2, y1, y2)
			end
			
			P_RemoveMobj(msl)
		end,
		
}


PSO.info.photonArts["radiga"] = {
		-- PA is a technique
		technique = true,
		name = "Radiga",
		desc = "Earth Technique. Lifts rocks from below the target to pierce them, dealing high damage and sending them flying.",
		cost = 19,
		potency = 360,
		icon = "H_TRDIGA",
		casttime = TICRATE,	-- how long it takes before the tech weapon swings to cast.
		
		sound = sfx_rocks3,
		
		aim = true,				-- tech can be aimed up/down
		reaction = RT_BLOWDOWN,
		element = ELEM_EARTH,
		object = MT_THOK,
		actionspeed = 30,
		spawndist = 96,			-- distance to spawn from the player if spawntarget is false
		spawntarget = true,	-- if true, spawns ontop of the target, or somewhere in front of the player if no target is detected.
		spawnheight = 50,		-- height % to spawn the object at
		
		spawnfunc = function(msl)
			
			local mo = msl.target
			
			for i = 1, 8
				
				local x = msl.x + P_RandomRange(-48, 48)*msl.scale
				local y = msl.y + P_RandomRange(-48, 48)*msl.scale
				
				local r = P_SpawnMobj(x, y, msl.z, MT_THOK)
				r.state = S_ROCKCRUMBLEA
				r.fuse = P_RandomRange(20, 35)
				r.scale = 1
				r.destscale = FRACUNIT
				r.scalespeed = FRACUNIT / P_RandomRange(3, 8)
					
				r.momz = P_RandomRange(4, 16)*mo.scale
				
				r.spriteyscale = FRACUNIT*2 + P_RandomRange(0, FRACUNIT-7)
				
				r.color = SKINCOLOR_ORANGE
				r.colorized = true
			end

			local hb = PSO.lib.spawnHurtBox(msl.x, msl.y, msl.z, 32<<FRACBITS, 180<<FRACBITS, msl.target, msl.pso.potency, DMT_TECHNIQUE, 4, RT_BLOWDOWN)
			hb.pso.combobreaker = msl.pso.combobreaker
			hb.pso.element = ELEM_EARTH
			hb.angle = R_PointToAngle2(mo.x, mo.y, msl.x, msl.y)
			
			P_RemoveMobj(msl)
		end,
		
}

PSO.info.photonArts["resta"] = {
		-- PA is a technique
		technique = true,
		name = "Resta",
		desc = "Light Technique. Heals the caster and nearby allies depending on the caster's T-ATK stat.",
		cost = 80,
		potency = 40,
		actionspeed = 10,
		icon = "H_TRESTA",
		casttime = TICRATE*3/2,
		sound = sfx_heal,

		element = ELEM_LIGHT,
		spawndist = 0,
		speed = 30,
		
		object = MT_THOK,
		
		spawnfunc = function(msl)
			-- make sure razonde cannot have more than 1 instance per player
			local mo = msl.target
			local mop = mo.player
			
			if not mop return end
			
			local range = 768
			
			for i = 1, 16
				local ang = (360/16)*i*ANG1
				
				local s = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SUPERSPARK)
				s.color = SKINCOLOR_GREEN
				s.colorized = true
				
				P_InstaThrust(s, ang, FRACUNIT*32)
			end
			
			
			-- get the player's weapon
			local wep = PSO.lib.getPlayerActiveWeapon(mop)
			local tatk = mop.pso.tatk
			
			if PSO.info.weaponDamageType[wep.weapontype] == DMT_TECHNIQUE
				tatk = $ + wep.atk
			end
			
			local heal_amount = msl.pso.potency*tatk / 100
			heal_amount = $ + P_RandomRange(-heal_amount/10, heal_amount/10)	-- variance
			
			for p in players.iterate do
				
				if p and p.mo and p.mo.valid
				and R_PointToDist2(mo.x, mo.y, p.mo.x, p.mo.y) < range*mo.scale
					PSO.lib.healObject(p.mo, p.mo, heal_amount)
				end
			end
			
			P_RemoveMobj(msl)
		end,
}

PSO.info.photonArts["shifta"] = {
		-- PA is a technique
		technique = true,
		name = "Shifta",
		desc = "Fire Technique. Increase nearby allies' attack by "..PSO.info.SHIFTA.."% for 1:30",
		cost = 40,
		potency = 100+PSO.info.SHIFTA,
		icon = "H_TSHIFT",
		casttime = TICRATE,	-- how long it takes before the tech weapon swings to cast.
		
		sound = sfx_shifta,
		
		element = ELEM_FIRE,
		spawndist = 0,
		speed = 30,
		
		object = MT_THOK,
		
		spawnfunc = function(msl)
			-- make sure razonde cannot have more than 1 instance per player
			local mo = msl.target
			local mop = mo.player
			
			if not mop return end
			
			local range = 768
			
			for i = 1, 16
				local ang = (360/16)*i*ANG1
				
				local s = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SUPERSPARK)
				s.color = SKINCOLOR_RED
				s.colorized = true
				
				P_InstaThrust(s, ang, FRACUNIT*32)
			end
			
			
			-- gib shifta
			local duration = TICRATE*90
			if PSO.lib.hasSkill(mop, "fo_supportbitch")
				duration = $*2
			end
			
			for p in players.iterate do
				
				if p and p.mo and p.mo.valid
				and R_PointToDist2(mo.x, mo.y, p.mo.x, p.mo.y) < range*mo.scale
					p.pso.shifta = $ or 0
					p.pso.shifta = min(TICRATE*180, $ + duration)
				end
			end			
			
			P_RemoveMobj(msl)
		end,	
}

PSO.info.photonArts["deband"] = {
		-- PA is a technique
		technique = true,
		name = "Deband",
		desc = "Ice Technique. Increase nearby allies' defence by "..PSO.info.DEBAND.."% for 1:30",
		cost = 40,
		potency = 100+PSO.info.DEBAND,
		icon = "H_TDEBND",
		casttime = TICRATE,	-- how long it takes before the tech weapon swings to cast.
		
		sound = sfx_deband,
		
		element = ELEM_ICE,
		spawndist = 0,
		speed = 30,
		
		object = MT_THOK,
		
		spawnfunc = function(msl)
			-- make sure razonde cannot have more than 1 instance per player
			local mo = msl.target
			local mop = mo.player
			
			if not mop return end
			
			local range = 768
			
			for i = 1, 16
				local ang = (360/16)*i*ANG1
				
				local s = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SUPERSPARK)
				s.color = SKINCOLOR_BLUE
				s.colorized = true
				
				P_InstaThrust(s, ang, FRACUNIT*32)
			end
			
			
			-- gib deband
			local duration = TICRATE*90
			if PSO.lib.hasSkill(mop, "fo_supportbitch")
				duration = $*2
			end
			
			for p in players.iterate do
				
				if p and p.mo and p.mo.valid
				and R_PointToDist2(mo.x, mo.y, p.mo.x, p.mo.y) < range*mo.scale
					p.pso.deband = $ or 0
					p.pso.deband = min(TICRATE*180, $ + duration)
				end
			end
			
			P_RemoveMobj(msl)
		end,	
}

PSO.info.photonArts["rod_cast"] = {
		name = "Cast (Rod)",
		desc = "Cast a Technique",
		cost = 0,
		potency = 50,	-- +5 potency when casting with a rod
		speed = 20,		-- speed % multiplier.
						-- 100: full speed
						-- 0: stand still
		
		icon = "H_SWATK1",

		use = 	function(p, timer, hold, input, combo)
					local mo = p.mo
					local pos = p.pso.weaponpos[1]
					local tech = PSO.info.photonArts[p.pso.taction]
					local wep = PSO.lib.getPlayerActiveWeapon(p)
					
					if not tech
						return true
					end
					
					local object = tech.object
					local potency = tech.potency +PSO.info.photonArts[p.pso.action].potency	-- rod cast bonus
					local casttime = tech.casttime or TICRATE/2
					local swingtime = TICRATE*10
					
					if timer <= casttime
						
						local x = FixedMul(mo.scale, 8*cos(p.drawangle - ANG1*40))
						local y = FixedMul(mo.scale, 8*sin(p.drawangle - ANG1*40))
						local z = FixedDiv(mo.height*12/10, mo.scale)
						
						pos[2] = x
						pos[3] = y
						pos[4] = z
						pos[5] = ANG1*90
						pos[6] = ANG1*90
					end
					
					if timer == casttime
						p.pso.actiontimer = swingtime
					end
					
					p.pso.actionspeed = (tech.actionspeed or 30)
					
					-- swing the rod and cast the tech!
					if timer >= swingtime
						-- swing rod from top to bottom!
						local newtimer = timer-swingtime
						pos[7] = {
							use = true,
							o = {x = mo.x, y = mo.y, z = mo.z + mo.height/2},	-- the center of the slope is the center of the player.
							xy = p.drawangle + ANG1*90,	-- tilt sideways
							za = -ANG1*45,				-- tilt by 30 degrees
						}			

						
						local ang
						
						if newtimer < 3
							mo.rodangle = -ANG1*80
						elseif newtimer < TICRATE/2
							local nt = newtimer-3
							mo.rodangle = $ + max(0, ANG1*(20 - (nt-2)*4))
						end
						
						local ang = p.drawangle + mo.rodangle
						local x = 64 * cos(ang)
						local y = 64 * sin(ang)
						
						pos[2] = x
						pos[3] = y
						pos[4] = mo.height/2
						pos[5] = 0
						pos[6] = ang + ANG1*90
						
						-- spawn the tech effect
						if timer == swingtime + TICRATE/4
							
							local dist = tech.spawndist or 0
							local offx = dist*cos(p.drawangle)
							local offy = dist*sin(p.drawangle)
							local offz = FixedDiv(mo.height*(tech.spawnheight or 0)/100, mo.scale)
							
							local spd = tech.speed
							if spd == nil
								spd = 40	-- 0 spd is still valid
							end
							
							local msl = PSO.lib.spawnMissile(mo, object, p.drawangle, spd*FRACUNIT, potency, DMT_TECHNIQUE, offx, offy, 0, tech.reaction)
							if p.pso.lockon and p.pso.lockon.valid and tech.aim
								PSO.lib.aimMissile(msl, p.pso.lockon)
							end
							
							if tech.sound
								S_StartSound(mo, tech.sound)
							end
							
							if tech.element
								msl.pso.element = tech.element
							end
							
							local range = wep.range or 512
							msl.fuse = FixedDiv(range*FRACUNIT, FixedHypot(msl.momx, msl.momy))/FRACUNIT
							
							-- tech is a combo breaker is it's action 2 or 4
							-- yes it's magic numbers but also that's never changing so who cares.
							if p.pso.actionnum%2 == 0	-- if we ever add more for some reason lol
								msl.pso.combobreaker = true
							end
							
							if tech.spawntarget
							and p.pso.target
							and p.pso.target.valid	-- make sure the target is alive.
								P_MoveOrigin(msl, p.pso.target.x, p.pso.target.y, p.pso.target.z)
							end
							
							if tech.spawnfunc
								tech.spawnfunc(msl)
							end
							
						end
						
						if timer == swingtime + TICRATE/2
							return true
						end
					end
				end,
}

PSO.info.photonArts["wand_cast"] = {
		name = "Cast (Wand)",
		desc = "Cast a Technique",
		cost = 0,
		potency = 0,	-- +no extra potency when casting with a wand
		speed = 40,		-- speed % multiplier.
						-- 100: full speed
						-- 0: stand still
		
		icon = "H_SWATK1",

		use = 	function(p, timer, hold, input, combo)
					local mo = p.mo
					local pos = p.pso.weaponpos[HAND_RIGHT]
					local tech = PSO.info.photonArts[p.pso.taction]
					local wep = PSO.lib.getPlayerActiveWeapon(p)
					
					if not tech
						return true
					end
					
					local object = tech.object
					local potency = tech.potency +PSO.info.photonArts[p.pso.action].potency	--cast bonus
					local casttime = (tech.casttime or TICRATE/2) *6/10	-- 40% faster
					local swingtime = TICRATE*10
					
					if timer <= casttime
						
						local x = FixedMul(mo.scale, 8*cos(p.drawangle - ANG1*40))
						local y = FixedMul(mo.scale, 8*sin(p.drawangle - ANG1*40))
						local z = FixedDiv(mo.height*12/10, mo.scale)
						
						pos[2] = x
						pos[3] = y
						pos[4] = z
						pos[5] = ANG1*90
						pos[6] = ANG1*90
					end
					
					if timer == casttime
						p.pso.actiontimer = swingtime
					end
					
					p.pso.actionspeed = (tech.actionspeed or 30)
					
					-- swing the rod and cast the tech!
					if timer >= swingtime
						-- swing rod from top to bottom!
						local newtimer = timer-swingtime
						pos[7] = {
							use = true,
							o = {x = mo.x, y = mo.y, z = mo.z + mo.height/2},	-- the center of the slope is the center of the player.
							xy = p.drawangle + ANG1*90,	-- tilt sideways
							za = -ANG1*45,				-- tilt by 30 degrees
						}			

						
						local ang
						
						if newtimer < 3
							mo.rodangle = -ANG1*80
						elseif newtimer < TICRATE/2
							local nt = newtimer-3
							mo.rodangle = $ + max(0, ANG1*(20 - (nt-2)*4))
						end
						
						local ang = p.drawangle + mo.rodangle
						local x = 64 * cos(ang)
						local y = 64 * sin(ang)
						
						pos[2] = x
						pos[3] = y
						pos[4] = mo.height/2
						pos[5] = 0
						pos[6] = ang + ANG1*90
						
						-- spawn the tech effect
						if timer == swingtime + TICRATE/4
							
							local dist = tech.spawndist or 0
							local offx = dist*cos(p.drawangle)
							local offy = dist*sin(p.drawangle)
							local offz = FixedDiv(mo.height*(tech.spawnheight or 0)/100, mo.scale)
							
							local spd = tech.speed
							if spd == nil
								spd = 40	-- 0 spd is still valid
							end
							
							local msl = PSO.lib.spawnMissile(mo, object, p.drawangle, spd*FRACUNIT, potency, DMT_TECHNIQUE, offx, offy, 0, tech.reaction)
							if p.pso.lockon and p.pso.lockon.valid and tech.aim
								PSO.lib.aimMissile(msl, p.pso.lockon)
							end
							
							if tech.sound
								S_StartSound(mo, tech.sound)
							end
							
							if tech.element
								msl.pso.element = tech.element
							end
							
							local range = wep.range or 512
							msl.fuse = FixedDiv(range*FRACUNIT, FixedHypot(msl.momx, msl.momy))/FRACUNIT
							
							-- tech is a combo breaker is it's action 2 or 4
							-- yes it's magic numbers but also that's never changing so who cares.
							if p.pso.actionnum%2 == 0	-- if we ever add more for some reason lol
								msl.pso.combobreaker = true
							end
							
							if tech.spawntarget
							and p.pso.target
							and p.pso.target.valid	-- make sure the target is alive.
								P_MoveOrigin(msl, p.pso.target.x, p.pso.target.y, p.pso.target.z)
							end
							
							if tech.spawnfunc
								tech.spawnfunc(msl)
							end
							
						end
						
						if timer == swingtime + TICRATE/2
							return true
						end
					end
				end,
}