-- Technique object definitions
-- must be defined before photon arts

-- FOIE
freeslot("MT_FOIE", "S_FOIE", "S_FOIEDEATH")

mobjinfo[MT_FOIE] = {
	doomednum = -1,
	spawnhealth = 1000,
	spawnstate = S_FOIE,
	deathstate = S_FOIEDEATH,
	xdeathstate = S_FOIEDEATH,
	radius = 24<<FRACBITS,
	height = 48<<FRACBITS,	
	flags = MF_NOGRAVITY|MF_MISSILE,
}

function A_FoieVisual(mo)
	local e = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SONIC3KBOSSEXPLODE)
	e.scale = mo.scale*2
	e.destscale = 1
	e.scalespeed = mo.scale/24
	e.momx = P_RandomRange(-8, 8)*mo.scale
	e.momy = P_RandomRange(-8, 8)*mo.scale
	e.momz = P_RandomRange(-8, 8)*mo.scale
end

function A_FoieDeath(mo)
	for i = 1, 16
		local e = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SONIC3KBOSSEXPLODE)
		e.scale = mo.scale*3/2
		e.destscale = 1
		e.scalespeed = mo.scale/12
		e.momx = P_RandomRange(-32, 32)*mo.scale
		e.momy = P_RandomRange(-32, 32)*mo.scale
		e.momz = P_RandomRange(-32, 32)*mo.scale
	end

	-- spawn an explosion hitbox:
	local h = PSO.lib.spawnHurtBox(mo.x, mo.y, mo.z, 200<<FRACBITS, 64<<FRACBITS, mo.target, mo.pso.potency, DMT_TECHNIQUE, 4, RT_KNOCKBACK)
	h.pso.whitelist = $ or {}
	h.pso.combobreaker = mo.pso.combobreaker
		
	-- search for the enemy we hit:
	local enemies = {}
	searchBlockmap("objects", 
		function(ref, found)
				
			if found and found.valid and found.flags & MF_SHOOTABLE
				enemies[#enemies+1] = found
					
			end
		end,
	mo)
		
	table.sort(enemies, 
		function(a, b)
			return R_PointToDist2(mo.x, mo.y, a.x, a.y) < R_PointToDist2(mo.x, mo.y, b.x, b.y)
		end
	)
		
	if enemies[1] and enemies[1].valid
		h.pso.whitelist[#h.pso.whitelist+1] = enemies[1]
	end
	
end

states[S_FOIE] = {SPR_NULL, A, 1, A_FoieVisual, 0, 0, S_FOIE}
states[S_FOIEDEATH] = {SPR_NULL, A, 1, A_FoieDeath, 0, 0, S_NULL}


-- BARTA
freeslot("MT_BARTA", "S_BARTA")
mobjinfo[MT_BARTA] = {
	doomednum = -1,
	spawnstate = S_BARTA,
	deathstate = S_NULL,
	xdeathstate = S_NULL,
	radius = 48<<FRACBITS,
	height = 48<<FRACBITS,
	flags = MF_MISSILE,
}

-- disable collision for barta
addHook("MobjCollide", function(mo1, mo2) return false end, MT_BARTA)
addHook("MobjMoveCollide", function(mo1, mo2) return false end, MT_BARTA)

function A_Barta(mo)
	
	mo.friction = FRACUNIT	-- no slowdown
	if mo.target and mo.target.valid 
		if not mo.hurtbox
			mo.hurtbox = PSO.lib.spawnHurtBox(mo.x, mo.y, mo.z, 96<<FRACBITS, 64<<FRACBITS, mo.target, mo.pso.potency, DMT_TECHNIQUE, TICRATE*999)
			mo.hurtbox.pso.combobreaker = mo.pso.combobreaker
		else
			mo.hurtbox.fuse = 3
			P_MoveOrigin(mo.hurtbox, mo.x, mo.y, mo.z)
		end
	end
end

states[S_BARTA] = {SPR_FHZI, A|TR_TRANS10, 1, A_Barta, 0, 0, S_BARTA}

-- ZONDE
freeslot("SPR_LZI1", "S_ZONDE")
states[S_ZONDE] = {SPR_LZI1, FF_ANIMATE, (E+1)*3, nil, E, 3}

freeslot("MT_RAZONDE")
mobjinfo[MT_RAZONDE] = {
	doomednum = -1,
	spawnstate = S_INVISIBLE,
	deathstate = S_NULL,
	xdeathstate = S_NULL,
	radius = 1, 
	height = 1,
	flags = MF_NOGRAVITY|MF_NOCLIP|MF_NOCLIPTHING,
}

local razonde_dist = 512
addHook("MobjThinker", function(mo)
	
	if not mo.extravalue1
		mo.tics = TICRATE*10 + TICRATE/2
		mo.fuse = mo.tics
		mo.extravalue1 = 1
	end
	
	if mo.tics % (TICRATE*2) == 0
		
		for i = 1, 16
			
			local ang = (360/16)*ANG1*i
			local s = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SUPERSPARK)
			s.color = SKINCOLOR_GOLD
			s.colorized = true
			
			P_InstaThrust(s, ang, FRACUNIT*48)
		end
		
		local x1 = mo.x - razonde_dist*FRACUNIT
		local x2 = mo.x + razonde_dist*FRACUNIT
		local y1 = mo.y - razonde_dist*FRACUNIT
		local y2 = mo.y + razonde_dist*FRACUNIT
		
		local search = function(mo, found)
			if found and found.flags & MF_SHOOTABLE
			and R_PointToDist2(mo.x, mo.y, found.x, found.y) < razonde_dist*FRACUNIT
				
				local hb = PSO.lib.spawnHurtBox(found.x, found.y, found.z, 96<<FRACBITS, 64<<FRACBITS, mo.target, mo.pso.potency/5, DMT_TECHNIQUE, 2)
				hb.pso.combobreaker = mo.pso.combobreaker
				hb.pso.locks = 1
			end
		end
		
		S_StartSound(mo, sfx_zonde)
		searchBlockmap("objects", search, mo, x1, x2, y1, y2)
	end
	
	for i = 1, 2
		local ang = ANG1*mo.tics*7 + (ANG1*180*i)
		local x = mo.x + razonde_dist*cos(ang)
		local y = mo.y + razonde_dist*sin(ang)
		
		local s = P_SpawnMobj(x + P_RandomRange(-32, 32)*mo.scale, y + P_RandomRange(-32, 32)*mo.scale, mo.z + P_RandomRange(-32, 32)*mo.scale, MT_SUPERSPARK)
		s.color = SKINCOLOR_GOLD
		s.colorized = true
	end
	
end, MT_RAZONDE)


freeslot("MT_DIGA")
mobjinfo[MT_DIGA] = {
	doomednum = -1,
	spawnstate = S_ROCKCRUMBLEA,
	spawnhealth = 1000,
	radius = 32<<FRACBITS,
	height = 128<<FRACBITS,
	flags = MF_MISSILE,
}

addHook("MobjThinker", function(mo)
	if leveltime & 1
		P_SpawnGhostMobj(mo)
	end
end, MT_DIGA)

