-- Generates the shop for the player.
-- Separated from other files as the shop item list is pretty big.

-- shop resets every PSO.info.SHOPRESET minutes spent outside.

-- {itname, chance%, level_min, level_max}
-- itname appears chance% of the time if the player level is between level_min and level_max.
-- level_min/max can be left nil to always allow it to appear

local shoplist = {
	{"monomate", 100},
	{"dimate", 100},
	{"trimate", 100},
	{"monogrinder", 100, 10},
	{"digrinder", 33, 10},
	{"trigrinder", 33, 30},
	
	-- weapons
	--1
	{"sword", 100, 1, 15},
	{"doublesaber", 100, 15},
	{"rifle", 100, 1, 15},
	{"shotgun", 100, 15},
	{"rod", 100, 1, 15},
	{"saber", 100, 1, 15},
	{"handgun", 100, 1, 15},
	{"wand", 100, 1, 15},
	
	--2
	{"gigush", 100, 7, 19},
	{"doublerapier", 100, 7, 19},
	{"beam", 100, 7, 19},
	{"baton", 100, 7, 19},
	{"brand", 100, 7, 19},
	{"lockgun", 100, 7, 19},
	{"staff", 100, 7, 19},

	--3
	{"breaker", 100, 14},
	{"doublegladius", 100, 14},
	{"laser", 100, 14},
	{"ulrod", 100, 14},
	{"slash", 100, 14},
	{"autogun", 100, 14},
	{"batnara", 100, 14},

	--4
	{"pine", 100, 21, 35},
	{"doubliss", 100, 21, 35},
	{"burster", 100, 21, 35},
	{"crescent", 100, 21, 35},
	{"claw", 100, 21, 35},
	{"peacekeeper", 100, 21, 35},
	{"hammer", 100, 21, 35},

	{"alva pine", 100, 30},
	{"alva doubliss", 100, 30},
	{"alva burster", 100, 30},
	{"alva crescent", 100, 30},
	{"alva claw", 100, 30},
	{"alva peacekeeper", 100, 30},
	{"alva hammer", 100, 30},

	{"vita pine", 100, 35},
	{"vita doubliss", 100, 35},
	{"vita burster", 100, 35},
	{"vita crescent", 100, 35},
	{"vita claw", 100, 35},
	{"vita peacekeeper", 100, 35},
	{"vita hammer", 100, 35},
	
	-- units

	{"frame", 100, 1, 15},
	{"scrap frame", 100, 7, 19},
	{"stone frame", 100, 14, 29},
	{"icicle frame", 100, 20},
	{"rostig frame", 100, 30},
	{"aileron frame", 100, 45},
	
	{"unit_hp_1", 100, 5},
	{"unit_pp_1", 100, 5},
	{"unit_satk_1", 100, 5},
	{"unit_ratk_1", 100, 5},
	{"unit_tatk_1", 100, 5},

	{"unit_hp_2", 30, 20},
	{"unit_pp_2", 30, 20},
	{"unit_satk_2", 30, 20},
	{"unit_ratk_2", 30, 20},
	{"unit_tatk_2", 30, 20},
	
	-- PAs / Techs
	{"sword_pa_extendcutter", 100, 15},
	{"sword_pa_spiralbreak", 100, 25},
	{"dsaber_pa_spinadvance", 100, 15},
	{"dsaber_pa_tornadodance", 100, 25},
	{"rifle_pa_piercinground", 100, 15},
	{"rifle_pa_slideshot", 100, 25},
	{"shotgun_pa_gloriousrain", 100, 15},
	{"shotgun_pa_impactrecoil", 100, 25},
	
	
	{"gifoie", 100, 15},
	{"rafoie", 100, 25},
	{"shifta", 100, 20},
	{"gibarta", 100, 15},
	{"rabarta", 100, 25},
	{"deband", 100, 20},
	{"gizonde", 100, 15},
	{"razonde", 100, 25},
	{"gidiga", 100, 15},
	{"radiga", 100, 25},
	{"resta", 100, 20},
}

PSO.lib.rollPlayerShop = function(p)
	
	local tabl = {}
	local clevel = p.pso.level
	p.pso.shop = {}	-- reset
	
	for i = 1, #shoplist
		local l = shoplist[i]
		
		if l[3] and clevel < l[3]
		or l[4] and clevel > l[4]
		or l[2] and P_RandomRange(1, 100) > l[2] 
			continue
		end
		
		-- if we've made it here, generate the item
		local it = PSO.lib.makeItem(l[1])
		local info = PSO.lib.itemInfo(it)
		
		local qty = 1
		if type(it) == "string"	-- stackable
			local limit = info.carrylimit or 1
			qty = P_RandomRange(limit/2 or 1, limit)
		end
		
		
		table.insert(p.pso.shop, {it, qty})
	end
	
	PSO.lib.playerSortShop(p)
end

COM_AddCommand("psod_resetshop", function(p)
	PSO.lib.rollPlayerShop(p)
end)