-- Type Skill definitions

-- Worth noting that by the nature of type Skills their functionalities aren't implemented here but where they are relevant.
-- (eg: dodge roll extension goes in the dodge roll code where it checks if the player has the skill)

PSO.info.typeSkills = {
	
	["all_automate"] = {
		name = "Auto-Mate Half",
		desc = "Automatically uses Mate items when your HP drops to 50% or below.",
		cost = 1,
		level = 1,
	},
	
	["all_hp1"] = {
		name = "HP Up",
		desc = "Increases Max HP by 50. Stacks with other Skills.",
		cost = 1,
		level = 10,
		equip = function(p)
					p.pso.maxhp = $+50
				end,
		unequip = function(p)
					p.pso.maxhp = $-50
					p.pso.hp = min($, p.pso.maxhp)
				end,
	},
	
	["all_pp1"] = {
		name = "PP Up",
		desc = "Increases Max PP by 10. Stacks with other Skills.",
		cost = 1,
		level = 10,
		equip = function(p)
					p.pso.maxpp = $+10
				end,
		unequip = function(p)
					p.pso.maxpp = $-10
					p.pso.pp = min($, p.pso.maxpp)
				end,		
	},
	
	["all_dodgeextension"] = {
		name = "Dodge Master",
		desc = "Doubles the Dodge Roll's invulnerability period.",
		cost = 2,
		level = 1,
	},
	
	-- Class specific Skills
	-- HUNTER
	["hu_ironwill"] = {
		name = "Iron Will",
		desc = "Survive a fatal attack with 1 HP once per quest.",
		cost = 2,
		level = 5,
	},

	["hu_hunterphysique"] = {
		name = "Hunter Physique",
		desc = "Lowers the strength tier of knockback received.",
		cost = 1,
		level = 10,
	},

	["hu_hatefulaggression"] = {
		name = "Hateful Aggression",
		desc = "Increases damage dealt by 5% when attacking an enemy targetting you.",
		cost = 1,
		level = 20,
	},
	
	-- RANGER
	["ra_chaintrigger"] = {
		name = "Chain Trigger",
		desc = "Detonating a chain restores PP based on the chain count and number of players.",
		cost = 2,
		level = 5,
	},

	["ra_quickload"] = {
		name = "Quick Load",
		desc = "Reduces ranged weapon Photon Art charge time for those which have one.",
		cost = 1,
		level = 10,
	},	

	["ra_tacticalroll"] = {
		name = "Tactical Roll",
		desc = "Recover 35PP when succesfully evading an attack with a dodge roll.",
		cost = 1,
		level = 20,
	},
	
	-- FORCE
	["fo_ppwellup"] = {
		name = "Cast PP Well Up",
		desc = "Allows PP restoration to occur at half the regular rate while using Techniques.",
		cost = 2,
		level = 5,
	},

	["fo_ppabsorb"] = {
		name = "Photon Absorb",
		desc = "When taking damage, recover PP proportional to damage taken.",
		cost = 1,
		level = 10,
	},	
	
	["fo_supportbitch"] = {
		name = "Support Amplifier",
		desc = "Doubles the duration of Shifta & Deband.",
		cost = 1,
		level = 20,
	},
}

PSO.info.typeSkillsList = {
	[TYPE_HUNTER] = {
		"all_automate",
		"all_hp1",
		"all_pp1",
		"all_dodgeextension",
		"hu_ironwill",
		"hu_hunterphysique",
		"hu_hatefulaggression",
	},

	[TYPE_RANGER] = {
		"all_automate",
		"all_hp1",
		"all_pp1",
		"all_dodgeextension",	
		"ra_chaintrigger",
		"ra_quickload",		
		"ra_tacticalroll"
	},

	[TYPE_FORCE] = {
		"all_automate",
		"all_hp1",
		"all_pp1",
		"all_dodgeextension",	
		"fo_ppwellup",
		"fo_ppabsorb",		
		"fo_supportbitch"
	},
}

PSO.lib.addSkill = function(p, sk)
	
	local dta = PSO.info.typeSkills[sk]
	local points = PSO.lib.skillCosts(p)
	
	assert(dta, "No valid data for skill "..tostring(sk))
	
	local cost = dta.cost or 0
	
	if points + cost > p.pso.maxskillpoints
		return false
	end
	
	p.pso.typeskills[p.pso.type][#p.pso.typeskills[p.pso.type]+1] = sk
	
	if PSO.info.typeSkills[sk].equip
		PSO.info.typeSkills[sk].equip(p)
	end
	
	return true
end

PSO.lib.hasSkill = function(p, sk)
	for i = 1, #p.pso.typeskills[p.pso.type]
		if p.pso.typeskills[p.pso.type][i] == sk
			return i
		end
	end
	return false
end

PSO.lib.removeSkill = function(p, sk)
	local index = PSO.lib.hasSkill(p, sk)
	if index
		table.remove(p.pso.typeskills[p.pso.type], index)
		
		if PSO.info.typeSkills[sk].unequip
			PSO.info.typeSkills[sk].unequip(p)
		end
		
	end
end

PSO.lib.skillCosts = function(p)
	local totalcost = 0
	for i = 1, #p.pso.typeskills[p.pso.type]
		local sk = PSO.info.typeSkills[p.pso.typeskills[p.pso.type][i]]
		if sk and sk.cost
			totalcost = $+sk.cost
		end
	end
	
	return totalcost
end