-- EMERGENCY! CODE: SPAGHETTI
-- code that handles the display of las vegas emergency codes for hud

-- sounds
freeslot("sfx_emrgy", "sfx_emrgc", "sfx_emrgf", "sfx_cquick", "sfx_cduel", "sfx_cavd")

-- quick font rendering code taken from SRB2P
rawset(_G, "ascii_03d", {})
for i = 0, 255	-- generate ascii %03d table so we don't have to do string.format everytime, this function is very slow:
	ascii_03d[i] = string.format("%03d", tostring(i))
end
local prefix = "EMRF"

rawset(_G, "V_drawEmergencyLetter", function(v, x, y, str, flags, scale, colormap)
	if not (v and tonumber(x)~=nil and tonumber(y)~=nil and str and prefix) return end
	str = tostring(str)
	scale = $ or FRACUNIT
	if str:len() > 1
		return
	end

	if not v.patchExists(prefix..ascii_03d[str:byte()]) return end	-- patch doesn't exist, don't draw
	v.drawScaled(x, y, scale, v.cachePatch(prefix..ascii_03d[str:byte()]), flags, colormap)
end)

rawset(_G, "V_emergencyStringWidth", function(v, str, flags, scale)
	str = tostring(str)
	if not (v and str and prefix) return 0 end
	scale = $ or FRACUNIT
	local xoffs = 0

	for i = 1, str:len()
		local s = str:sub(i, i)

		if not v.patchExists(prefix..ascii_03d[s:byte()]) -- patch doesn't exist, assume it's a space:
			xoffs = $+10*scale
			continue
		end
		local pp = v.cachePatch(prefix..ascii_03d[s:byte()])
		xoffs = $+ pp.width*scale + FRACUNIT
	end
	return xoffs
end)


rawset(_G, "V_drawEmergencyString", function(v, x, y, str, flags, alignment, scale, colormap)
	if not (v and tonumber(x)~=nil and tonumber(y)~=nil and str and prefix) return end
	scale = $ or FRACUNIT

	x = $<<FRACBITS
	y = $<<FRACBITS	-- don't break compability...

	local xoffs = 0
	local yoffs = 0

	-- alignment coordinate adjustment:
	if alignment == "right"
		x = $ - V_emergencyStringWidth(v, str, flags, scale)
	elseif alignment == "center"
		x = $ - V_emergencyStringWidth(v, str, flags, scale)/2
	end

	for i = 1, str:len()
		local s = str:sub(i, i)
		if s == "\n"	-- line break
			yoffs = $+16*scale
			xoffs = 0
			continue
		end

		if not v.patchExists(prefix..ascii_03d[s:byte()]) -- patch doesn't exist, assume it's a space:
			xoffs = $+10*scale
			continue
		end

		V_drawEmergencyLetter(v, (x+xoffs), (y+yoffs), s, flags, scale, colormap)	-- letter
		local pp = v.cachePatch(prefix..ascii_03d[s:byte()])
		xoffs = $+ pp.width*scale + FRACUNIT

	end
end)

freeslot("SPR_ECRC")	-- hack...
local patchcache = {}

local EMERGENCY_TIME = TICRATE*3

PSO.lib.startEmergency = function(p, top, bottom, sound1, sound2, delay, etype)
	
	etype = $ or EMER_NORMAL
	p.pso2_emergency = {
		text = {top, bottom, objective},
		sfx = {sound1, sound2},
		delay = delay or 0,
		time = EMERGENCY_TIME,
	}
end

PSO.lib.handleEmergency = function(p)

	-- bit of a hack...
	local mo = p.mo

	if not p.pso2_emergency return end

	local e = p.pso2_emergency
	
	if e.delay
		e.delay = $-1
		return
	end
	
	e.time = $-1

	if e.time == EMERGENCY_TIME - TICRATE/2
	and e.sfx[1]
		S_StartSound(nil, e.sfx[1], p)

	elseif e.time == EMERGENCY_TIME - (TICRATE*3 - 8)
	and e.sfx[2]
		S_StartSound(nil, e.sfx[2], p)
	end

	if not e.time
		p.pso2_emergency = nil
		return
	end

end

local function drawEmergency(v, p)

	-- cache the circle rotation to avoid HEAVY lag in game
	if not #patchcache or not patchcache[i].valid	-- wtf.
		-- time to lag your game fucker
		for i = 1, 120
			patchcache[i] = v.getSpritePatch(SPR_ECRC, A, 0, (360/120 *i)*ANG1)
			--print("Cached #"..i)
		end
	end

	if not p.pso2_emergency return end
	local e = p.pso2_emergency
	
	if e.delay return end	-- not yet

	local t = e.time
	local rt = EMERGENCY_TIME - t	-- reverse timer, useful for some effects.

	local basescale = min(FRACUNIT, rt* (FRACUNIT/(TICRATE/2)))	-- grow to normal scale in 17 tics

	local frame = (leveltime%120) +1	-- circle frame

	-- Draw the circles

	-- this one circle here only appears at the start and scales up way too fast:
	if rt < TICRATE
		v.drawScaled(160<<FRACBITS, 100<<FRACBITS, rt *FRACUNIT/5, patchcache[frame], V_60TRANS)
	end

	if t < TICRATE/2
		local newtimer = TICRATE/2 -t
		basescale = FRACUNIT + newtimer*FRACUNIT/4
	end

	-- Start from the inner
	v.drawScaled(160<<FRACBITS, 100<<FRACBITS, basescale*30/100, patchcache[frame], V_60TRANS|V_ADD)	-- Back
	v.drawScaled(160<<FRACBITS, 100<<FRACBITS, basescale*40/100, patchcache[121 - frame], V_20TRANS|V_ADD)	-- Front

	-- Middle
	v.drawScaled(160<<FRACBITS, 100<<FRACBITS, basescale*60/100, patchcache[121 - frame], V_60TRANS|V_ADD)	-- Back
	v.drawScaled(160<<FRACBITS, 100<<FRACBITS, basescale*70/100, patchcache[frame], V_20TRANS|V_ADD)	-- Front

	-- Front
	v.drawScaled(160<<FRACBITS, 100<<FRACBITS, basescale*90/100, patchcache[frame], V_60TRANS|V_ADD)	-- Back
	v.drawScaled(160<<FRACBITS, 100<<FRACBITS, basescale, patchcache[121 - frame], V_20TRANS|V_ADD)	-- Front


	-- text
	if rt > TICRATE/2
	and t > TICRATE/2 -10	-- for fade out
		-- top text:

		local newtimer = rt - TICRATE/2

		local bg_basetrans = 4

		local flag = 0
		-- fadein (hacky there's probably a better way to do this i cba)
		if rt < TICRATE/2 +10
			local transtimer = 10- (newtimer)
			if transtimer and transtimer > 0
				flag = transtimer << V_ALPHASHIFT
				bg_basetrans = max(4, transtimer)
			end
		end

		-- fadeout (ditto)
		if t < TICRATE/2
			local transtimer = TICRATE/2 - t
			if transtimer and transtimer < 10
				flag = transtimer << V_ALPHASHIFT
				bg_basetrans = 4+ transtimer
			end
		end

		-- background
		local x = 340 - newtimer/5
		local pp = v.cachePatch("EMERBG")
		for i = 1, 5 do
			if not bg_basetrans or bg_basetrans >= 10 break end	-- yikes, get out of here
			v.draw(x, 75, pp, bg_basetrans << V_ALPHASHIFT)

			bg_basetrans = $+1
			x = $ - pp.width
		end

		if e.text[1]
			V_drawEmergencyString(v, 80 + newtimer/6, 80, e.text[1], flag)
		end
		if e.text[2]
			V_drawEmergencyString(v, 270 - newtimer/6, 100, e.text[2], flag, "right")
		end

		-- emergency star
		v.draw(40 - newtimer/10, 100, v.cachePatch("EMERST"), flag)
	end
end

PSO.lib.registerHudFunc(drawEmergency)

/*COM_AddCommand("funny", function(p, arg, arg2)
	K_startEmergency(p, arg, arg2)
end)*/