-- Rare Drip KOI KOI!

for i = 0, 3
	freeslot("sfx_rdrp"..i)	-- rare drip
end

freeslot("SPR_RDRS")	-- RDR shine. Needs to be cached as a sprite for rotations.

PSO.lib.startRareDropAnim = function(p, rarity)
	if not p.mo return end

	-- play sound
	S_StartSound(nil, sfx_rdrp0, p)

	if rarity >= 7
		S_StartSound(nil, sfx_rdrp1, p)

		if rarity >= 13 and rarity <= 14
			S_StartSound(nil, sfx_rdrp2, p)
		elseif rarity >= 15
			S_StartSound(nil, sfx_rdrp3, p)
		end

		if not p.mo.pso2_raredroptimer or p.mo.pso2_rarity and rarity >= p.mo.pso2_rarity
			-- Start the animation only if it either isn't playing or if you just got a higher or same rarity item
			p.mo.pso2_raredroptimer = TICRATE*3
			p.mo.pso2_rarity = rarity
		end
	end
end

PSO.lib.playerHandleRareDropAnim = function(p)
	local mo = p.mo
	if p.mo.pso2_raredroptimer
		p.mo.pso2_raredroptimer = $-1
	end
end

local rainbow = {
	SKINCOLOR_PINK,
	SKINCOLOR_YOGURT,
	SKINCOLOR_TAN,
	SKINCOLOR_MOSS,
	SKINCOLOR_AQUA,
	SKINCOLOR_LILAC,
}

local lastindex
local prefix = "EMRF"
rawset(_G, "V_drawRDString", function(v, x, y, str, flags, alignment, scale, colormap, special, timer)
	timer = $ or 0
	str = tostring(str)
	if not (v and tonumber(x)~=nil and tonumber(y)~=nil and str and prefix) return end
	scale = $ or FRACUNIT

	x = $<<FRACBITS
	y = $<<FRACBITS	-- don't break compability...

	local xoffs = 0
	local yoffs = 0

	-- alignment coordinate adjustment:
	if alignment == "right"
		x = $ - V_emergencyStringWidth(v, str, flags, scale)
	elseif alignment == "center"
		x = $ - V_emergencyStringWidth(v, str, flags, scale)/2
	end

	if special	-- rainbow text...
		lastindex = leveltime % ((#rainbow)*2) /2 +1
	end

	if timer
		str = str:sub(1, timer/2)
	end

	for i = 1, str:len()
		local s = str:sub(i, i)
		local let_time = timer - 2*(i-1)

		if s == "\n"	-- line break
			yoffs = $+16*scale
			xoffs = 0
			continue
		end

		if not v.patchExists(prefix..ascii_03d[s:byte()]) -- patch doesn't exist, assume it's a space:
			xoffs = $+10*scale
			continue
		end

		if special
			colormap = v.getColormap(TC_RAINBOW, rainbow[lastindex])
			lastindex = $-1
			if lastindex < 1
				lastindex = #rainbow
			end
		end

		local specialoffset = 0
		if let_time >= 0 and let_time <= 6
		and timer
			local fakeang = let_time * 55 * ANG1 -- = 55
			specialoffset = $ + FixedMul(scale, sin(fakeang))*8
		end

		V_drawEmergencyLetter(v, (x+xoffs), (y+yoffs+specialoffset), s, flags, scale, colormap)	-- letter
		local pp = v.cachePatch(prefix..ascii_03d[s:byte()])
		xoffs = $+ pp.width*scale + FRACUNIT

	end
end)

-- anim test...

local patchcache = {}

local function handleRareDropAnimation(v, x, y, timer, rarity)
	-- cache the rotating bits
	if not #patchcache or not patchcache[1].valid	-- wtf
		for i = 1, 32
			patchcache[i] = v.getSpritePatch(SPR_RDRS, C, 0, (360/32 *i)*ANG1)
		end
	end

	if rarity < 7 return end

	local t = TICRATE*3 - timer
	local str = "RARE DROP"
	local strw = (V_emergencyStringWidth(v, str)/2)/FRACUNIT
	local patchscale = FRACUNIT
	local fade = 0

	if t < 5
		patchscale = FRACUNIT/4 *t
	end
	if timer < 10
		fade = (10 - timer) << V_ALPHASHIFT
	end


	local frame = (leveltime % 18) / 3 +1
	local patch = v.cachePatch("RDRA"..frame)
	local patchr = v.cachePatch("RDRRA"..frame)

	if rarity <= 12
		v.drawScaled(x<<FRACBITS, (y + (8*sin(leveltime*ANG1*16)/FRACUNIT))<<FRACBITS, patchscale, patch, fade)
	else

		-- if rarity >= 13, rainbow drop

		-- attempt at rainbow drop
		local colour = rainbow[leveltime % (10*(#rainbow)) / 10 +1]
		local tcolour = rainbow[(leveltime+10) % (10*(#rainbow)) / 10 +1]
		local trans = (9 - (leveltime % 10)) << V_ALPHASHIFT

		-- 15* aura
		if rarity >= 15
			for i = 1, 2

				local timer = leveltime + (i-1)*3

				local shinecolour = rainbow[timer % (2*(#rainbow)) / 2 +1]
				local shinescale = FRACUNIT/2 + (timer%6) *FRACUNIT/6
				local shinetrans = (2 + (timer%8) -1)
				if shinetrans and (shinetrans) < 10
					local pname = "RAURA3"	--..max(1, shinetrans)	-- use gradually darker patches for additive translucency
					v.drawScaled(x<<FRACBITS, (y)<<FRACBITS, shinescale, v.cachePatch(pname), V_ADD|trans, v.getColormap(TC_RAINBOW, shinecolour))
				end
			end
		end

		-- the shine...
		local shinecolour = rainbow[leveltime % (4*(#rainbow)) / 4 +1]
		local shinescale = FRACUNIT/2 + (leveltime%10) *FRACUNIT/6
		local shineframe = leveltime% (32*4) /4 +1
		local shinetrans = (1 + (leveltime%8)) << V_ALPHASHIFT
		if shinetrans and (shinetrans) < (10 << V_ALPHASHIFT)
			--print(shinetrans.."|"..shineframe)
			local patch = patchcache[shineframe]
			v.drawScaled(x<<FRACBITS, (y-10)<<FRACBITS, shinescale, patch, V_ADD|shinetrans, v.getColormap(TC_RAINBOW, shinecolour))
		end

		v.drawScaled(x<<FRACBITS, (y + (8*sin(leveltime*ANG1*16)/FRACUNIT))<<FRACBITS, patchscale, patchr, fade, v.getColormap(TC_RAINBOW, colour))
		-- translucent layer
		if leveltime%10
			v.drawScaled(x<<FRACBITS, (y + (8*sin(leveltime*ANG1*16)/FRACUNIT))<<FRACBITS, patchscale, patchr, (fade or trans), v.getColormap(TC_RAINBOW, tcolour))
		end
	end
	-- the text...
	-- defaults to tc_rainbow black, special makes it rainbow if rarity > 9
	V_drawRDString(v, x-strw, y+10, str:sub(1, t/2), fade, nil, nil, v.getColormap(TC_RAINBOW, SKINCOLOR_GREY), rarity > 9, t)

end

local function HU_rareDropAnim(v, p, c)
	if p and p.mo and p.mo.pso2_raredroptimer
		handleRareDropAnimation(v, 160, 40, p.mo.pso2_raredroptimer, p.mo.pso2_rarity)
	end
end

PSO.lib.registerHudFunc(HU_rareDropAnim)

COM_AddCommand("funny2", function(p, arg)
	PSO.lib.startRareDropAnim(p, tonumber(arg))
end)
