local BOXSEG = 16
local BOXSCALE = FRACUNIT/2

-- debug, if false, the source isn't checked to see if we caan just use an existing box.
local BOX_REPLACE = true

local MAXBOXW = 12
local MAXLINES = 7

-- if the source is farther than this distance, don't draw the box at all
local BOX_MAXDIST = 3184*FRACUNIT

-- for "physics"
local BOX_PUSHSPEED = 12*FRACUNIT

local BOX_MINX = 0
local BOX_MAXX = 320*FRACUNIT

local BOX_MINY = 0
local BOX_MAXY = 140*FRACUNIT

local pso_chatboxes = {}

-- prepares a string for the chatbox
-- returns the string, box width and box height necessary to hold it
local function chat_prepString(v, s)
	local w = min(MAXBOXW, s:len()/2 ) +1
	s = PSO.lib.wordWrap(v, s, (MAXBOXW*BOXSEG*BOXSCALE) /FRACUNIT, V_ALLOWLOWERCASE, "small")
	local h = select(2, s:gsub("\n", ""))
	-- with the way the boxes are shaped, we need another thing here, unclean but it works
	if h == 2
		h = $+1
	end
	return s, max(3, w), max(3, h+1)
end

local function draw_chatbox(v, dta)	--x, y, w, h, name, text, col, flags)
	
	local x = dta.x
	local y = dta.y
	local w = dta.w
	local h = dta.h
	local name = dta.name
	local text = dta.content
	local col = dta.color

	local flags = 0
	if dta.timer and dta.timer < 10
		flags = (10 - dta.timer)<<V_ALPHASHIFT
	end
	
	-- /toge: Draw the emphasis!
	if dta.type == 1
	and dta.timer and dta.timer > 10
		if dta.existfor == 1
			
			local cx = x + dta.realw/2
			local cy = y + dta.realh/2
			
			dta.togetri = {}
			for i = 1, 6
				
				local a = v.RandomRange(0, 359)*ANG1
				local nx = cx + FixedMul(dta.realw/4, cos(a))
				local ny = cy + FixedMul(dta.realh/4, sin(a))
				
				local points = {}
				for j = 1, 2
					local ang = a + ANG1*90 + (j-1)*180*ANG1
					points[j] = {
						nx + 4*cos(ang) - dta.x,
						ny + 4*sin(ang) - dta.y
					}
				end
				
				points[3] = {
					nx + FixedMul(dta.realw/8, cos(a)) - dta.x,
					ny + FixedMul(dta.realh/8, sin(a)) - dta.y,
				}
				
				dta.togetri[i] = {points, {nx-dta.x, ny-dta.y}, a, v.RandomRange(8, 16)}
			end
			
		elseif dta.existfor > 1
			
			for i = 1, 6
				local toge = dta.togetri[i]
				local points = toge[1]
				local origin = toge[2]
				local ang = toge[3]
				local amp = max(0, toge[4] - dta.existfor/2)
				
				local pt = points[3]
				
				local bdist = toge[4]*2
				local dist = -amp*sin(leveltime*ANG1*32)
				
				pt[1] = origin[1] + (bdist* cos(ang)) + FixedMul(dist, cos(ang))
				pt[2] = origin[2] + (bdist* sin(ang)) + FixedMul(dist, sin(ang))
				
				-- i want die
				PSO.lib.HUD_drawTriangle(v, dta.x+points[1][1], dta.y+points[1][2], dta.x+points[2][1], dta.y+points[2][2], dta.x+points[3][1], dta.y+points[3][2], 0)
			end
		end
	
	elseif dta.type == 2	-- moya
		
		if dta.existfor == 1
			-- prepare a bunch of "bubbles"
			
			local cx = x + dta.realw/2
			local cy = y + dta.realh/2
			
			dta.moya = {}
			for i = 1, 8
				
				local a = v.RandomRange(0, 359)*ANG1
				local nx = cx + FixedMul(dta.realw/2, cos(a)) -dta.x
				local ny = cy + FixedMul(dta.realh/2, sin(a)) -dta.y
				
				local point = {nx, ny}
				
				dta.moya[i] = {
					coord = point,
					maxscale = FRACUNIT/4,
					scale = 1,
					scalespeed = FRACUNIT/18,
					scaledelay = v.RandomRange(0, 16)
				}
			end
		
		elseif dta.existfor > 1
			
			for i = 1, #dta.moya
				
				local moya = dta.moya[i]
				
				if dta.existfor < moya.scaledelay
					continue
				end
				
				local scale = max(1, min(moya.maxscale, (dta.existfor-moya.scaledelay)*moya.scalespeed) + cos(leveltime*ANG1*8 + i*ANG1*30)/16)
				
				v.drawScaled(dta.x+moya.coord[1], dta.y+moya.coord[2], scale, v.cachePatch("NTHKA0"), flags, v.getColormap(TC_DEFAULT, SKINCOLOR_WHITE))
			end
		end
	end
	
	
	-- draw the box
	local typh = {"TOP", "MID", "BOT"}
	local typw = {"L", "", "R"}
	
	local bx = x
	local by = y
	
	if dta.type == 1	-- shaking
	and dta.existfor < 14
		bx = $ + v.RandomRange(-4, 4)*FRACUNIT
		by = $ + v.RandomRange(-4, 4)*FRACUNIT
	end

	for i = 1, w
		
		by = y
		
		for j = 1, h
			
			local type_w
			if i == 1
				type_w = typw[1]
			elseif i == w
				type_w = typw[3]
			else
				type_w = typw[2]
			end

			local type_h
			if j == 1
				type_h = typh[1]
			elseif j == h
				type_h = typh[3]
			else
				type_h = typh[2]
			end
			
			local pp = "BOX_"..type_h..type_w
			
			v.drawScaled(bx, by, BOXSCALE, v.cachePatch(pp), flags, v.getColormap(TC_DEFAULT, col))
			by = $ + BOXSEG*BOXSCALE
		end
		
		bx = $ + BOXSEG*BOXSCALE
	end
	
	-- Now, draw the text:
	if dta.type == 3	-- DM
		v.drawScaled(x+ (3<<FRACBITS), y+ (1<<FRACBITS), BOXSCALE, v.cachePatch("BOX_DM"), tflag)
	end
	
	v.drawString(x+ ((dta.type == 3 and 10 or 4)<<FRACBITS), y+ (1<<FRACBITS), name, flags|V_ALLOWLOWERCASE, "small-fixed")
	
	if dta.symbol
		PSO.lib.drawSymbolArt(v, x/FRACUNIT +3, y/FRACUNIT +8, dta.symbol)
	else	-- normal text
		v.drawString(x+ (4<<FRACBITS), y+ (BOXSEG*BOXSCALE), "\x8f"..text, flags|V_ALLOWLOWERCASE, "small-fixed")
	end
end

PSO.lib.spawnChatBox = function(source, name, content, typ, sa)
	
	if sa
		local sadta = server.sa_data[sa]
		if not sadta return end	-- ???
		
		if sadta.sound
			S_StartSound(nil, sadta.sound, consoleplayer)
		end
	else
		if typ == 1
			S_StartSound(nil, sfx_toge, consoleplayer)
		elseif typ == 2
			S_StartSound(nil, sfx_moya, consoleplayer)
		elseif typ == 3
			S_StartSound(nil, sfx_chpm, consoleplayer)
		else
			S_StartSound(nil, sfx_chat, consoleplayer)
		end
	end
	
	-- check existing chatboxes if necessary
	if BOX_REPLACE
		for i = 1, #pso_chatboxes
			
			local dta = pso_chatboxes[i]
			if dta and dta.source and dta.source.valid
				if dta.source == source
					
					dta.content = content
					dta.timer = TICRATE*3 + content:len()*2
					dta.resetcontent = true
					dta.type = typ
					dta.existfor = 0
					dta.symbol = sa
					return
				end
			end
		end
	end
	
	local payload = {
		
		source = source,
		name = name,
		content = content,
		symbol = sa,
		timer = TICRATE*3 + content:len()*2,
		color = (source and source.valid) and source.color or SKINCOLOR_RED,
		
		-- used for toge/moya anims
		existfor = 0,
		type = typ,
		
		-- x, y are initialized later on in HUD processing.
		momx = 0,
		momy = 0,
	}
	
	table.insert(pso_chatboxes, payload)
end

local colorcode_convert = {
	[V_MAGENTAMAP]		= "\x81",	
	[V_YELLOWMAP]		= "\x82",
	[V_GREENMAP]		= "\x83",
	[V_BLUEMAP]			= "\x84",
	[V_GREENMAP]		= "\x85",
	[V_GRAYMAP]			= "\x86",
	[V_SKYMAP]			= "\x88",
	[V_PURPLEMAP]		= "\x89",
	[V_AQUAMAP]			= "\x8a",
	[V_PERIDOTMAP]		= "\x8b",
	[V_AZUREMAP]		= "\x8c",
	[V_BROWNMAP]		= "\x8d",
	[V_ROSYMAP]			= "\x8e",
}

freeslot("sfx_chat", "sfx_chpm", "sfx_toge", "sfx_moya")

addHook("PlayerMsg", function(p, typ, targetp, msg)
	
	-- do NTOHING, that aint for you.
	if targetp
	and (p ~= consoleplayer and targetp ~= consoleplayer)
		return true
	end
	
	-- check for dedicated server......
	local firstp
	for p2 in players.iterate do
		firstp = p2
		break
	end
	
	local dedi = firstp ~= server
	
	
	local bubbletype = 0
	local symbol
	--local sound = sfx_chat

	
	-- kill leading & trailing spaces...
	msg = msg:gsub("^%s*(.-)%s*$", "%1")
	
	if targetp
		bubbletype = 3
		--sound = sfx_chpm
	else
		local cmd = msg:sub(1, 6)
		
		if cmd == "/toge "
			bubbletype = 1
			msg = msg:sub(7, #msg)
			--sound = sfx_toge
		elseif cmd == "/moya "
			bubbletype = 2
			msg = msg:sub(7, #msg)
			--sound = sfx_moya
		end
	end
	
	-- kill leading & trailing spaces... (again after command processing...)
	msg = msg:gsub("^%s*(.-)%s*$", "%1")
	
	-- check for symbol arts...
	-- "/symbol"
	if msg:sub(1, 7) == "/symbol"
		local snum = tonumber(msg:sub(8, 8))
		
		if server.sa_data and server.sa_data[snum]
		and server.sa_data[snum].bmp
			PSO.lib.spawnChatBox(p.mo, p.name, "", bubbletype, snum)
			return true
		end
	end
	
	-- also works with /sa...
	if msg:sub(1, 3) == "/sa"
		local snum = tonumber(msg:sub(4, 4))
		
		if server.sa_data and server.sa_data[snum]
		and server.sa_data[snum].bmp
			PSO.lib.spawnChatBox(p.mo, p.name, "", bubbletype, snum)
			return true
		end
	end
	
	-- regular print
	local lead
	local pname
	
	if dedi and p == server
		lead = "\x82"..">>SERVER: ".."\x80"
		pname = "SERVER"
		
	elseif targetp
		if targetp == consoleplayer
			lead = "\x82".."[FROM]<"..p.name.."> "
		else
			lead = "\x82".."[TO]<"..targetp.name.."> "
		end
	
	else
		local chcol = skincolors[p.skincolor].chatcolor
		local chflag = colorcode_convert[chcol] or "\x80"
		
		lead = chflag.."<"..p.name.."> \x80"
	end
	
	if consoleplayer
		chatprintf(consoleplayer, lead..msg, false)
		PSO.lib.spawnChatBox(p.mo, pname or p.name, msg, bubbletype)
	end
	return true
end)

local function boxes_collide(b1, b2)
	
	if b1.x == nil
	or b2.x == nil
		return false
	end
	
	if b1.x + b1.realw > b2.x
	and b1.x < b2.x + b2.realw
	
	and b1.y + b1.realh > b2.y
	and b1.y < b2.y + b2.realh
		return true
	end
	return false
end

local function changed_sign(a, b)
	return (a < 0 and b > 0 or a > 0 and b < 0) and a ~= b
end

-- process chatbox stuff
local prevleveltime
hud.add(function(v, p, c)
	
	--draw_chatbox(v, 100*FRACUNIT, 50*FRACUNIT, 17, 6, "testname", string.rep("a", 32), SKINCOLOR_PINK)
	for i = 1, #pso_chatboxes
		
		local dta = pso_chatboxes[i]
		if not dta continue end
		
		-- process (account for interp, ugh)
		if prevleveltime ~= leveltime
			
			-- init
			if dta.x == nil	-- position not initialized
			or dta.resetcontent
				
				if dta.symbol
					
					dta.w = 12
					dta.h = 7
					
				else
					local s, w, h = chat_prepString(v, dta.content)
					dta.content = s
					dta.w = max(w, (dta.name:len()/2) +1)
					
					if dta.type == 3	-- PM
						dta.w = $+1
					end
					dta.h = h
				end

				dta.realw = dta.w*BOXSEG*BOXSCALE
				dta.realh = dta.h*BOXSEG*BOXSCALE				
				
				-- try to find the source:
				if not dta.resetcontent
					if dta.source and dta.source.valid
					
						local x, y = PSO.lib.HUD_screenCoords(v, p, c, dta.source.x, dta.source.y, dta.source.z+dta.source.height+FRACUNIT*10)	
						x = $ - dta.realw/2
						
						if x > -9
							dta.x = x
							dta.y = y - dta.realh
						end
					end
					
					if dta.x == nil	-- 2nd check in case the first one worked.
						
						-- just spawn randomly
						local ranges = {
							{8, 60},
							{260, (320 - 8) - (dta.realw/FRACUNIT)}
						}
						
						local side = v.RandomRange(1, 2)
						local x = v.RandomRange(ranges[side][1], ranges[side][2])*FRACUNIT
						local y = v.RandomRange(60, (BOX_MAXY/FRACUNIT) - (dta.realh/FRACUNIT))*FRACUNIT
						
						dta.x = x
						dta.y = y
					end
				end
				
				dta.resetcontent = nil
			end

			-- *try* to follow our source
			if dta.source and dta.source.valid
				
				local x, y = PSO.lib.HUD_screenCoords(v, p, c, dta.source.x, dta.source.y, dta.source.z+dta.source.height+FRACUNIT*10)
				x = $ - dta.realw/2
				
				if x > -9
					local tx = x
					local ty = y - dta.realh
					
					dta.momx = (tx-dta.x)/2
					dta.momy = (ty-dta.y)/2
					
					dta.momx = min($, BOX_PUSHSPEED)/2
					dta.momy = min($, BOX_PUSHSPEED)/2
					dta.momx = max($, -BOX_PUSHSPEED)/2
					dta.momy = max($, -BOX_PUSHSPEED)/2
				end
			end		
			
			-- if we're overlapping with another box, push each other away
			for j = 1, #pso_chatboxes
				if i == j continue end	-- no...
				local dta2 = pso_chatboxes[j]
				
				if dta2 and dta2.timer < 10 continue end	-- don't count boxes that are about to die.
				
				if boxes_collide(dta, dta2)
					-- get each box's center and push them away from each other in opposite directions
					local center1x = dta.x + dta.realw/2
					local center1y = dta.y + dta.realh/2

					local center2x = dta2.x + dta2.realw/2
					local center2y = dta2.y + dta2.realh/2
					

					local a = R_PointToAngle2(center1x, center1y, center2x, center2y)
					local spdx = FixedMul(BOX_PUSHSPEED, cos(a))/2
					local spdy = FixedMul(BOX_PUSHSPEED, sin(a))/2
					
					local prevmomx = dta.momx
					local prevmomy = dta.momy
					
					dta.momx = $-spdx
					dta.momy = $-spdy
						
					if changed_sign(dta.momx, prevmomx)
						dta.momx = 0
						dta2.momx = 0
					else
						dta2.momx = -dta.momx
					end
					
					if changed_sign(dta.momy, prevmomy)
						dta.momy = 0
						dta2.momy = 0
					else
						dta2.momy = -dta.momy
					end
				end
			end

			
			-- "physics" processing or some shit
			-- only process if the box isn't fading
			if dta.timer and dta.timer > 10
				
				dta.existfor = $+1
				
				if dta.momx
					dta.x = $ + dta.momx
					dta.momx = $*30/100
				end
				if dta.momy
					dta.y = $ + dta.momy
					dta.momy = $*30/100
				end

				
				-- are you trying to leave the screen?
				if dta.x < BOX_MINX
					dta.x = BOX_MINX
					dta.momx = 0
				end
				
				if dta.x + dta.realw > BOX_MAXX
					dta.x = BOX_MAXX - dta.realw
					dta.momx = 0
				end

				if dta.y < BOX_MINY
					dta.y = BOX_MINY
					dta.momy = 0
				end
				
				if dta.y + dta.realh > BOX_MAXY
					dta.y = BOX_MAXY - dta.realh
					dta.momy = 0
				end	
			end
			
			-- timer stuff
			if dta.timer
				dta.timer = $-1
				
				if not dta.timer
					table.remove(pso_chatboxes, i)
					continue
					
				end
				
			end
			
		end
		
		
		-- drawing
		-- oin this loop we only draw the triangle pointing to the target
		if dta.x ~= nil
		and dta.y ~= nil
		and dta.timer
		and dta.timer > 10
		and p.mo and p.mo.valid
			
			if dta.source and dta.source.valid
				
				if R_PointToDist2(p.mo.x, p.mo.y, dta.source.x, dta.source.y) > BOX_MAXDIST
				or not P_CheckSight(p.mo, dta.source)
					continue
				end
				
				local targetx, targety = PSO.lib.HUD_screenCoords(v, p, c, dta.source.x, dta.source.y, dta.source.z+dta.source.height/2)
				
				if targetx > -9
					
					if dta.type == 2	-- moya
						
						-- for moya, get the angle and draw 3 circles along...
						local centerx = dta.x + dta.realw/2
						local centery = dta.y + dta.realh/2
						
						local ang = R_PointToAngle2(targetx, targety, centerx, centery)
						local dist = R_PointToDist2(targetx, targety, centerx, centery)
						
						for i = 1, 3
							
							local cx = targetx + FixedMul(dist/4 *i, cos(ang))
							local cy = targety + FixedMul(dist/4 *i, sin(ang))
							local tflag = 0
							if dta.timer < 10
								tflag = (10 - dta.timer)<<V_ALPHASHIFT
							end
							
							local tmr = dta.existfor - i*6
							local scale = min(tmr*FRACUNIT/6, FRACUNIT/8 *i) + cos(dta.existfor*ANG1*2)/8
							local scale_corrected = max(0, scale)
							v.drawScaled(cx, cy, scale_corrected, v.cachePatch("NTHKA0"), tflag, v.getColormap(TC_DEFAULT, SKINCOLOR_WHITE))

						end
						
					else	-- normal
						
						local centerx = dta.x + dta.realw/2
						local centery = dta.y + dta.realh/2
						
						local ang = R_PointToAngle2(centerx, centery, targetx, targety)
						local ang_int = PSO.lib.angToInt(ang)
						if ang_int > 85 and ang_int < 95
							ang = 85*ANG1	-- this is a hack but it prevents a weird edge case.
						end
						
						
						local points1 = {}
						for j = 1, 2
							local a = ang + ANG1*90 + (j-1)*ANG1*180
							
							local px = centerx + 8*cos(a)
							local py = centery + 8*sin(a)
							
							points1[#points1+1] = {px, py}
							
							--print(px)
						end
						
						points1[#points1+1] = {targetx, targety}
						
						
						PSO.lib.HUD_drawTriangle(v, points1[1][1], points1[1][2], points1[2][1], points1[2][2], points1[3][1], points1[3][2], 0)
					end
				end
			end
		end
	end
	
	-- now draw the chatboxes
	for i = 1, #pso_chatboxes
		local dta = pso_chatboxes[i]
		draw_chatbox(v, dta)
		--draw_chatbox(v, dta.x, dta.y, dta.w, dta.h, dta.name, dta.content, dta.color, tflag)
	end
	
	prevleveltime = leveltime
end)