-- settings related menus

local function noQuestClose(p)
	if not server.questdata
		PSO.lib.closeMenu(p)
		return true
	end
end

PSO.info.menus["mission"] = {
	
	hudfunc = function(v, p, c, choices, choice, timer)
		drawMainMenu(v, p, c, 4, 999)

		local xpos = 2
		local w = 190
		local y = 20
		local h = 24
		
		v.drawFill(xpos, y, w, h, 253|V_SNAPTOLEFT)
		
		for i = 1, #choices
			
			local tflag = 0
			if i == 1
			and not PSO.lib.questData()
				tflag = V_50TRANS
			elseif i == 2
			and (not PSO.lib.questData() or p != PSO.lib.getLeader())
				tflag = V_50TRANS
			end
			
			v.drawString(xpos + 2, y+4, (i == choice and "\x82"..">" or "")..choices[i][1], V_SNAPTOLEFT|tflag)
			y = $ + 8
		end
	end,
	
	runfunc = function(p)
		local input = PSO.lib.playerInputDown
		
		if input(p, IPT_BACK) == 1
			
			PSO.lib.changeMenu(p, "main", true, true)
			p.pso.menu.timer = TICRATE*10	-- cheat.
			p.pso.menu.choice = 4
			S_StartSound(nil, sfx_mback, p)
		
		elseif input(p, IPT_CONFIRM) == 1
		
			if p.pso.menu.choice == 1
			and not PSO.lib.questData()
				S_StartSound(nil, sfx_mnot, p)
				return true
				
			elseif p.pso.menu.choice == 2
			and (not PSO.lib.questData() or p != PSO.lib.getLeader())
				S_StartSound(nil, sfx_mnot, p)
				return true
			end
		end
	end,
	
	choices = {
		
		{"QUEST INFO", "mission_info"},
		{"ABANDON QUEST (LEADER)", "mission_abort"},
	},
}

PSO.info.menus["mission_abort"] = {
	
	hudfunc = function(v, p, c, choices, choice, timer)
		drawMainMenu(v, p, c, 4, 999)

		local xpos = 2
		local w = 128
		local y = 32
		local h = 28
		
		v.drawFill(160-w/2, 100-h/2, w, h, 253)
		
		for i = 1, #choices
			v.drawString(160, 93 + (i-1)*8, (i == choice and "\x82"..">" or "")..choices[i][1], 0, "center")
			y = $ + 8
		end
	end,
	
	runfunc = function(p)
		local input = PSO.lib.playerInputDown
		
		-- close menu asap if the guy is not leader anymore somehow
		if p != PSO.lib.getLeader()
			PSO.lib.closeMenu(p)
			return true
		end
		
		if noQuestClose(p)
			return true
		end
		
		if input(p, IPT_BACK) == 1
		or input(p, IPT_CONFIRM) == 1
		and p.pso.menu.choice == 1
			
			PSO.lib.changeMenu(p, "main", true, true)
			p.pso.menu.timer = TICRATE*10	-- cheat.
			p.pso.menu.choice = 4
			S_StartSound(nil, sfx_mback, p)
		
		elseif input(p, IPT_CONFIRM) == 1
			PSO.lib.cancelQuest()
			PSO.lib.closeMenu(p)
			
			for p2 in players.iterate
				if p2 and p2.mo.valid
					if netgame
						PSO.lib.playerNotification(p2, "Quest abandonned by leader...")
					else
						PSO.lib.playerNotification(p2, "Quest abandonned...")
					end
				end
			end
		end
	end,
	
	choices = {
		
		{"CANCEL"},
		{"ABANDON QUEST"},
	},
}

PSO.info.menus["mission_info"] = {
	
	hudfunc = function(v, p, c, choices, choice, timer)
		drawMainMenu(v, p, c, 4, 999)
		
		if not server.questdata return end
		
		local xpos = 2
		local w = 316
		local y = 20
		local h = 130
		
		v.drawFill(xpos, y, w, h, 253|V_SNAPTOLEFT)
		
		local sinfo = PSO.lib.questData()
		local minfo = PSO.lib.questInfo()
		local diff = PSO.lib.questDifficulty()
		
		local pic = G_BuildMapName(sinfo.map).."P"
		
		v.drawScaled((xpos+2) *FRACUNIT, (y+2) *FRACUNIT, FRACUNIT/4, v.cachePatch(pic), V_SNAPTOLEFT)
		v.drawString(xpos+48, y+2, "\x82"..minfo.name.." ("..PSO.info.diff_shortnames[diff]..")", V_SNAPTOLEFT)
		v.drawString(xpos+48, y+10, "\x82".."ENEMY LEVEL: ".."\x80"..minfo.levels[diff], V_SNAPTOLEFT)
		
		v.drawFill(xpos+48, y+24, w - 48-4, 1, V_SNAPTOLEFT)
		
		v.drawString(xpos+2, y+40, "\x82".."DESCRIPTION:", V_SNAPTOLEFT)
		local dstr = PSO.lib.wordWrap(v, minfo.desc, w-4)
		v.drawString(xpos+2, y+48, dstr, V_SNAPTOLEFT|V_RETURN8|V_ALLOWLOWERCASE)

		v.drawString(xpos+2, y+90, "\x82".."OBJECTIVE:", V_SNAPTOLEFT)
		local ostr = PSO.lib.wordWrap(v, minfo.objective, w-4)
		v.drawString(xpos+2, y+98, ostr, V_SNAPTOLEFT|V_RETURN8|V_ALLOWLOWERCASE)
		
	end,
	
	runfunc = function(p)
		local input = PSO.lib.playerInputDown
		
		if noQuestClose(p)
			return true
		end
		
		if input(p, IPT_BACK) == 1
			
			PSO.lib.changeMenu(p, "mission", true, true)
			p.pso.menu.timer = TICRATE*10	-- cheat.
			p.pso.menu.choice = 1
			S_StartSound(nil, sfx_mback, p)
		
		end
	end,
	
	choices = {
		
		{"QUEST INFO", "mission_info"},
		{"ABANDON QUEST (LEADER)", "mission_abort"},
	},
}
