-- Type counter: Allows you to change your Type (class)

------------------------------------------------------
-- 	TYPE MAIN
------------------------------------------------------
local choices = {
		"Change Type",
		"Type Skills",
		"Type Proficiency",
		"Leave",
}


local function drawTypeMenu(v, p, c, choice, timer)
	local w = 140
	local h = 60	
	local x = 160-(w/2)
	local y = 100-(h/2)
	
	v.drawFill(x, y, w, h, 253)
	
	v.drawString(160, y+4, "TYPE COUNTER", V_YELLOWMAP, "center")
	
	local cy = y+16
	for i = 1, #choices
		v.drawString(160, cy, (i == choice and "\x82"..">" or "")..choices[i], 0, "center")
		cy = $+10
	end
end

PSO.info.menus["type"] = {
	
	hudfunc = function(v, p, c, choices, choice, timer)
		drawTypeMenu(v, p, c, choice, timer)
	end,
	
	runfunc = function(p)
		local input = PSO.lib.playerInputDown
		
		if input(p, IPT_CONFIRM) == 1
		and p.pso.menu.choice < 4
			
			p.pso.shoptype = p.pso.menu.choice
			-- open the next menu:
			local nextmenu = {"type_change", "type_skill", "type_extend"}
			
			PSO.lib.changeMenu(p, nextmenu[p.pso.menu.choice], true, true)
			
			return true
			
		elseif input(p, IPT_BACK) == 1
		or (input(p, IPT_CONFIRM) and p.pso.menu.choice == 3)
			S_StartSound(nil, sfx_mback, p)
			PSO.lib.closeMenu(p)
		end
	end,
	
	choices = choices,
}

------------------------------------------------------
-- 	TYPE EXTENSION
------------------------------------------------------

local extend_costs = {100, 400, 800, 1500, 3000}
local init_proficiency_mult = {
	[0] = 200,
	[1] = 150,
	[2] = 100,
}

local function drawTypeExtend(v, p, c, choice, timer)
	local w = 160
	local h = 150	
	local x = 4	--160-(w/2)
	local y = 4
	local align = V_SNAPTOTOP|V_SNAPTOLEFT
	
	v.drawFill(x, y, w, h, 253|align)
	
	v.drawString(x+3, y+4, "TYPE PROFICIENCY", V_YELLOWMAP|align)
	
	v.drawString(x+3, y+12, PSO.info.typenames[p.pso.type], V_YELLOWMAP|align)
	v.drawString(x+w-3, y+12, "TP: "..p.pso.typepoints[p.pso.type], V_YELLOWMAP|align, "right")
	
	v.drawFill(x+4, y+22, w-8, 2, 27|align)
	v.drawFill(x+4, y+22, w-7, 1, align)
	
	local cy = y+32
	for i = p.pso.menu.scroll+1, min(#p.pso.menu.choices, p.pso.menu.scroll+6)
		
		local name
		local desc
		local ico
		
		if i == #p.pso.menu.choices
			name = "BACK"
		else
			local wp = p.pso.menu.choices[i]
			name = PSO.info.weaponnames[wp]
			
			ico = "H_WICO"..i
			
			if i == choice
				desc = PSO.lib.wordWrap(v, PSO.info.weaponDesc[wp] or "", w-8, V_ALLOWLOWERCASE, "thin")
				ico = $.."2"
			end

			if desc
				v.drawString(x+4, 102, desc, align|V_ALLOWLOWERCASE, "thin")
			end	
			
			-- get prof level
			local prof = p.pso.weaponproficiencies[p.pso.type][wp]

			-- extend menu
			if p.pso.extendask
			and i == choice
				
				v.drawFill(x+w+2, y, 320 - (x+w+4), 32, 253|align)
				v.drawString(x+w+6, y, "SELECT ACTION", V_YELLOWMAP|align)
				
				local baseprof = PSO.info.typestats[p.pso.type].weapons[wp]
				local profmult = init_proficiency_mult[baseprof] or 100
				--print(prof)
				local acts = {"RANK UP", "RANK DOWN"}
				local tpcost = {(extend_costs[prof+1] or 0)*profmult/100, (extend_costs[prof] or 0)*profmult/100}
				
				for j = 1, #acts
					
					local act = acts[j]
					local cost = tpcost[j]
					local tflag = 0
					
					if cost
						act = $.." ("..(j == 2 and "+" or "-")..cost.."TP)"
					else
						tflag = V_50TRANS
					end
					
					v.drawString(x+w+6, y+12 +10*(j-1), (p.pso.extendask == j and "\x82"..">" or "")..act, align|tflag)
				end
			end
			
			-- draw weapon prof
			local px = x+w - 10
			while prof
				v.draw(px, cy, v.cachePatch("H_STAR"..prof), align)
				px = $-10
				prof = $-1
			end
			
		end
		
		if ico
			v.draw(x+12, cy, v.cachePatch(ico), align)
		end
		
		v.drawString(x+24, cy, (i == choice and "\x82" or "")..name, align)
		cy = $+10
	end

	v.drawFill(x+4, y+92, w-8, 2, 27|align)
	v.drawFill(x+4, y+92, w-7, 1, align)
end

PSO.info.menus["type_extend"] = {
	
	openfunc = function(p)
		local weaponslist = {}
		local num = 0
		
		while PSO.info.weaponnames[2^num]
			weaponslist[#weaponslist+1] = 2^num
			num = $+1
		end
		
		weaponslist[#weaponslist+1] = 0
		
		p.pso.menu.choices = weaponslist
		p.pso.menu.minchoice = 1
		p.pso.menu.maxchoice = #weaponslist
		
	end,
	
	scroll = 6,
	
	hudfunc = function(v, p, c, choices, choice, timer)
		drawTypeExtend(v, p, c, choice, timer)
	end,
	
	runfunc = function(p)
		local input = PSO.lib.playerInputDown
		
		if p.pso.extendask
			
			if input(p, "up") == 1 or input(p, "down") == 1
				p.pso.extendask = (p.pso.extendask == 1 and 2 or 1)
				S_StartSound(nil, sfx_mhvr, p)
			
			elseif input(p, IPT_BACK) == 1
				p.pso.extendask = nil
				S_StartSound(nil, sfx_mback, p)
			
			elseif input(p, IPT_CONFIRM) == 1
				
				local wp = p.pso.menu.choices[p.pso.menu.choice]
				local prof = p.pso.weaponproficiencies[p.pso.type][wp]
				local baseprof = PSO.info.typestats[p.pso.type].weapons[wp]
				local profmult = init_proficiency_mult[baseprof] or 100
				local tpcost = {(extend_costs[prof+1] or 0)*profmult/100, (extend_costs[prof] or 0)*profmult/100}
				
				
				if not tpcost[p.pso.extendask]
					S_StartSound(nil, sfx_mnot, p)
					return true
				end
				
				-- rank UP
				if p.pso.extendask == 1
					-- check points...
					local cost = tpcost[1]
					if p.pso.typepoints[p.pso.type] >= cost --or 1
						p.pso.typepoints[p.pso.type] = $-cost
						p.pso.weaponproficiencies[p.pso.type][wp] = $+1
						
						p.pso.extendask = nil
						S_StartSound(nil, sfx_mconf, p)
					else
						S_StartSound(nil, sfx_mnot, p)
					end
					
				else
					local cost = tpcost[2]
					p.pso.typepoints[p.pso.type] = $+cost
					p.pso.weaponproficiencies[p.pso.type][wp] = $-1

					p.pso.extendask = nil
					S_StartSound(nil, sfx_mconf, p)
					
					-- we have to check the player's palettes and unequip weapons that shouldn't be there anymore
					local pal = p.pso.palette.layout
					local wpal = pal[2]
					
					for i = 1, PSO.info.MAXPALSIZE
						local slot = wpal[i]
						
						if slot
							for j = 1, 2
								local wep = slot[j]
								
								if wep and not PSO.lib.canEquip(p, wep)
									--print("Unequipped "..i..", "..j)
									wep.equipped = nil
									slot[j] = nil
								end
							end
						end
					end
					
					p.pso.forceupdateweapon = true
				end
			end
			
			return true
		end
		
		if input(p, IPT_CONFIRM) == 1
		and p.pso.menu.choice < #p.pso.menu.choices 
				
			p.pso.extendask = 1
			S_StartSound(nil, sfx_mconf, p)

		elseif input(p, IPT_BACK) == 1
		or input(p, IPT_CONFIRM) == 1 and p.pso.menu.choice == #p.pso.menu.choices 
			PSO.lib.changeMenu(p, "type")
			S_StartSound(nil, sfx_mback, p)
			return true
		end
	end,
	
	choices = choices,
}

------------------------------------------------------
-- 	TYPE SKILLS
------------------------------------------------------

local function drawTypeSkills(v, p, c, choice, timer)
	local w = 180
	local h = 140	
	local x = 4	--160-(w/2)
	local y = 4
	local align = V_SNAPTOTOP|V_SNAPTOLEFT
	
	v.drawFill(x, y, w, h, 253|align)
	
	v.drawString(x+3, y+4, "ADD TYPE SKILLS", V_YELLOWMAP|align)
	
	v.drawString(x+3, y+12, PSO.info.typenames[p.pso.type], V_YELLOWMAP|align)
	
	local sx = x+w -10
	local used = PSO.lib.skillCosts(p)
	for i = 1, p.pso.maxskillpoints
		v.draw(sx, y+12, i <= used and v.cachePatch("H_SKOFF") or v.cachePatch("H_SKON"), align)
		sx = $-10
	end
	
	v.drawFill(x+4, y+22, w-8, 2, 27|align)
	v.drawFill(x+4, y+22, w-7, 1, align)
	
	local cy = y+32
	for i = p.pso.menu.scroll+1, min(#p.pso.skillslist, p.pso.menu.scroll+6)
		
		local name
		local desc
		local tflag = 0
		if i == #p.pso.skillslist
			name = "BACK"
		else
			local sk = PSO.info.typeSkills[p.pso.skillslist[i]]
			
			if not sk.level or p.pso.level >= sk.level
				
				name = sk.name
				
				if sk.cost
					local costx = x+w-10
					for j = 1, sk.cost
						v.draw(costx, cy, v.cachePatch("H_SKON"), align)
						costx = $-10				
					end
				end
				
				if i == choice
					desc = PSO.lib.wordWrap(v, sk.desc or "", w-8, V_ALLOWLOWERCASE, "thin")
				end
				
				if PSO.lib.hasSkill(p, p.pso.skillslist[i])
					v.draw(x+4, cy, v.cachePatch("H_EQ"), align)
				end
			else
				name = "??? (LV"..sk.level..")"
				tflag = V_50TRANS
				if i == choice
					local str = "Locked skill. Reach level "..sk.level.." to unlock it."
					desc = PSO.lib.wordWrap(v, str or "", w-8, V_ALLOWLOWERCASE, "thin")
				end
			end	

			if desc
				v.drawString(x+4, 102, desc, align|V_ALLOWLOWERCASE, "thin")
			end
			
		end
		v.drawString(x+20, cy, (i == choice and "\x82"..">" or "")..name, align|tflag)
		cy = $+10
	end

	v.drawFill(x+4, y+92, w-8, 2, 27|align)
	v.drawFill(x+4, y+92, w-7, 1, align)
	
end

PSO.info.menus["type_skill"] = {
	
	openfunc = function(p)
		p.pso.skillconf = nil
		
		-- get a list of all the skills we can use at our level
		p.pso.skillslist = {}
		for i = 1, #PSO.info.typeSkillsList[p.pso.type]
			local sname = PSO.info.typeSkillsList[p.pso.type][i]
			local sdata = PSO.info.typeSkills[sname]
			
			if 1	--not sdata.level or p.pso.level >= sdata.level
				p.pso.skillslist[#p.pso.skillslist+1] = sname
			end
		end
		
		p.pso.skillslist[#p.pso.skillslist+1] = "cancel"
		
		p.pso.menu.minchoice = 1
		p.pso.menu.maxchoice = #p.pso.skillslist
		p.pso.menu.choices = p.pso.skillslist
		
	end,
	
	scroll = 6,
	
	hudfunc = function(v, p, c, choices, choice, timer)
		--drawTypeMenu(v, p, c, choice, timer)
		drawTypeSkills(v, p, c, choice, timer)
	end,
	
	runfunc = function(p)
		local input = PSO.lib.playerInputDown
		
		if input(p, IPT_CONFIRM) == 1
			
			local choice = p.pso.menu.choice
			
			if choice == p.pso.menu.maxchoice
				-- close menu
				PSO.lib.changeMenu(p, "type")
				S_StartSound(nil, sfx_mback, p)
				return
			else
				local sk = p.pso.menu.choices[p.pso.menu.choice]
				local dta = PSO.info.typeSkills[sk]
				
				if dta.level and p.pso.level < dta.level
					S_StartSound(nil, sfx_mnot, p)
					return true
				end
				
				if PSO.lib.hasSkill(p, sk)
					PSO.lib.removeSkill(p, sk)
				else
					if not PSO.lib.addSkill(p, sk)
						S_StartSound(nil, sfx_mnot, p)
						return true
					end
				end
			end
			
			S_StartSound(nil, sfx_mconf, p)
			
			return true
		elseif input(p, IPT_BACK) == 1
			PSO.lib.changeMenu(p, "type")
			S_StartSound(nil, sfx_mback, p)
			return true
		end
	end,
}

------------------------------------------------------
-- 	TYPE CHANGE
------------------------------------------------------


PSO.info.menus["type_change"] = {
	
	openfunc = function(p)
		p.pso.typeyn = nil
	end,
	
	hudfunc = function(v, p, c, choices, choice, timer)
		
		local xpos = 2
		local ypos = 12
		local w = 128
		local h = 60
		local ochoice = choice
		
		v.drawFill(xpos, ypos, w, h, 253|V_SNAPTOLEFT)
		
		local cy = ypos+2+10
		
		if p.pso.typeyn
			v.drawString(xpos+2, ypos+2, "CONFIRM CHOICE?", V_YELLOWMAP|V_SNAPTOLEFT)
			choices = {{"Yes"}, {"No"}}
			ochoice = p.pso.typeyn
		else
			v.drawString(xpos+2, ypos+2, "CHANGE TYPE", V_YELLOWMAP|V_SNAPTOLEFT)
		end
		
		for i = 1, #choices
			
			if i == p.pso.type
				v.draw(xpos+2, cy, v.cachePatch("H_EQ"), V_SNAPTOLEFT)
			end
			
			v.drawString(xpos+12, cy, ((i == ochoice) and "\x82>" or "")..choices[i][1], V_SNAPTOLEFT)
			cy = $+8
		end
		
		if PSO.info.typedescs[choice]
		
			-- description
			local nw = 320 - xpos - w - 2 - 2
			v.drawFill(xpos + w + 2, ypos, nw, h, 253|V_SNAPTOLEFT)
			
			if p.pso.typeyn
				local str = "Changing type will reset the weapon palette and unequip all currently equipped gear"
				local dstr = PSO.lib.wordWrap(v, str, nw-8)
				v.drawString(xpos + w + 4, ypos+2, "\x85".."NOTICE\x80\n"..dstr, V_SNAPTOLEFT|V_ALLOWLOWERCASE|V_RETURN8)
				
			else
				local str = PSO.lib.wordWrap(v, PSO.info.typedescs[choice], nw-8)
				v.drawString(xpos + w + 4, ypos+2, "\x82".."TYPE INFORMATION\x80\n"..str, V_SNAPTOLEFT|V_ALLOWLOWERCASE|V_RETURN8)
			end
			
			-- stats:
			local stats = PSO.lib.playerPreviewStats(p, choice, p.pso.level)
			
			v.drawFill(xpos, ypos + h + 2, w, h, 253|V_SNAPTOLEFT)
			v.drawString(xpos+2, ypos + h + 4, "TYPE STATS", V_YELLOWMAP|V_SNAPTOLEFT)
			local staty = ypos + h + 4 + 8
			
			local statrefs = {"maxhp", "maxpp", "satk", "ratk", "tatk", "def"}
			local statnames = {"HP", "PP", "S.ATK", "R.ATK", "T.ATK", "DEF"}
			
			for i = 1, #statrefs
				
				v.drawString(xpos+43, staty, "\x82"..statnames[i]..":", V_SNAPTOLEFT, "right")
				v.drawString(xpos+47, staty, stats[statrefs[i]], V_SNAPTOLEFT)
				staty = $+8
			end
			
			-- proficiency:
			local chart = p.pso.weaponproficiencies[choice]
			local i = 0
			
			v.drawFill(xpos + w + 2, ypos + h + 2, nw, h, 253|V_SNAPTOLEFT)
			v.drawString(xpos + w + 4, ypos + h + 4, "TYPE WEAPON PROFICIENCY", V_YELLOWMAP|V_SNAPTOLEFT)
			
			local wy = ypos + h + 4 + 8
			while chart[2^i] ~= nil
				local xoffs = (i/5)*48
				local ico = "H_WICO"..PSO.info.weaponnums[2^i]
				v.draw(xpos + w + 4 + 8 + xoffs, wy, v.cachePatch(ico), V_SNAPTOLEFT)
				
				local plevel = "H_STAR"..chart[2^i]
				v.draw(xpos + w + 4 + 18 + xoffs, wy, v.cachePatch(plevel), V_SNAPTOLEFT)
				
				wy = $ + 10
				i = $+1
				
				if i%5 == 0
					wy = ypos + h + 4 + 8
				end
				
			end
		end
	end,
	
	runfunc = function(p)
		local input = PSO.lib.playerInputDown
		
		if p.pso.typeyn
			
			if input(p, "down") == 1
			or input(p, "up") == 1
				p.pso.typeyn = p.pso.typeyn == 1 and 2 or 1
				S_StartSound(nil, sfx_mhvr, p)
			
			elseif input(p, IPT_BACK) == 1
			or input(p, IPT_CONFIRM) == 1 and p.pso.typeyn == 2
				p.pso.typeyn = nil
				S_StartSound(nil, sfx_mback, p)
			
			elseif input(p, IPT_CONFIRM) == 1
			and p.pso.typeyn == 1
				S_StartSound(nil, sfx_mconf, p)

				PSO.lib.playerChangeType(p, p.pso.menu.choice)
				p.pso.typeyn = nil
			end
			
			return true
		end
		
		if input(p, IPT_CONFIRM) == 1
		and p.pso.menu.choice < 4
			
			-- that's already your type dimwit
			if p.pso.menu.choice == p.pso.type
				S_StartSound(nil, sfx_mnot, p)
				return true
			end
			
			S_StartSound(nil, sfx_mconf, p)
			p.pso.typeyn = 1
			return true
			
		elseif input(p, IPT_BACK) == 1
		or (input(p, IPT_CONFIRM) and p.pso.menu.choice == 4)
			S_StartSound(nil, sfx_mback, p)
			PSO.lib.changeMenu(p, "type")
		end
	end,
	
	choices = {
		{"Hunter", TYPE_HUNTER},
		{"Ranger", TYPE_RANGER},
		{"Force", TYPE_FORCE},
		--{"Vanguard", TYPE_VANGUARD},
		{"Back", 0},
	},
}
