-- player menu handling & basic definitions

-- sound freeslots:
freeslot("sfx_mconf", "sfx_mnot", "sfx_mhvr", "sfx_mback")

PSO.info.menus = {}

PSO.lib.setMenuInputs = function(p)
	p.pflags = $|PF_FORCESTRAFE
	p.normalspeed = 0
end

PSO.lib.openMenu = function(p, nmenu, force)
	
	if not force
	and p.pso.menu and p.pso.menu.ref
		return false	-- can't open it.
	end
	
	local menu = PSO.info.menus[nmenu]
	assert(menu, "Invalid menu "..nmenu)
	PSO.lib.dprint("opening menu "..nmenu)
	
	local minchoice, maxchoice
	if menu.minchoice ~= nil
		minchoice = menu.minchoice
	else
		minchoice = 1
	end
	
	if menu.maxchoice ~= nil
		maxchoice = menu.maxchoice
	else
		if menu.choices
			maxchoice = #menu.choices
		else
			maxchoice = minchoice
		end
	end
	
	-- set data...
	p.pso.menu = {
		
		ref = nmenu,
		choice = minchoice,
		scroll = 0,
		minchoice = minchoice,
		maxchoice = maxchoice,
		timer = 0,
		
		anglelock = p.mo.angle,
	}
	
	-- this will be reset by the game when we get out of menus.
	PSO.lib.setMenuInputs(p)
	
	if menu.openfunc
		menu.openfunc(p)
	end
	
	PSO.lib.playerForceInputUp(p, IPT_CONFIRM)
	PSO.lib.playerForceInputUp(p, IPT_BACK)
end	

PSO.lib.closeMenu = function(p, skipmemory)
	
	if not p.pso.menu or not p.pso.menu.ref return end
	local menu = PSO.info.menus[p.pso.menu.ref]

	-- make sure to make these skip
	PSO.lib.playerForceInputUp(p, IPT_CONFIRM)
	PSO.lib.playerForceInputUp(p, IPT_BACK)
	
	if menu.closefunc
		menu.closefunc(p)
	end
	
	if not skipmemory and not menu.nomemory
		p.pso.menumemory[p.pso.menu.ref] = p.pso.menu.choice
	end
	
	p.pflags = $ & ~PF_FORCESTRAFE

	p.pso.menu = {}	
end

-- changes the menu, closefunc and skipmemory may be skipped if so chosen.
PSO.lib.changeMenu = function(p, nmenu, skipclosefunc, skipmemory)
	
	if not p.pso.menu or not p.pso.menu.ref return end
	local menu = PSO.info.menus[p.pso.menu.ref]
	
	if not skipclosefunc
		PSO.lib.closeMenu(p, skipmemory)
	elseif not skipmemory and not menu.nomemory
		p.pso.menumemory[p.pso.menu.ref] = p.pso.menu.choice
	end
	
	-- force open
	PSO.lib.openMenu(p, nmenu, true)
end

local function menuHandler(p)
	
	if not p.pso.menu return end	-- what
	
	-- open main menu...
	if not p.pso.menu.ref
	and PSO.lib.playerInputDown(p, IPT_MENU)
		S_StartSound(nil, sfx_mopen, p)
		PSO.lib.openMenu(p, "main")
		--PSO.lib.openMenu(p, "type_set")
	end
	
	if p.pso.menu.ref
		
		local menu = p.pso.menu
		local info = PSO.info.menus[menu.ref]
		
		menu.timer = $+1
		
		-- lock player angle:
		p.cmd.angleturn = p.pso.menu.anglelock/FRACUNIT
		p.drawangle = p.pso.menu.anglelock
		
		-- control func, overwrite default behaviour if we return true
		if info.runfunc
			if info.runfunc(p, info.choices, menu.choice, menu.timer)
				return
			end
		end
		
		local popt = info.inputtype and "left" or "up"
		local nopt = info.inputtype and "right" or "down"
		
		-- for now
		local minchoice = p.pso.menu.minchoice or 1
		local maxchoice = p.pso.menu.maxchoice or 1
		local scrollstart = 0
		local maxscroll = 0
		local scrollback = 0
		
		-- build scroll info if we need to.
		if info.scroll and (maxchoice - (minchoice-1)) > info.scroll
			scrollstart = minchoice + info.scroll/2
			scrollback = maxchoice - info.scroll/2
			maxscroll = (maxchoice - (minchoice-1)) - info.scroll
		end
		
		if PSO.lib.playerInputDown(p, popt) == 1
		and maxchoice ~= minchoice
			menu.choice = $-1

			if menu.choice < scrollback
			and (menu.choice > scrollstart or menu.scroll)
				menu.scroll = max(0, $-1)
			end
		
			if menu.choice < minchoice
				menu.choice = maxchoice
				menu.scroll = maxscroll
			end
			S_StartSound(nil, sfx_mhvr, p)
		elseif PSO.lib.playerInputDown(p, nopt) == 1
		and maxchoice ~= minchoice
			menu.choice = $+1
			
			if menu.choice > scrollstart
				menu.scroll = min(maxscroll, $+1)
			end
			
			if menu.choice > maxchoice
				menu.choice = minchoice
				menu.scroll = 0
			end
			S_StartSound(nil, sfx_mhvr, p)
		elseif PSO.lib.playerInputDown(p, IPT_CONFIRM) == 1
		and info.choices
			local curr = info.choices[menu.choice]
			if curr and curr[2]
				PSO.lib.changeMenu(p, curr[2])
				S_StartSound(nil, sfx_mconf, p)
			end
		end
	end
end

PSO.lib.registerPlayerFunc(menuHandler)


local function menuDrawer(v, p, c)
	
	if not p.pso.menu return end
	
	local menu = PSO.info.menus[p.pso.menu.ref]
	
	if menu and menu.hudfunc
		menu.hudfunc(v, p, c, menu.choices, p.pso.menu.choice, p.pso.menu.timer)
	end
end

PSO.lib.registerHudFunc(menuDrawer)