local exportmode = false

PSO.info.questmaps = {}
-- list of maps that can be selected for quests

local questgrading = {
	"QG_TIME",	-- grade by time
	"QG_KILLS",	-- grade # of kills (of a specific enemy?)
	"QG_DAMAGETAKEN",	-- damage taken
	"QG_NOINCAP",		-- extra points for no incapacitation
	"QG_QUESTPOINTS",	-- TBI
}

for i = 1, #questgrading
	rawset(_G, questgrading[i], i)
end

PSO.info.quests = {}
/*
	quest format:
	{
		>>COSMETIC STUFF
		name: 		quest name
		objective:	description of the objective
		desc: 		description
	
	
		>>ACTUAL GAMEPLAY RELEVANT STUFF
		map:		map number the quest takes place in
					(players will be warped to it as soon as everyone gets to the teleporter)
		
		layout = {}: 	object layout for the map.
						worth noting the object layout CANNOT be changed in the lobby.
		
		levels = {
			[DIFF_NORMAL] = x,
			[DIFF_HARD] = x,
			[...]
		}: enemy level per difficulty
		
		drops = {
			[DIFF_NORMAL] = {
			
			},
			
			[DIFF_HARD] = {
			
			},
			(...)
			
			special drop tables per difficulty (used for crates at the end of the mission)
		},
		
		
		grading = {
			{QG_TIME, full_point_time, 0_point_time, full_time, low_time},
			{QG_KILLS, max_points, max_kills, [MT_ type]},
			{QG_NOINCAP, points_for_no_incapacitation},
			{QG_DAMAGETAKEN, full_point_damagetaken, 0_point_damagetaken, low_damage_taken, high_damage_taken},
		}
		
		-- Grading will total all the points and then award them as Type points (with a multiplier depending on difficulty)
		-- Grading works like so based on a percentage:
		
		-- 0% - 49%: C Rank
		-- 50% - 79%: B Rank
		-- 80% - 99%: A Rank
		-- 100% S Rank
	}
*/


-- exports the object layout in the console so that it can be used for quests

PSO.lib.questExport = function()
	
	local fstr = "{\n"
	
	for mt in mapthings.iterate do
		if mt.y > PSO.info.LOBBY_YPOS
			local mtype = 0
			
			local mnum = mt.type
			if mt.mobj
				mtype = mt.mobj.type
				
				if PSO.info.mobjNameLookup[mtype]
					mtype = PSO.info.mobjNameLookup[mtype]	-- get mobj name.
				end
			end
			
			fstr = $.."{"..mtype..", "..mt.x..", "..mt.y..", "..mt.angle..", "..mt.options..", "..mt.z..", "..mt.extrainfo..", "..mnum.."},\n"
		end
	end
	fstr = $.."}"
	
	print(fstr)
end

COM_AddCommand("qdump", function(p)
	if netgame return end	-- DO NOT.
	exportmode = (exportmode == false and true or false)
end)


-- accepts a quest and executes linedef 3000
-- opens the door to the teleporter and sets the exit params
PSO.lib.acceptQuest = function(map, qnum, difficulty)
	
	assert(gamemap == 1, "Attempting to accept a quest outside the Low Orbit Station?")
	
	server.questdata = {
		
		map = map,
		index = qnum,
		difficulty = difficulty
	}
		
	S_StartSound(nil, sfx_qacpt)
	G_SetCustomExitVars(map, 1)
	P_LinedefExecute(3000)
	
	for p in players.iterate
		if p.pso
			p.pso.ironwill = nil	-- not used
		end	
	end
end

PSO.lib.questData = function()
	if not server.questdata return nil end
	
	return server.questdata
end

PSO.lib.questInfo = function()
	local dta = PSO.lib.questData()
	if not dta return end
	
	
	return PSO.info.quests[dta.map][dta.index]
end

PSO.lib.cancelQuest = function()
	server.questdata = nil
	G_SetCustomExitVars(1, 1)
	G_ExitLevel()
end

PSO.lib.goToQuest = function()
	assert(gamemap == 1, "Attempting to get to a quest from outside the Low Orbit Station?")
	assert(server.questdata, "No quest data")
	
	G_SetCustomExitVars(1, 1)
end

local function hardModeTextureSwaps()
	if PSO.lib.questDifficulty() >= DIFF_HARD
		
		local header = mapheaderinfo[gamemap]
		if header.hardsky
			P_SetupLevelSky(header.hardsky)
		end
		
		if header.hardweather
			P_SwitchWeather(_G[header.hardweather] or 0)
		end
		
		for s in sectors.iterate
			
			if s.tag == PSO.info.ENCORETAG
				--print("Found ref")
				for s2 in sectors.iterate
					
					if s2 ~= s1
					and s2.lines[0].v1.y > PSO.info.LOBBY_YPOS*FRACUNIT	-- don't affect lobby
						if s2.floorpic == s.floorpic
							s2.floorpic = s.ceilingpic
						end
						if s2.ceilingpic == s.floorpic
							s2.ceilingpic = s.ceilingpic
						end
					end
				end
			end
			
			if header.hardbrightness
			and s.lines[0].v1.y > PSO.info.LOBBY_YPOS*FRACUNIT
				s.lightlevel = max(1, $+header.hardbrightness)
			end
		end
		
		for l in lines.iterate
			if l.tag == PSO.info.ENCORETAG
				local s = l.frontside
				for s2 in sides.iterate
					
					if s2.line ~= s
					and s2.v1.y > PSO.info.LOBBY_YPOS*FRACUNIT	-- don't affect lobby
						local checks = {"toptexture", "midtexture", "bottomtexture"}
						for i = 1, #checks
							
							local c = checks[i]
							if s2[c] == s.toptexture
								s2[c] = s.midtexture
							end
						end
					end
				end
			end
		end
	end
end

-- call before processing objects (i.e: rooms)
PSO.lib.spawnQuestObjects = function(tp_only)
	
	if not server.questdata return end
	if gamemap ~= server.questdata.map return end
	
	local q = PSO.info.quests[server.questdata.map][server.questdata.index]
	local objs = q.layout
	
	local lastspawnpoint
	local playersspawned = {}
	
	for i = 1, #objs
		-- format: {type, x, y, options, z, extrainfo}
		local o = objs[i]
		local mobjtype = o[1]
		local x = o[2]*FRACUNIT
		local y = o[3]*FRACUNIT
		local ang = o[4]
		local options = o[5]
		local z = o[6]*FRACUNIT
		local extrainfo = o[7]
		local mapthingnum = o[8]
		
		local sec = R_PointInSubsector(x, y).sector
			-- mapthing z is distance from floor and not absolute z height (or ceiling in antigrav)
		if options & 2
			z = sec.ceilingheight - z
		else
			z = $ + sec.floorheight
		end
		
		if mapthingnum <= 32	-- player start 1-32
			local p = players[mobjtype]
			
			if p and p.valid and p.mo and p.mo.valid
				P_MoveOrigin(p.mo, x, y, z)
				p.mo.angle = ang*ANG1
				if options & 2
					p.mo.flags2 = $|MF2_OBJECTFLIP
				end
			end
			playersspawned[mobjtype] = $ and $+1 or 1
			lastspawnpoint = o
			
		elseif not tp_only	-- regular object
		
			local mt = P_SpawnMobj(x, y, z, mobjtype)
			mt.options = options
			mt.extrainfo = extrainfo
			mt.mtangle = ang
			mt.angle = ang*ANG1
			if options & 2
				mt.flags2 = $|MF2_OBJECTFLIP
			end			
		end
	end
	
	-- check for players that haven't been spawned
	if lastspawnpoint
		server.lastspawnpoint = {lastspawnpoint[2]*FRACUNIT, lastspawnpoint[3]*FRACUNIT, lastspawnpoint[6]*FRACUNIT, lastspawnpoint[4]*ANG1, lastspawnpoint[5]}
		for p in players.iterate
			
			if not playersspawned[#p] and p.mo and p.mo.valid
				
				local mobjtype = lastspawnpoint[1]
				local x = lastspawnpoint[2]*FRACUNIT
				local y = lastspawnpoint[3]*FRACUNIT
				local ang = lastspawnpoint[4]*ANG1
				local options = lastspawnpoint[5]
				local z = lastspawnpoint[6]*FRACUNIT
				
				local numspawns = playersspawned[mobjtype]
				
				-- make players spawn at different positions i guess
				local angs = {ang*ANG1 - ANG1*180, ang*ANG1 - ANG1*90, ang*ANG1 + ANG1*90}
				local shift_ang = angs[ ((numspawns-1) %3) +1 ] or 0
				local shift_dist = 32 + (numspawns/3)*32
				
				local sx = x + shift_dist*cos(shift_ang)
				local sy = y + shift_dist*sin(shift_ang)
				
				P_MoveOrigin(p.mo, sx, sy, z)
				p.mo.angle = ang*ANG1
				if options & 2
					p.mo.flags2 = $|MF2_OBJECTFLIP
				end
				
				playersspawned[mobjtype] = $ and $+1 or 1
				--lastspawnpoint = o
				
			end
		end
	end
end

addHook("PreThinkFrame", function()
	
	/*if leveltime == 0
	and not server.questspawned
		PSO.lib.spawnQuestObjects(true)	-- setup player spawns
		server.questspawned = true
	end*/
	
	if leveltime == 0
	and server
	and not server.questspawned
		
		if exportmode
		and not netgame	-- BY NO MEANS
			PSO.lib.dprint("Exporting quest objects...")
			PSO.lib.questExport()
		end
		
		hardModeTextureSwaps()
		
		PSO.lib.dprint("Spawn quest objects...")
		PSO.lib.spawnQuestObjects()
		
		server.questspawned = true
	end
	
	if server		
		
		server.psounlocks = $ or {}
		
		if server.questdata and server.questdata.failed
			server.questdata.failed = $+1
			
			if server.questdata.failed == 2
				S_FadeOutStopMusic(500)
			end
			
			if server.questdata.failed == TICRATE
				S_StartSound(nil, sfx_qfail)
			end
			
			if server.questdata.failed == TICRATE*5
				
				G_SetCustomExitVars(1, 1)
				G_ExitLevel()

				for p in players.iterate
					if p and p.mo and p.mo.valid
						PSO.lib.healObject(p.mo, p.mo, p.pso.maxhp)
						p.pso.deadtimer = 0
					end
				end
				
			end
		end
	end
end)

-- unset variable on map change
addHook("MapChange", function()
	server.questspawned = nil
end)

-- in map 1, remove server quest data.
addHook("MapLoad", do
	if gamemap == 1
		server.questdata = nil
	end
	
	if server and server.unlockmsg
		
		for p in players.iterate
			PSO.lib.playerNotification(p, "\x82".."UNLOCKED: ".."\x80"..server.unlockmsg)
		end
		server.unlockmsg = nil
	end
	
end)

PSO.lib.questDifficulty = function()
	if not server.questdata
		return DIFF_NORMAL
	else
		return server.questdata.difficulty
	end
end

-- table that keeps track of unlocked maps & quests
PSO.lib.unlockQuest = function(map, quest, diff)
	if not server.psounlocks[map]
		server.psounlocks[map] = {}
	end
	
	-- SPECIFICALLY...
	if map == 2
	and quest == 1
	and diff == DIFF_HARD
		server.unlockmsg = "1-1 Hard Mode"
	end
	
	if not server.unlockmsg
		server.unlockmsg = PSO.info.quests[map][quest].name.." ("..PSO.info.diff_shortnames[diff]..")"
	end
	
	server.psounlocks[map][quest] = diff or DIFF_NORMAL
	
	PSO.lib.server_saveData()
end

PSO.lib.questUnlocked = function(map, quest)
	if not server.psounlocks[map] return false end
	
	if quest == nil
		return server.psounlocks[map] != nil
	end
	
	return server.psounlocks[map][quest]
end

-- a bit of a hack but because new difficulties are first unlocked on 1-1...
PSO.lib.maxDifficultyUnlocked = function()
	return PSO.lib.questUnlocked(2, 1) or DIFF_NORMAL
end

COM_AddCommand("psod_unlock", function(p, map, quest, diff)

	if not server.psocheats
		print("Enable cheats with 'psod_debugcheats' to use this command")
		return 
	end
	
	PSO.lib.unlockQuest(tonumber(map), tonumber(quest), tonumber(diff) or DIFF_NORMAL)
	
end, COM_ADMIN)